C
C       ======================================================================
C      =========================================================================
C
C
C     DDDDDDD   IIII   CCCCCC   VV    VV   OOOOOO   LL         999999        11
C     DD    DD   II   CC    CC  VV    VV  OO    OO  LL        99    99     1111
C     DD    DD   II   CC        VV    VV  OO    OO  LL        99    99   111 11
C     DD    DD   II   CC        VV    VV  OO    OO  LL         9999999       11
C     DD    DD   II   CC         VV  VV   OO    OO  LL              99       11
C     DD    DD   II   CC    CC    VVVV    OO    OO  LL    LL  99    99       11
C     DDDDDDD   IIII   CCCCCC      VV      OOOOOO   LLLLLLLL   999999       1111
C
C     ==========================================================================
C       ======================================================================
C
C     DICVOL91-VERSION 1.0  : 23/04/1992
C
C     modif. A. Le Bail  for Alpha AXP et ZERO (a additionner)
C
C
C     TRIAL AND ERROR METHOD FOR THE AUTOMATIC INDEXING OF POWDER DIFFRACTION
C     PATTERNS - VARIATION OF PARAMETERS BY SUCCESSIVE DICHOTOMY.
C     THIS VERSION INCLUDES PARTITION OF VOLUME SPACE.
C     THE FOLLOWING CRYSTALLINE SYSTEMS CAN BE EXAMINED :
C     CUBIC, TETRAGONAL,HEXAGONAL, ORTHORHOMBIC, MONOCLINIC
C     AND TRICLINIC.
C
C     PRECURSOR PROGRAMS :   P1 AND P2 (REF. 1), DICVOL (REF. 2)
C
C     DICVOL91 HAS BEEN WRITTEN BY D. LOUER AND A. BOULTIF (REF. 3)
C
C     FOR FURTHER INFORMATION CONTACT
C               D. LOUER
C               LABORATOIRE DE CRISTALLOCHIMIE
C               UNIVERSITE DE RENNES I
C               CAMPUS DE BEAULIEU
C               35042 RENNES CEDEX    -FRANCE-
C               TEL.   (33) 99-28-62-48
C               FAX    (33) 99-38-34-87
C               E-MAIL  Louer@cicb.fr
C
C      ---------------------------------------------------------------
C      |            G E N E R A L   I N S T R U C T I O N S          |
C      ---------------------------------------------------------------
C
C      DICVOL PROPOSES SOLUTIONS; THE USER DISPOSES OF THEM.
C      THE AUTHORS DISCLAIM RESPONSABILITY FOR ALL USE OF THIS
C      PROGRAM, WHETHER GOOD, BAD OR INDIFFERENT.
C
C      DATA CARDS
C      ----------
C
C  CARD 1  TITLE                                       FORMAT(16A4)
C
C
C  CARD 2  N,ITYPE,JC,JT,JH,JO,JM,JTR                  FREE FORMAT
C
C          N               NUMBER OF LINES USED.
C          ITYPE           SPACING DATA TYPE.
C                      =1  THETA BRAGG IN DEGREES.
C                      =2  2-THETA ANGLE IN DEGREES.
C                      =3  D-SPACING IN ANGSTROMS.
C                      =4  Q SPECIFIED IN Q-UNITS AS E+04/D**2.
C          JC          =0  CUBIC SYSTEM IS NOT TESTED.
C                      =1  CUBIC SYSTEM IS TESTED.
C          JT          =0  TETRAGONAL SYSTEM IS NOT TESTED.
C                      =1  TETRAGONAL SYSTEM IS TESTED.
C          JH          =0  HEXAGONAL SYSTEM IS NOT TESTED.
C                      =1  HEXAGONAL SYSTEM IS TESTED.
C          JO          =0  ORTHORHOMBIC SYSTEM IS NOT TESTED.
C                      =1  ORTHORHOMBIC SYSTEM IS TESTED.
C          JM          =0  MONOCLINIC SYSTEM IS NOT TESTED.
C                      =1  MONOCLINIC SYSTEM IS TESTED.
C          JTR         =0  TRICLINIC SYSTEM IS NOT TESTED.
C                      =1  TRICLINIC SYSTEM IS TESTED.
C
C
C  CARD 3  AMAX,BMAX,CMAX,VOLMIN,VOLMAX,BEMIN,BEMAX    FREE FORMAT
C
C          AMAX    MAXIMUM VALUE OF UNIT CELL DIMENSION A IN ANGSTROMS.
C                  (IF AMAX= 0.0 DEFAULT= 20. ANGSTROMS)
C          BMAX    MAXIMUM VALUE OF UNIT CELL DIMENSION B IN ANGSTROMS.
C                  (IF BMAX= 0.0 DEFAULT= 20. ANGSTROMS)
C          CMAX    MAXIMUM VALUE OF UNIT CELL DIMENSION C IN ANGSTROMS.
C                  (IF CMAX= 0.0 DEFAULT= 20. ANGSTROMS)
C          VOLMIN  MINIMUM VOLUME FOR TRIAL UNIT CELLS IN ANGSTROMS**3.
C          VOLMAX  MAXIMUM VOLUME FOR TRIAL UNIT CELLS IN ANGSTROMS**3.
C                  (IF VOLMAX= 0.0 DEFAULT= 1500. ANGSTROMS**3)
C          BEMIN   MINIMUM ANGLE FOR UNIT CELL IN DEGREES
C                  (IF BEMIN= 0.0 DEFAULT= 90. DEGREES).
C          BEMAX   MAXIMUM ANGLE FOR UNIT CELL IN DEGREES
C                  (IF BEMAX= 0.0 DEFAULT= 125. DEGREES).
C
C
C  CARD 4  WAVE,POIMOL,DENS,DELDEN                     FREE FORMAT
C
C          WAVE    WAVELENGTH IN ANGSTROMS (DEFAULT=0.0 IF CU K ALPHA1).
C          POIMOL  MOLECULAR WEIGHT OF ONE FORMULA UNIT IN A.M.U.
C                  (DEFAULT =0.0 IF FORMULA WEIGHT NOT KNOWN).
C          DENS    MEASURED DENSITY IN G.CM(-3)
C                  (DEFAULT =0.0 IF DENSITY NOT KNOWN).
C          DELDEN  ABSOLUTE ERROR IN MEASURED DENSITY.
C
C
C  CARD 5  EPS,FOM,ZERO                                 FREE FORMAT
C
C          EPS       =0.0  THE ABSOLUTE ERROR ON EACH OBSERVED LINE
C                          IS TAKEN TO .03 DEG. 2THETA, WHATEVER THE
C                          SPACING DATA TYPE (ITYPE IN CARD 2).
C                    =1.0  THE ABSOLUTE ERROR ON EACH OBSERVED LINE IS
C                          INPUT INDIVIDUALLY IN THE FOLLOWING CARDS,
C                          TOGETHER WITH THE OBSERVED 'D(I)', ACCORDING
C                          WITH THE SPACING DATA UNIT.
C                    EPS NE 0.0 AND 1.0
C                          THE ABSOLUTE ERROR IS TAKEN AS A CONSTANT
C                          (=EPS),IN DEG. 2THETA, WHATEVER THE SPACING
C                          DATA TYPE (ITYPE IN CARD 2).
C          FOM             LOWER FIGURE OF MERIT M(N) REQUIRED FOR PRINTED
C                          SOLUTION(S) (DEFAULT=0.0 IF LOWER M(N)=5.0).
C
C          ZERO            Zeropoint to be added
C                          To be used only with 2-theta or theta angles
C
C
C
C  CARD 6 TO 6+N  D(I),EPSIL(I)                        FREE FORMAT
C
C          (ONE FOR EACH OBSERVED LINE, UP TO N)
C          D(I)    VALUE DESCRIBING THE OBSERVED POSITION
C                  OF THIS LINE ACCORDING TO 'ITYPE'.
C          EPSIL   ABSOLUTE ERROR IN 'D(I)', ACCORDING TO 'ITYPE',
C                  ONLY IF EPS=1.0 (CARD 5).
C  NOTE:
C          IF ITYPE=1,2,4 THE VALUES OF 'D(I)' AND 'EPSIL(I)' MUST
C          BE LISTED IN INCREASING ORDER.
C          IF ITYPE=3 THEY MUST BE IN DECREASING ORDER.
C
C  REFERENCES:
C     1.-  LOUER, D. AND LOUER, M., METHODE D'ESSAIS ET ERREURS POUR
C          L'INDEXATION AUTOMATIQUE DES DIAGRAMMES DE POUDRE,
C          J. APPL. CRYST. 5, 271-275 (1972).
C     2.-  LOUER, D. AND VARGAS, R., INDEXATION AUTOMATIQUE DES
C          DIAGRAMMES DE POUDRE PAR DICHOTOMIES SUCCESSIVES,
C          J. APPL. CRYST. 15, 542-545 (1982).
C     3.-  BOULTIF, A. AND LOUER, D., INDEXING OF POWDER DIFFRACTION
C          PATTERNS FOR LOW SYMMETRY LATTICES BY THE SUCCESSIVE
C          DICHOTOMY METHOD, J. APPL. CRYST. 24, 987-993 (1991).
C     4.-  DE WOLFF, P.M., A SIMPLIFIED CRITERION FOR THE RELIABILITY
C          OF A POWDER PATTERN INDEXING, J. APPL. CRYST. 5, 108-113 (1968).
C     5.-  SMITH, G. S. AND SNYDER, R. L., F(N): A CRITERION FOR RATING
C          POWDER DIFFRACTION PATTERNS AND EVALUATING THE RELIABILITY 
C          OF POWDER-PATTERN INDEXING, J. APPL. CRYST. 12, 60-65 (1979).
C     6.-  MIGHELL, A.D., HUBBARD, C.R. AND STALIK, J.K., NBS*AIDS80:
C          A FORTRAN PROGRAM FOR CRYSTALLOGRAPHIC DATA EVALUATION.
C          NAT. BUR. STAND. (U.S.) TECH. NOTE 1141 (1981). (NBS*AIDS83
C          IS AN EXPANDED VERSION OF NBS*AIDS80).
C     7.-  LOUER, D., AUTOMATIC INDEXING: PROCEDURES AND APPLICATIONS. IN
C          'ACCURACY IN POWDER DIFFRACTION II', NIST, SPEC. PUBL. No. 846,
C          GAITHERSBURG, MD, USA, pp. 92-104, 1992.
C        *****************************************************
C        *                                                   *
C        *         THE USE OF DICVOL91 (VERSION 1.0)         *
C        *                                                   *
C        *****************************************************
C
C      USE AS INPUT DATA THE FIRST 20 LINES (ALTHOUGH THIS NUMBER
C      CAN BE DIFFERENT). AVOID SPURIOUS LINES OR TOO INACCURATE DATA.
C      ALL SYMMETRIES CAN BE EXAMINED IN ONE RUN. HOWEVER, IT IS
C      RECOMMENDED TO USE A TWO- OR THREE-STAGE PROCEDURE :
C        1- SEARCH IN HIGH SYMMETRIES DOWN TO ORTHORHOMBIC.
C           CARD 2 : N,ITYPE,1,1,1,1,0,0
C        2- SEARCH IN MONOCLINIC SYMMETRY.
C           CARD 2 : N,ITYPE,0,0,0,0,1,0
C        3- IF NECESSARY, SEARCH IN TRICLINIC SYMMETRY.
C           CARD 2 : N,ITYPE,0,0,0,0,0,1
C
C      NOTE THAT FOR SOLUTIONS WITH TRICLINIC SYMMETRY THE USE OF A
C      REDUCTION CELL PROGRAM IS USUALLY USEFULL TO OBTAIN THE
C      CONVENTIONAL UNIT CELL (E.G. CDF-SRCH/JCPDS).
C
C      NOTE - CASE OF TRIGONAL SYMMETRY WITH RHOMBOEDRAL LATTICE:
C            THE PATTERN IS INDEXED WITH AN HEXAGONAL LATTICE, HAVING
C            A UNIT CELL VOLUME THREE TIMES GREATER. OWING TO THE
C    	     STRATEGY BASED ON THE PARTITION OF VOLUME SPACE (SHELLS OF
C            400 A**3) YOU CAN MISS THE SOLUTION IF, BY ACCIDENT, A
C	     SMALLER (PSEUDO) SOLUTION IS FOUND IN AN OTHER SYMMETRY.
C            IF YOU SUSPECT SUCH A CASE, RUN DICVOL91 WITH THE HEXAGONAL
C	     CASE ONLY (JC=0, JT=0, JH=1, JO=0, JM=0, JTR=0).
C	        
C  RECOMMENDATIONS
C   ---------------
C   1- PLEASE SPEND TIME TO ENSURE THE QUALITY OF YOUR OBSERVED DATA.
C      WITH ACCURATE DATA THE SUCCESS RATE OF DICVOL91 IS VERY HIGH
C      (SEE REF. 3).
C      WITH BAD DATA THE CHANCE TO OBTAIN THE CORRECT SOLUTION IS VERY
C      SMALL AND THE CALCULATION CAN BE TIME-CONSUMMING.
C
C      WITH MODERN X-RAY POWDER DIFFRACTOMETERS (THE USE OF MONOCHROMATIC
C      RADIATION IS STRONGLY RECOMMENDED) ABSOLUTE ERRORS ON PEAK
C      POSITIONS LOWER THAN 0.03 DEGREES 2-THETA CAN BE ROUTINELY OBTAINED.
C      FOR INDEXING PURPOSES, ERRORS SHOULD NOT EXCEED 0.03 DEG. 2-THETA
C      [IN EXCEPTIONAL CASES, A FEW LINES WITH GREATER ERROR CAN BE
C      INTRODUCED IN INPUT DATA. IN THIS CASE, USE THE PARAMETER EPS= 1.
C      (CARD 5), AND ENTER INDIVIDUAL ERRORS ON EACH LINE(CARD 6 TO 6+N)].
C
C      WITH HIGH RESOLUTION POWDER DIFFRACTION DATA (CONVENTIONAL OR,
C      PARTICULARLY, SYNCHROTRON X-RAY SOURCES) THE ABSOLUTE ERROR IS
C      USUALLY LESS THAN 0.02 (OR EVEN 0.01) DEG. 2-THETA; CONSEQUENTLY,
C      EPS=0.02 (OR EVEN EPS=0.01) IS THEN RECOMMENDED; THE CONVERGENCE
C      OF THE DICHOTOMY PROCEDURE WILL BE IMPROVED. HOWEVER, BE SURE THAT
C      THIS CONDITION IS TRUE FOR ALL THE LINES USED AS INPUT DATA.
C
C   2- IF DICVOL91 HAS FOUND A SOLUTION FROM THE FIRST 20 (OR N) LINES,
C      THIS SOLUTION MUST INDEXED THE COMPLETE POWDER DIFFRACTION
C      PATTERN FOR THIS PURPOSE, THE PROGRAM NBS*AIDS83 (REF. 6) CAN BE
C      USED.
C
C  NOTE : FUNCTION TIME
C         +++++++++++++
C
C       1 - THIS PROGRAM USES THE FUNCTION 'SECOND()' OF THE LIBRARY
C           FORTRAN UTILITY SUBPROGRAMS CDC - SYSTEM NOS/VE.
C       2 - THIS FUNCTION CAN BE EASILY REMOVED BY ADDING A 'C' IN THE
C           FIRST COLUNM OF THE LINES CONTAINING THE VARIABLE 'TIME'.
C       3 - IF YOU WANT TO ADAPT IT TO A microVAX COMPUTER:
C              * ADD AT THE OUSET OF THE PROGRAM
C                    T1 = SECNDS(0.0)
C                    TIME2 = 3600.*24.
C              * SUBSTITUTE IN THE PROGRAM ALL INSTRUCTIONS
C                    TIME = SECOND()
C                BY
C                    TIME = SECNDS(T1)
C                    IF(TIME.LT.0.)TIME = TIME + TIME2
C
C           ----------          ------------------          ----------
C           |        |=========>|     CUBIC      |=========>|        |
C           |        |          ------------------          |        |
C           |        |                   V                  |        |
C           |        |              ----------              |        |
C           |        |              | CUDHKL |              |        |
C           |        |              ----------              |    A   |
C           |        |                                      |    F   |
C           |        |                                      |    F   |
C           |        |                                      |    I   |
C           |        |          ------------------          |    N   |
C           |        |=========>|   TETRAGONAL   |=========>|    E   |
C           |    P   |          ------------------          |    M   |
C           |    R   |                   V                  |    E   |
C           |    O   |              ----------              |    N   |
C           |    G   |              | TEDHKL |              |    T   |
C           |    R   |              ----------              |        |
C           |    A   |                                      |        |
C           |    M   |                                      |    D   |
C           |    M   |                                      |    E   |
C           |    E   |          ------------------          |    S   |
C           |        |=========>|    HEXAGONAL   |=========>|        |
C           |        |          ------------------          |        |
C           |    P   |                   V                  |    P   |
C           |    R   |              ----------              |    A   |
C           |    I   |              | TEDHKL |              |    R   |
C           |    N   |              ----------              |    A   |
C           |    C   |                                      |    M   |
C           |    I   |                                      |    E   |
C           |    P   |                                      |    T   |
C           |    A   |          ------------------          |    R   |
C           |    L   |=========>|  ORTHORHOMBIC  |=========>|    E   |
C           |        |          ------------------          |    S   |
C           |        |                   V                  |        |
C           |        |              ----------              |        |
C           |        |              | ORDHKL |              |        |
C           |        |              ----------              |        |
C           |        |                                      |        |
C           |        |                                      |        |
C           |        |          ------------------          |    R   |
C           |        |=========>|   MONOCLINIC   |=========>|    E   |
C           |        |          ------------------          |    S   |
C           |        |                   V                  |        |
C           |        |              ----------              |        |
C           |        |              | MODHKL |              |        |
C           |        |              ----------              |        |
C           |        |                                      |        |
C           |        |                                      |        |
C           |        |          ------------------          |        |
C           |        |=========>|   TRICLINIC    |=========>|        |
C           |        |          ------------------          ----------
C           |        |                   V                       V
C           |        |              ----------              ----------
C           |        |              | TRIHKL |              | PASAJE |
C           ----------              ----------              ----------
C     ------------------------------------------------------------------
C     |              P R O G R A M M E   P R I N C I P A L             |
C     ------------------------------------------------------------------
      character*20 file
      character*24 tmp
      logical qex
      DIMENSION IDENT(16)
      COMMON D(50),Q(50),KQ(50),TH(50),EPSIL(50),EPSQ(50),KEPSQ(50),DTH,
     1       V,VINF,VSUP,PAS,PAS2,PAS4,PAS8,PAS16,PAS32,PAS64,COEFF,RAP,
     2       WAVE2,PIDEG,PIRAD,N,MH,MH2,MK,MK2,ML,ML2,KDENS,KZ,KQT(50)
      COMMON /AMAX DD/AMAX,DD1,DD2
      COMMON /MONO/AMAXM,BMAXM,CMAXM,MC,PETIAMAX
      COMMON /JCOUNT/JCOUNT,EPST,NINI,FOM
      DATA IR,IW/15,16/
C     L E C T U R E   D E S   D O N N E E S
      print 5
    5 FORMAT('         DICVOL91   (by D. Louer)'/
     &'  is a program for indexing powder data files')
      print 1
1     format('  entry .dat file (no extension) ??',$)
      read 2,file
2     format(A20)
      lfile=len(file)
      do while (file(lfile:lfile).eq.' ')
      lfile=lfile-1
      enddo
      tmp=file(1:lfile)//'.dic'
      call open_read1(ir,tmp)
      tmp=file(1:lfile)//'.vol'
      inquire(file=tmp,exist=qex)
      if(.not. qex) go to 3
      print *,'warning, this old file will be deleted :',tmp
      call filedel(ipr,tmp)
3     call open_write1(iw,tmp)
      READ(IR,525,ERR=343)(IDENT(K),K=1,16)
      READ(IR,*,ERR=343)N,ITYPE,JC,JT,JH,JO,JM,JTR
      READ(IR,*,ERR=343)AMAX,BMAX,CMAX,VOLMIN,VOLMAX,BEMIN,BEMAX
      READ(IR,*,ERR=343)WAVE,POIMOL,DENS,DELDEN
      T1=SECNDS(0.0)
      TIME2=3600.*24.
      IF(AMAX.EQ.0.)AMAX=20.
      IF(BMAX.EQ.0.)BMAX=20.
      IF(CMAX.EQ.0.)CMAX=20.
      IF(VOLMAX.EQ.0.)VOLMAX=1500.
      IF(WAVE.EQ.0.0)WAVE=1.540598
      READ(IR,*,ERR=343)EPS,FOM,ZERO
C
      PI=4.*ATAN(1.)
      PIRAD=PI/180.
      PIDEG=180./PI
      WAVE2=WAVE/2.
      PAS=0.4
      COEF4=1.E-04
      COEFF=1.E+08
      IF(FOM.EQ.0.)FOM=5.
      IF(EPS.EQ.1.)THEN
      EPST=.0
      DO 36 I=1,N
      READ(IR,*,ERR=343)D(I),EPSIL(I)
      D(I)=D(I)+ZERO
      GO TO(42,43,44,45),ITYPE
   42 EPSTI=2.*EPSIL(I)+.015
      GO TO 36
   43 EPSTI=EPSIL(I)+.015
      GO TO 36
   44 EPSTI=2.*EPSIL(I)*WAVE/D(I)/SQRT(4.*D(I)**2-WAVE**2)*PIDEG+.015
      GO TO 36
   45 D(I)=COEF4*D(I)
      EPSIL(I)=COEF4*EPSIL(I)
      EPSTI=2.*WAVE*EPSIL(I)/2./SQRT(D(I)*(4.-D(I)*WAVE**2))*PIDEG+.015
      D(I)=D(I)/COEF4
      EPSIL(I)=EPSIL(I)/COEF4
   36 IF(EPSTI.GT.EPST)EPST=EPSTI
      GO TO 35
      ENDIF
      READ(IR,*,ERR=343)(D(I),I=1,N)
      DO 37 I=1,N
   37 D(I)=D(I)+ZERO
      IF(EPS.EQ.0.)EPS=.03
      EPST=EPS+.015
      IF(ITYPE.EQ.1)EPS=EPS/2.
      IF(ITYPE.EQ.1.OR.ITYPE.EQ.2)THEN
      DO 32 I=1,N
   32 EPSIL(I)=EPS
      GO TO 35
      ENDIF
      IF(ITYPE.EQ.3)THEN
      EPS=EPS/2.
      DO 33 I=1,N
   33 EPSIL(I)=D(I)/WAVE*EPS*PIRAD*SQRT(4.*D(I)**2-WAVE**2)
      GO TO 35
      ENDIF
      IF(ITYPE.EQ.4)THEN
      EPS=EPS/2.
      DO 34 I=1,N
      D(I)=COEF4*D(I)
      EPSIL(I)=2./COEF4*PIRAD*EPS/WAVE*SQRT(D(I)*(4.-D(I)*WAVE**2))
   34 D(I)=D(I)/COEF4
      ENDIF
C
C     P R E P A R A T I O N   E T   E C R I T U R E
C                 D E S   D O N N E E S
C
   35 CONTINUE
      BB=0.
      CC=0.
      BETA=0.
      JCOUNT=0
      NINI=N
C
C CDC TIME=SECOND()
      TIME=SECNDS(T1)
      IF(TIME.LT.0.)TIME=TIME+TIME2
C
      WRITE(IW,515)(IDENT(K),K=1,16)
      WRITE(IW,506)
      DTH=0.
      GOTO(110,112,114,116),ITYPE
  116 WRITE(IW,501)
      WRITE(IW,511)(D(I),EPSIL(I),I=1,N)
      DO 118 I=1,N
      Q(I)=D(I)*COEF4
      EPSQ(I)=EPSIL(I)*COEF4
      D(I)=1./SQRT(Q(I))
      QTERME=EPSQ(I)/(2.*Q(I))
      EPSIL(I)=D(I)*QTERME
      ANGULO=ASIN(WAVE2/D(I))
      TH(I)=2.*PIDEG*ANGULO
      DTH=DTH+(QTERME*PIDEG*TAN(ANGULO))
  118 CONTINUE
      VN=.6/(1./N-.0052)*D(N)**3
      GOTO 122
  114 WRITE(IW,502)
      WRITE(IW,510)(D(I),EPSIL(I),I=1,N)
      DO 124 I=1,N
      ANGULO=ASIN(WAVE2/D(I))
      TH(I)=2.*PIDEG*ANGULO
      DTH=DTH+(2.*TAN(ANGULO)*EPSIL(I)*PIDEG/D(I))
  124 CONTINUE
      VN=.6/(1./N-.0052)*D(N)**3
      GOTO 126
  112 WRITE(IW,503)
      WRITE(IW,511)(D(I),EPSIL(I),I=1,N)
      DOS=1.
      DOSINV=.5
      GOTO 128
  110 WRITE(IW,504)
      WRITE(IW,511)(D(I),EPSIL(I),I=1,N)
      DOS=2.
      DOSINV=1.
  128 DO 132 I=1,N
      TH(I)=DOS*D(I)
      ANGULO=PIRAD*DOSINV*D(I)
      D(I)=WAVE2/SIN(ANGULO)
      DTH=DTH+DOS*EPSIL(I)
      EPSIL(I)=D(I)*EPSIL(I)*PIRAD*DOSINV/TAN(ANGULO)
  132 CONTINUE
      VN=.6/(1./N-.0052)*D(N)**3
  126 DO 134 I=1,N
      PR=D(I)*D(I)
      Q(I)=1./PR
      EPSQ(I)=2.*EPSIL(I)/(PR*D(I))
  134 CONTINUE
      DTH=DTH/N
  122 CONTINUE
  123 NR=NINT(D(1)/D(2))
      IF(NR.EQ.1)GO TO 2079
      IF(ABS((NR*D(2)-D(1))).GE.(EPSIL(1)+NR*EPSIL(2)))GO TO 2079
      DI=D(2)
      THI=TH(2)
      QI=Q(2)
      EPSILI=EPSIL(2)
      EPSQI=EPSQ(2)
      DO 2040 I=2,N-1
      D(I)=D(I+1)
      TH(I)=TH(I+1)
      Q(I)=Q(I+1)
      EPSIL(I)=EPSIL(I+1)
 2040 EPSQ(I)=EPSQ(I+1)
      D(N)=DI
      TH(N)=THI
      Q(N)=QI
      EPSIL(N)=EPSILI
      EPSQ(N)=EPSQI
      N=N-1
      IF(N.EQ.0)THEN
      WRITE(*,*)' EXPERIMENTAL ERROR TOO LARGE !'
      WRITE(IW,*)' EXPERIMENTAL ERROR TOO LARGE !'
      GO TO 2200
      ENDIF
      GO TO 123
2079  DO 101 I=1,N
      KQ(I)=COEFF*Q(I)
      KEPSQ(I)=COEFF*EPSQ(I)
  101 CONTINUE
      IF (JM.EQ.0) GOTO 201
      IF (BEMIN.EQ.0.0) BEMIN=90.
      IF (BEMAX.EQ.0.0) BEMAX=125.
      WRITE(IW,524)AMAX,BEMIN,VOLMIN,BMAX,BEMAX,VOLMAX,CMAX
      GOTO 202
  201 IF((JC+JT+JH+JO).EQ.0)THEN
      WRITE(IW,529)VOLMIN,VOLMAX
      ELSE
      WRITE(IW,526)AMAX,VOLMIN,BMAX,VOLMAX,CMAX
      ENDIF
  202 WRITE(IW,527)WAVE
      WRITE(IW,827)NINI,FOM
C
C     A V E C   O U   S A N S   D E N S I T E ?
C
      KDENS=1
      X=POIMOL*DENS*DELDEN
      IF(X.LE.1.0E-04)GO TO 70
      KDENS=2
      WRITE(IW,528)POIMOL,DENS,DELDEN
   70 WRITE(IW,22)
C
C     F O R M A T S
C
    4 FORMAT(/1H ,10X,'MOLECULAR WEIGHTE =',F9.4,10X,8HDENSITY=,F7.4,10X
     1,10HPRECISION=,F7.4)
  515 FORMAT(1H1/1H ,28X,68(1H-)/1H ,28X,2H| ,16A4,2H |/1H ,28X,68(1H-))
  506 FORMAT(/58X,'INPUT DATA'/58X,10(1H*))
    7 FORMAT(//1H ,55X,10(1H*)/1H ,10X,11HA MAXIMUM =,F6.2/1H ,10X,11
     1HB MAXIMUM =,F6.2/1H ,10X,11HC MAXIMUM =,F6.2/1H ,10X,16HVOLUME MI
     2NIMUM =,F9.2/1H ,10X,16HVOLUME MAXIMUM =,F9.2)
    8 FORMAT(8X,'CUBIC SYSTEM'/8X,12('-'))
    9 FORMAT(/,58X,'NO SOLUTION')
   10 FORMAT(8X,'TETRAGONAL SYSTEM'/8X,17('-'))
   11 FORMAT(8X,'HEXAGONAL SYSTEM'/8X,16('-'))
   12 FORMAT(8X,'ORTHORHOMBIC SYSTEM'/8X,19('-'))
  206 FORMAT(//46X,'END OF SEARCH FOR CUBIC SOLUTION(S)'/46X,35('*'))
   13 FORMAT(//25X,77HEND OF SEARCH FOR TETRAGONAL AND/OR HEXAGONAL AND/
     1OR ORTHORHOMBIC SOLUTION(S)/25X,77(1H*))
  510 FORMAT(33X,F10.4,37X,F10.4)
   16 FORMAT(8X,'MONOCLINIC SYSTEM'/8X,17('-'))
   17 FORMAT(//10X,'VOLUME-DOMAIN BEING SCANNED       LOWER BOUND =',F9.
     12,'A**3',5X,'HIGHER BOUND =',F9.2,'A**3'/10X,27('=')//)
   22 FORMAT(///34X,60('*')/34X,'**',56X,'**'/34X,'**  ATTENTION  : VOS
     SDONNEES SONT-ELLES IRREPROCHABLES ?  **'/34X,'**  WARNING  :   ARE
     S YOUR DATA IRREPROACHABLE ?           **'/34X,'**',56X,'**'/
     S34X,60('*')///)
   23 FORMAT(//45X,38HEND OF SEARCH FOR MONOCLINIC SOLUTIONS/
     S45X,38(1H*))
   24 FORMAT(/1H ,10X,14HBETA MINIMUM =,F9.2/1H ,10X,14HBETA MAXIMUM =,F
     19.2)
   25 FORMAT(//50X,'SEARCH OF CUBIC SOLUTION(S)'/50X,27('*')//)
   26 FORMAT(////29X,'SEARCH OF TETRAGONAL AND/OR HEXAGONAL AND/OR ORTHO
     SRHOMBIC SOLUTION(S)'/29X,69('*')/)
   28 FORMAT(////48X,'SEARCH OF MONOCLINIC SOLUTION(S)'/48X,33('*')/)
   29 FORMAT(////49X,'SEARCH OF TRICLINIC SOLUTION(S)'/49X,32('*')/)
  525 FORMAT(16A4)
  501 FORMAT(34X,'EXPERIMENTAL',35X,'EXPERIMENTAL'/
     S       36X,'Q VALUES',41X,'ERROR'/)
  502 FORMAT(34X,'EXPERIMENTAL',35X,'EXPERIMENTAL'/
     S       36X,'D VALUES',41X,'ERROR'/)
  503 FORMAT(34X,'EXPERIMENTAL',35X,'EXPERIMENTAL'/
     S       36X,'2-THETA',42X,'ERROR'/)
  504 FORMAT(34X,'EXPERIMENTAL',35X,'EXPERIMENTAL'/
     S       37X,'THETA',43X,'ERROR'/)
  511 FORMAT(33X,F10.3,36X,F10.3)
  524 FORMAT(//43X,42('*')//20X,9('-'),'----  PARAMETER LIMITS  ----',
     S       10('-'),5x,5('-'),'---  VOLUME LIMITS  ---',6('-')/
     S       20X,'|',45X,'|',5X,'|',32X,'|'/20X,'|',4X,'A MAXIMUM =',
     S       F6.2,24X,'|',5X,'|',32X,'|'/20X,'|',19X,'BETA MINIMUM =',
     S       F8.2,4X,'|',5X,'|',4X,'VOLUME MINIMUM =',F8.2,4X,'|'/
     S       20X,'|',4X,'B MAXIMUM =',F6.2,24X,'|',5X,'|',32X,'|'/
     S       20X,'|',19X,'BETA MAXIMUM =',F8.2,4X,'|',5X,'|',4X,
     S       'VOLUME MAXIMUM =',F8.2,4X,'|'/20X,'|',4X,'C MAXIMUM =',
     S       F6.2,24X,'|',5X,'|',32X,'|'/20X,'|',45X,'|',5X,'|',32X,
     S       '|'/20X,47('-'),5X,34('-')//)
  526 FORMAT(//43X,42('*')//20X,9('-'),'----  PARAMETER LIMITS  ----',
     S       10('-'),5X,5('-'),'---  VOLUME LIMITS  ---',6('-')/
     S       20X,'|',45X,'|',5X,'|',32X,'|'/20X,'|',14X,'A MAXIMUM =',
     S       F6.2,14X,'|',5X,'|',32X,'|'/20X,'|',45X,'|',5X,'|',4X,
     S       'VOLUME MINIMUM =',F8.2,4X,'|'/20X,'|',14X,'B MAXIMUM =',
     S       F6.2,14X,'|',5X,'|',32X,'|'/20X,'|',45X,'|',5X,'|',4X,
     S       'VOLUME MAXIMUM =',F8.2,4X,'|'/20X,'|',14X,'C MAXIMUM =',
     S       F6.2,14X,'|',5X,'|',32X,'|'/20X,'|',45X,'|',5X,'|',32X,
     S       '|'/20X,47('-'),5X,34('-')//)
  529 FORMAT(//43X,42('*')//47X,6('-'),'---  VOLUME LIMITS  ---',5('-')/
     S       47X,'|',32X,'|'/47X,'|',32X,'|'/47X,'|',3X,' VOLUME MINIMUM
     S =',F8.2,4X,'|'/47X,'|',32X,'|'/47X,'|',3X,' VOLUME MAXIMUM =',
     SF8.2,4X,'|'/47X,'|',32X,'|'/47X,'|',32X,'|'/47X,34('-')//)
  527 FORMAT(53X,'WAVELENGTH =',F9.6)
  528 FORMAT(//5X,'MOLECULAR WEIGHT =',F9.4,5X,
     S       'MEASURED DENSITY =',F8.4,' G.CM(-3)',5X,
     S       'DENSITY ERROR =',F8.4,' G.CM(-3)'/)
  808 FORMAT(//,10X,'SEARCH OF MONOCLINIC SOLUTION(S) WITHIN THE LIMITS
     SON LINEAR PARAMETERS (SLIGHT TOLERANCE ACCEPTED) :'/
     S14X,'A MAX =',F8.3,6X,'B MAX =',F8.3,6X,'C MAX =',F8.3,6X,F8.3//)
  809 FORMAT(//,10X,'EXTENSION OF THE SEARCH OF MONOCLINIC SOLUTION(S) W
     SITHIN THE LIMITS ON LINEAR PARAMETERS :'/
     S14X,'A MAX =',F8.3,6X,'B MAX =',F8.3,6X,'C MAX =',F8.3,6X,F8.3//)
  827 FORMAT(/5X,'LOWER FIGURE OF MERIT REQUIRED FOR PRINTED SOLUTION(S)
     S :  M(',I3,') = ',F5.1/)
C
C     P A R T I E   C O M M U N E
C
      V=VOLMAX
      VOLMAXC=VOLMAX
      VV=V
      PAS2=PAS/2.
      PAS4=PAS/4.
      PAS8=PAS/8.
      PAS16=PAS/16.
      PAS32=PAS/32.
      PAS64=PAS/64.
      VSUPM=VOLMIN
      AMIN=D(1)-EPSIL(1)
      DD1=AMIN
      ZB=2.*EPSIL(2)+EPSIL(1)
      ZZ=ABS(2.*D(2)-D(1))
      IF(ZZ-ZB)19,19,18
   19 BMIN=D(3)-EPSIL(3)
      DD2=BMIN
      GO TO 27
   18 BMIN=D(2)-EPSIL(2)
      DD2=BMIN
   27 CMIN=2.5
      NA=(AMAX-AMIN)/PAS+1.
      NB=(BMAX-BMIN)/PAS+1.
      NC=(CMAX-CMIN)/PAS+1.
      IF(KDENS.EQ.2)GO TO 3001
C
C     T R A I T E M E N T   S A N S   D E N S I T E
C
      VSUP=VOLMIN
      PASVOL=400.
      NVOL=(VOLMAXC-VOLMIN)/PASVOL
      NVOL=NVOL+1
      IF(JC.EQ.0)GO TO 21
      WRITE(IW,25)
      VINF=VOLMIN
      VSUP=VOLMAXC
      WRITE(IW,17)VINF,VSUP
      WRITE(IW,8)
      CALL CUBIQU(AMIN,AMAX,NA)
      DIFVOL=ABS(VV-V)
      IF(DIFVOL-1E-12)20,20,52
   20 WRITE(IW,9)
   52 WRITE(IW,206)
      VSUP=VSUPM
      VV=V
      VOLMAXC=AMIN 1(VOLMAXC,V)
   21 IF(JT.EQ.0.AND.JH.EQ.0.AND.JO.EQ.0)GO TO 2100
      WRITE(IW,26)
      DO 1000 KVOL=1,NVOL
      VINF=VSUP
      VSUP=VINF+PASVOL
      IF(VSUP.GT.VOLMAXC)VSUP=VOLMAXC
      IF(VINF.GE.VOLMAXC)GO TO 1100
      WRITE(IW,17)VINF,VSUP
      IF(JT.EQ.0)GO TO 31
      WRITE(IW,10)
      ICHOIX=-1
      CALL TETHEX(CMIN,NC,ICHOIX)
      DIFVOL=ABS(VV-V)
      IF(DIFVOL-1E-12)30,30,31
   30 WRITE(IW,9)
   31 IF(JH.EQ.0)GO TO 41
      VV=V
      WRITE(IW,11)
      ICHOIX=1
      CALL TETHEX(CMIN,NC,ICHOIX)
      DIFVOL=ABS(VV-V)
      IF(DIFVOL-1E-12)40,40,41
   40 WRITE(IW,9)
   41 IF(JO.EQ.0)GO TO 1000
      VV=V
      WRITE(IW,12)
      CALL ORTHOR(AMIN,BMIN,CMIN,NA,NB,NC)
      DIFVOL=ABS(VV-V)
      IF(DIFVOL-1E-12)50,50,1100
   50 WRITE(IW,9)
 1000 CONTINUE
 1100 WRITE(IW,13)
C
C CDC TIME=SECOND()
      TIME=SECNDS(T1)
      IF(TIME.LT.0.)TIME=TIME+TIME2
C     WRITE(*,614)TIME
      WRITE(IW,614)TIME
C
      VV=V
      VOLMAXC=AMIN 1(VOLMAXC,V)
2100  IF(JM.EQ.0)GO TO 2700
      WRITE(IW,28)
      WRITE(IW,16)
      BEMIN=BEMIN*PIRAD
      BEMAX=BEMAX*PIRAD
      PETIAMAXI=AMAX
      PETICMAXI=CMAX
      AMIN=BMIN*SIN(BEMAX)
      BMIN=CMIN
      CMIN=CMIN*SIN(BEMAX)
      NAC=NINT((AMIN-CMIN)/PAS)
      IF((NAC*PAS).LT.(AMIN-CMIN))NAC=NAC+1
      CMIN=AMIN-NAC*PAS
      AMAX=AMAX*SIN(BEMIN)
      CMAX=CMAX*SIN(BEMIN)
      AMAXI=AMAX
      BMAXI=BMAX
      CMAXI=CMAX
C
      Q11=2./SQRT(Q(1)-EPSQ(1))
      AMAX=AMIN 1(((Q11+.5)*SIN(BEMIN)),AMAX)
      BMAX=AMIN 1(Q11+.5,BMAX)
      IF(AMAX.GT.AMAXI)AMAX=AMAXI
      IF(BMAX.GT.BMAXI)BMAX=BMAXI
      IF(CMAX.GT.CMAXI)CMAX=CMAXI
      CMAX=AMIN 1(AMAX,CMAX)
      MC=1
      GO TO 21001
2001  AMAXM=AMAX
      BMAXM=BMAX
      CMAXM=CMAX
      JFL=JFL+1
      IF(AMAX.GE.AMAXI.AND.BMAX.GE.BMAXI.AND.CMAX.GE.CMAXI)GO TO 2300
      AMAX=AMAX+8.
      BMAX=BMAX+8.
      CMAX=CMAX+8.
      IF(AMAX.GT.AMAXI)AMAX=AMAXI
      IF(BMAX.GT.BMAXI)BMAX=BMAXI
      IF(CMAX.GT.CMAXI)CMAX=CMAXI
      IF(CMAX.GT.AMAX)CMAX=AMAX
      MC=0
      GO TO 21003
21001 PETIAMAX=AMIN 1(PETIAMAXI,AMAX/SIN(BEMIN))
      PETICMAX=AMIN 1(PETICMAXI,CMAX/SIN(BEMIN))
      WRITE(IW,808)PETIAMAX,BMAX,PETICMAX
21003 NA=(AMAX-AMIN)/PAS
      NB=(BMAX-BMIN)/PAS
      NC=(CMAX-CMIN)/PAS
      VSUP=VSUPM
      PETIAMAX=AMIN 1(PETIAMAXI,AMAX/SIN(BEMIN))
      PETICMAX=AMIN 1(PETICMAXI,CMAX/SIN(BEMIN))
      IF(MC.EQ.0)WRITE(IW,809)PETIAMAX,BMAX,PETICMAX
      PETIAMAX=AMIN 1(PETIAMAXI,AMAX/SIN(BEMAX))
      PETICMAX=AMIN 1(PETICMAXI,CMAX/SIN(BEMAX))
      DO 2000 KVOL=1,NVOL
      VINF=VSUP
      VSUP=VINF+PASVOL
      IF(VSUP.GT.VOLMAXC)VSUP=VOLMAXC
      IF(VINF.GE.VOLMAXC)GO TO 2000
      WRITE(IW,17)VINF,VSUP
      CALL MONOC(AMIN,NA,NB,CMIN,NC,BEMAX,BEMIN,BMIN)
      DIFVOL=ABS(VV-V)
      IF(DIFVOL-1E-12)60,60,2300
   60 WRITE(IW,9)
 2000 CONTINUE
      GO TO 2001
 2300 WRITE(IW,23)
C
C CDC TIME=SECOND()
      TIME=SECNDS(T1)
      IF(TIME.LT.0.)TIME=TIME+TIME2
C     WRITE(*,14)TIME
      WRITE(IW,14)TIME
C
 2700 IF(JTR.EQ.0)GO TO 2200
      WRITE(IW,29)
      WRITE(IW,702)
      AMAX=AMAXI
      BMAX=BMAXI
      CMAX=CMAXI
      VV=V
      VOLMAXC=AMIN 1(VOLMAXC,V)
      VMOI=AMAX 1(VN*.7,VOLMIN)
      VPLU=AMIN 1(VN*1.3,VOLMAXC)
      KLV=1
      IF(VPLU.LT.VMOI)THEN
      VMOI=VOLMIN
      VPLU=VOLMAXC
      KLV=3
      ENDIF
      VMOII=VMOI
 6801 PASVOL1=AMIN 1(200.,(VPLU-VMOI))
      IF(VMOI.EQ.VN*.7.AND.VPLU.EQ.VN*1.3)PASVOL1=VPLU-VMOI
      NVOL=(VPLU-VMOI)/PASVOL1
      DO 6802 I=0,NVOL
      VMIN=VMOI+I*PASVOL1
      VMAX=VMIN+PASVOL1
      IF(VMIN.GE.VPLU)THEN
      GO TO(6805,6806,6807),KLV
      ENDIF
      IF(VMAX.GT.VPLU)VMAX=VPLU
      WRITE(IW,17)VMIN,VMAX
      CALL TRICLINI(VMIN,VMAX)
      DIFVOL=ABS(V-VV)
      IF(DIFVOL-1E-12)6802,6802,6807
 6802 WRITE(IW,9)
      GO TO(6805,6806,6807),KLV
 6805 IF(VPLU.EQ.VOLMAXC)GO TO 6806
      VMOI=VPLU
      VPLU=VOLMAXC
      KLV=2
      GO TO 6801
 6806 IF(VMOII.EQ.VOLMIN)GO TO 6807
      VMOI=VOLMIN
      VPLU=VMOII
      KLV=3
      GO TO 6801
 6807 WRITE(IW,703)
      GO TO 2200
C
C     T R A I T E M E N T   A V E C   D E N S I T E
C
 3001 AVOG=0.602253
      RAP=POIMOL/AVOG
      VUNITP=RAP/(DENS-DELDEN)
      VUNITM=RAP/(DENS+DELDEN)
      IF(JC.EQ.0)GO TO 121
      WRITE(IW,25)
      WRITE(IW,8)
      DO 3002 KZ=1,50
      VINF=KZ*VUNITM
      VSUP=KZ*VUNITP
      IF (VSUP.LT.VOLMIN)GO TO 3002
      IF(VSUP.GT.VOLMAXC)VSUP=VOLMAXC	
      IF(VINF.GE.VOLMAXC)GO TO 3101
      IF(VINF.LT.VOLMIN)VIN=VOLMIN
      WRITE(IW,17)VINF,VSUP
      CALL CUBIQU(AMIN,AMAX,NA)
      DIFVOL=ABS(VV-V)
      IF(DIFVOL-1E-12)120,120,121
  120 WRITE(IW,9)
 3002 CONTINUE
 3101 WRITE(IW,206)
      VV=V
      VOLMAXC=AMIN 1(VOLMAXC,V)
  121 IF(JT.EQ.0.AND.JH.EQ.0.AND.JO.EQ.0)GO TO 3200
      WRITE(IW,26)
      DO 3000 KZ=1,50
      VINF=KZ*VUNITM
      VSUP=KZ*VUNITP
      IF (VSUP.LT.VOLMIN)GO TO 3000
      IF (VSUP.GT.VOLMAXC)VSUP=VOLMAXC
      IF(VINF.GE.VOLMAXC)GO TO 3100
      IF(VINF.LT.VOLMIN)VINF=VOLMIN
      WRITE(IW,17)VINF,VSUP
      IF(JT.EQ.0)GO TO 131
      VV=V
      WRITE(IW,10)
      ICHOIX=-1
      CALL TETHEX(CMIN,NC,ICHOIX)
      DIFVOL=ABS(VV-V)
      IF(DIFVOL-1E-12)130,130,131
  130 WRITE(IW,9)
  131 IF(JH.EQ.0)GO TO 141
      VV=V
      WRITE(IW,11)
      ICHOIX=1
      CALL TETHEX(CMIN,NC,ICHOIX)
      DIFVOL=ABS(VV-V)
      IF(DIFVOL-1E-12)140,140,141
  140 WRITE(IW,9)
  141 IF(JO.EQ.0)GO TO 3000
      VV=V
      WRITE(IW,12)
      CALL ORTHOR(AMIN,BMIN,CMIN,NA,NB,NC)
      DIFVOL=ABS(VV-V)
      IF(DIFVOL-1E-12)150,150,3100
  150 WRITE(IW,9)
 3000 CONTINUE
 3100 WRITE(IW,13)
C
C CDC TIME=SECOND()
      TIME=SECNDS(T1)
      IF(TIME.LT.0.)TIME=TIME+TIME2
C     WRITE(*,614)TIME
      WRITE(IW,614)TIME
C
 3200 IF(JM.EQ.0)GO TO 3300
      WRITE(IW,28)
      WRITE(IW,16)
      BEMIN=BEMIN*PIRAD
      BEMAX=BEMAX*PIRAD
      PETIAMAXI=AMAX
      PETICMAXI=CMAX
      AMIN=BMIN*SIN(BEMAX)
      BMIN=CMIN
      CMIN=CMIN*SIN(BEMAX)
      NAC=NINT((AMIN-CMIN)/PAS)
      IF((NAC*PAS).LT.(AMIN-CMIN))NAC=NAC+1
      CMIN=AMIN-NAC*PAS
      VV=V
      AMAX=AMAX*SIN(BEMIN)
      CMAX=CMAX*SIN(BEMIN)
      AMAXI=AMAX
      BMAXI=BMAX
      CMAXI=CMAX
C
      Q11=2./SQRT(Q(1)-EPSQ(1))
      AMAX=AMIN 1((Q11+.5)*SIN(BEMIN),AMAX)
      BMAX=AMIN 1(Q11+.5,BMAX)
      IF(AMAX.GT.AMAXI)AMAX=AMAXI
      IF(BMAX.GT.BMAXI)BMAX=BMAXI
      IF(CMAX.GT.CMAXI)CMAX=CMAXI
      CMAX=AMIN 1(AMAX,CMAX)
      MC=1
      GO TO 21002
2002  AMAXM=AMAX
      BMAXM=BMAX
      CMAXM=CMAX
      IF(AMAX.EQ.AMAXI.AND.BMAX.EQ.BMAXI.AND.CMAX.EQ.CMAXI)GO TO 4300
      AMAX=AMAX+8.
      BMAX=BMAX+8.
      CMAX=CMAX+8.
      IF(AMAX.GT.AMAXI)AMAX=AMAXI
      IF(BMAX.GT.BMAXI)BMAX=BMAXI
      IF(CMAX.GT.CMAXI)CMAX=CMAXI
      IF(CMAX.GT.AMAX)CMAX=AMAX
      MC=0
      GO TO 21004
21002 PETIAMAX=AMIN 1(PETIAMAXI,AMAX/SIN(BEMIN))
      PETICMAX=AMIN 1(PETICMAXI,CMAX/SIN(BEMIN))
      WRITE(IW,808)PETIAMAX,BMAX,PETICMAX
21004 NA=(AMAX-AMIN)/PAS
      NB=(BMAX-BMIN)/PAS
      NC=(CMAX-CMIN)/PAS
      PETIAMAX=AMIN 1(PETIAMAXI,AMAX/SIN(BEMIN))
      PETICMAX=AMIN 1(PETICMAXI,CMAX/SIN(BEMIN))
      IF(MC.EQ.0)WRITE(6,808)PETIAMAX,BMAX,PETICMAX
      PETIAMAX=AMIN 1(PETIAMAXI,AMAX/SIN(BEMAX))
      PETICMAX=AMIN 1(PETICMAXI,CMAX/SIN(BEMAX))
      DO 4000 KZ=1,50
      VINF=KZ*VUNITM
      VSUP=KZ*VUNITP
      IF(VSUP.LT.VOLMIN)GO TO 4000
      IF(VINF.GE.VOLMAXC)GO TO 4000
      IF(VSUP.GT.VOLMAXC)VSUP=VOLMAXC
      IF(VINF.LT.VOLMIN)VINF=VOLMIN	
      WRITE(IW,17)VINF,VSUP
      CALL MONOC(AMIN,NA,NB,CMIN,NC,BEMAX,BEMIN,BMIN)
      DIFVOL=ABS(VV-V)
      IF(DIFVOL-1E-12)160,160,4300
  160 WRITE(IW,9)
 4000 CONTINUE
      GO TO 2002
 4300 WRITE(IW,23)
C
C CDC TIME=SECOND()
      TIME=SECNDS(T1)
      IF(TIME.LT.0.)TIME=TIME+TIME2
C     WRITE(*,14)TIME
      WRITE(IW,14)TIME
C
 3300 IF(JTR.EQ.0)GO TO 2200
      WRITE(IW,29)
      WRITE(IW,702)
      AMAX=AMAXI
      BMAX=BMAXI
      CMAX=CMAXI
      VV=V
      VOLMAXC=AMIN 1(VOLMAXC,V)
      VMIN=.7*VN
      VMAX=1.3*VN
      IF(VMIN.LT.VOLMIN)VMIN=VOLMIN
      IF(VMAX.GT.VOLMAXC)VMAX=VOLMAXC
      VOINF=VMIN
      VOSUP=VMAX
      KZT=1
      KZ1=0
1085  CONTINUE
      DO 5000 KZ=1,50
      IF(KZT.EQ.1.AND.KZ.GE.KZ1.AND.KZ.LE.KZ2)GO TO 5000
      VMIN=KZ*VUNITM
      VMAX=KZ*VUNITP
      IF(VMAX.LT.VOINF)GO TO 5000
      IF(VMIN.GT.VOSUP)GO TO 701
      IF(VMAX.GT.VOLMAXC)VMAX=VOLMAXC
      WRITE(IW,17)VMIN,VMAX
      WRITE(IW,702)
      IF(KZ1.EQ.0)KZ1=KZ
      CALL TRICLINI(VMIN,VMAX)
  702 FORMAT(8X,'TRICLINIC SYSTEM'/8X,16('-'))
      DIFVOL=ABS(VV-V)
      IF(DIFVOL-1E-12)704,704,707
  704 WRITE(IW,9)
 5000 CONTINUE
  701 IF(VOINF.EQ.VOLMIN.AND.VOSUP.EQ.VOLMAXC)GO TO 707
      VOINF=VOLMIN
      VOSUP=VOLMAXC
      KZT=1
      KZ2=KZ-1
      GO TO 1085
  707 WRITE(IW,703)
  703 FORMAT(//39X,'END OF SEARCH FOR TRICLINIC SOLUTIONS',/
     +39X,37(1H*))
      GO TO 2200
  343 WRITE(IW,*)' ERROR IN READING !'
      WRITE(*,*)' ERROR IN READING !'
      STOP
 2200 CONTINUE
C
C CDC TIME=SECOND()
      TIME=SECNDS(T1)
      IF(TIME.LT.0.)TIME=TIME+TIME2
      WRITE(IW,1012)TIME
C
      WRITE(IW,1013)
  614 FORMAT(/,12X,'---  CALCULATION TIME FOR SEARCH DOWN TO ORTHORHOMBI
     +C SYMMETRY :',F10.4,' SECONDS'/)
   14 FORMAT(/,12X,'---  CALCULATION TIME FOR SEARCH DOWN TO MONOCLINIC
     + SYMMETRY :',F10.4,' SECONDS'/)
 1012 FORMAT(/,12X,'---  T O T A L   CALCULATION TIME : ',F10.4,
     +' SECONDS'///)
 1013 FORMAT(///14X,'DICVOL91 :  USEFUL REFERENCES'/14X,'--------'/16X,
     +'* LOUER, D. & LOUER, M. (1972). J. APPL. CRYST. 5, 271-275.'/16X
     +,'* BOULTIF, A. & LOUER, D. (1991). J. APPL. CRYST. 24, 987-993.')
      STOP
      END
C     ------------------------------------------------------------------
C     |                         C U B I Q U E                          |
C     ------------------------------------------------------------------
      SUBROUTINE CUBIQU(AMIN,AMAX,NA)
      COMMON D(50),Q(50),KQ(50),TH(50),EPSIL(50),EPSQ(50),KEPSQ(50),DTH,
     1       V,VINF,VSUP,PAS,PAS2,PAS4,PAS8,PAS16,PAS32,PAS64,COEFF,RAP,
     2       WAVE2,PIDEG,PIRAD,N,MH,MH2,MK,MK2,ML,ML2,KDENS,KZ,KQT(50)
      COMMON QCAL(50,20),AA,BB,CC,BETA,AR,BR,CR,DR,ER,FR
      COMMON /BAF/NT,IRJ(50,8),IH(50,50,8),IK(50,50,8),IL(50,100,8)
     1           ,NPOSS,FWOLFF
      DIMENSION NDICH(7)
      DATA IR,IW/15,16/
    1 FORMAT(1H ,10X,2HA=,F8.5,10X,33HVOLUME DE LA MAILLE ELEMENTAIRE =,
     1F10.3)
    2 FORMAT(1H ,10X,F8.5,4X,3(10X,F3.0),9X,F8.5,11X,F8.5,9X,F8.5)
    3 FORMAT(1H ,50X,9(1H-)/)
    4 FORMAT(1H ,10X,2HA=,F8.5,10X,33HVOLUME DE LA MAILLE ELEMENTAIRE =,
     1F10.3,5X,2HZ=,I2)
    5 FORMAT(1H ,7X,14HD EXPERIMENTAL,12X,1HH,12X,1HK,12X,1HL,10X,9HD CA
     1LCULE,12X,5HDE-DC,12X,7HEPSILON/1H )
      DO 21 I=1,7
   21 NDICH(I)=0
      AA=1000.
      DO 1000 IA=1,NA
      AW=IA-1
      AMOI1=AMIN+(AW*PAS)
      IF(AMOI1.GT.AMAX)GO TO 1000
      APLU1=AMOI1+PAS
      VTESTP=APLU1**3
      VTESTM=AMOI1**3
      IF(VTESTP.LT.VINF.OR.VTESTM.GT.VSUP)GO TO 1000
      IF(V-VTESTM)1000,20,20
   20 MH=APLU1/D(5)
      MH2=APLU1/D(N)
      KRAM1=COEFF/(AMOI1*AMOI1)
      KRAP1=COEFF/(APLU1*APLU1)
      CALL CUDHKL(KRAM1,KRAP1)
      IF(NT.EQ.-1)GO TO 1000
      DO 120 I=6,N
      IF(IRJ(I,1).EQ.0) GO TO 1000
  120 CONTINUE
      NDICH(1)=NDICH(1)+1
      DO 2000 IA2=1,2
      AW=IA2-1
      AMOI2=AMOI1+(AW*PAS2)
      IF(ABS(AMOI2-AA).LT.0.25)GO TO 2000
      APLU2=AMOI2+PAS2
      VTESTM=AMOI2*AMOI2*AMOI2
      IF(VSUP.LT.VTESTM)GO TO 1000
      VTESTP=APLU2*APLU2*APLU2
      IF(VINF.GT.VTESTP)GO TO 2000
      KRAM2=COEFF/(AMOI2*AMOI2)
      KRAP2=COEFF/(APLU2*APLU2)
      CALL CUDHKL1(KRAM2,KRAP2,2)
      IF(NT.EQ.-1)GO TO 2000
      NDICH(2)=NDICH(2)+1
      DO 3000 IA3=1,2
      AW=IA3-1
      AMOI3=AMOI2+(AW*PAS4)
      IF(ABS(AMOI3-AA).LT.0.25)GO TO 3000
      APLU3=AMOI3+PAS4
      VTESTM=AMOI3*AMOI3*AMOI3
      IF(VSUP.LT.VTESTM)GO TO 2000
      VTESTP=APLU3*APLU3*APLU3
      IF(VINF.GT.VTESTP)GO TO 3000
      KRAM3=COEFF/(AMOI3*AMOI3)
      KRAP3=COEFF/(APLU3*APLU3)
      CALL CUDHKL1(KRAM3,KRAP3,3)
      IF(NT.EQ.-1)GO TO 3000
      NDICH(3)=NDICH(3)+1
      DO 4000 IA4=1,2
      AW=IA4-1
      AMOI4=AMOI3+(AW*PAS8)
      IF(ABS(AMOI4-AA).LT.0.25)GO TO 4000
      APLU4=AMOI4+PAS8
      VTESTM=AMOI4*AMOI4*AMOI4
      IF(VSUP.LT.VTESTM)GO TO 3000
      VTESTP=APLU4*APLU4*APLU4
      IF(VINF.GT.VTESTP)GO TO 4000
      KRAM4=COEFF/(AMOI4*AMOI4)
      KRAP4=COEFF/(APLU4*APLU4)
      CALL CUDHKL1(KRAM4,KRAP4,4)
      IF(NT.EQ.-1)GO TO 4000
      NDICH(4)=NDICH(4)+1
      DO 5000 IA5=1,2
      AW=IA5-1
      AMOI5=AMOI4+(AW*PAS16)
      IF(ABS(AMOI5-AA).LT.0.25)GO TO 5000
      APLU5=AMOI5+PAS16
      VTESTM=AMOI5*AMOI5*AMOI5
      IF(VSUP.LT.VTESTM)GO TO 4000
      VTESTP=APLU5*APLU5*APLU5
      IF(VINF.GT.VTESTP)GO TO 5000
      KRAM5=COEFF/(AMOI5*AMOI5)
      KRAP5=COEFF/(APLU5*APLU5)
      CALL CUDHKL1(KRAM5,KRAP5,5)
      IF(NT.EQ.-1)GO TO 5000
      NDICH(5)=NDICH(5)+1
      DO 6000 IA6=1,2
      AW=IA6-1
      AMOI6=AMOI5+(AW*PAS32)
      IF(ABS(AMOI6-AA).LT.0.25)GO TO 6000
      APLU6=AMOI6+PAS32
      VTESTM=AMOI6*AMOI6*AMOI6
      IF(VSUP.LT.VTESTM)GO TO 5000
      VTESTP=APLU6*APLU6*APLU6
      IF(VINF.GT.VTESTP)GO TO 6000
      KRAM6=COEFF/(AMOI6*AMOI6)
      KRAP6=COEFF/(APLU6*APLU6)
      CALL CUDHKL1(KRAM6,KRAP6,6)
      IF(NT.EQ.-1)GO TO 6000
      NDICH(6)=NDICH(6)+1
      DO 7000 IA7=1,2
      AW=IA7-1
      AMOI7=AMOI6+(AW*PAS64)
      IF(ABS(AMOI7-AA).LT.0.25)GO TO 7000
      APLU7=AMOI7+PAS64
      VTESTM=AMOI7*AMOI7*AMOI7
      IF(VSUP.LT.VTESTM)GO TO 6000
      VTESTP=APLU7*APLU7*APLU7
      IF(VINF.GT.VTESTP)GO TO 7000
      RAM7=1./(AMOI7*AMOI7)
      RAP7=1./(APLU7*APLU7)
      KRAM7=COEFF*RAM7
      KRAP7=COEFF*RAP7
      CALL CUDHKL1(KRAM7,KRAP7,7)
      IF(NT.EQ.-1)GO TO 7000
      NDICH(7)=NDICH(7)+1
      AARE=AA
      AA=AMOI7+(PAS64/2.)
      VAP=AA*AA*AA
      DO 15 I=1,N
      DO 16 J=1,20
   16 QCAL(I,J)=-1.
   15 CONTINUE
      DO 45 I=1,N
      JJ=IRJ(I,7)
      DO 35 J=1,JJ
      CARL=IH(I,J,7)**2+IK(I,J,7)**2+IL(I,J,7)**2
      QCAL(I,J)=CARL/AA**2
  35  CONTINUE
  45  CONTINUE
      IND=1
      NRIND=1
      CALL AFFPAR(IND,NRIND,VAP)
      IF(FWOLFF.NE.-1000.)GO TO 7113
      AA=AARE
      GO TO 7000
 7113 WRITE(IW,3)
 7000 CONTINUE
 6000 CONTINUE
 5000 CONTINUE
 4000 CONTINUE
 3000 CONTINUE
 2000 CONTINUE
 1000 CONTINUE
      WRITE(IW,10)(NDICH(I),I=1,7)
   10 FORMAT(/,10X,'ITERATION NUMBER AT EACH DICHOTOMY LEVEL :',
     S7X,7(3X,I5))
      RETURN
      END
      SUBROUTINE CUDHKL(KRAM,KRAP)
      COMMON D(50),Q(50),KQ(50),TH(50),EPSIL(50),EPSQ(50),KEPSQ(50),DTH,
     1       V,VINF,VSUP,PAS,PAS2,PAS4,PAS8,PAS16,PAS32,PAS64,COEFF,RAP,
     2       WAVE2,PIDEG,PIRAD,N,MH,MH2,MK,MK2,ML,ML2,KDENS,KZ,KQT(50)
      COMMON /BAF/NT,IRJ(50,8),IH(50,50,8),IK(50,50,8),IL(50,100,8)
     1           ,NPOSS,FWOLFF
      NT=0
      DO 15 I=1,N
      IRJ(I,1)=0
   15 CONTINUE
      DO 10 M=1,MH
      MCARH=M*M
      DO 20 K=0,M
      MCARK=K*K+MCARH
      DO 30 MXL=0,K
      MCARL=MXL*MXL+MCARK
      KQQM=MCARL*KRAP
      IF((KQQM-KEPSQ(5)).GT.KQ(5))GO TO 20
      KQQP=MCARL*KRAM
      IF((KQQP+KEPSQ(1)).LT.KQ(1))GO TO 30
      DO 40 I=1,5
      IF((KQQM-KEPSQ(I)).GT.KQ(I))GO TO 40
      IF((KQQP+KEPSQ(I)).LT.KQ(I))GO TO 40
      IRJ(I,1)=IRJ(I,1)+1
      JJ=IRJ(I,1)
      IH(I,JJ,1)=M
      IK(I,JJ,1)=K
      IL(I,JJ,1)=MXL
   40 CONTINUE
   30 CONTINUE
   20 CONTINUE
   10 CONTINUE
      DO 70 I=1,5
      IF(IRJ(I,1).NE.0)GO TO 70
      NT=-1
      GO TO 3000
   70 CONTINUE
      DO 80 M=1,MH2
      MCARH=M*M
      DO 90 K=0,M
      MCARK=K*K+MCARH
      DO 100 MXL=0,K
      MCARL=MXL*MXL+MCARK
      KQQP=MCARL*KRAM
      IF((KQQP+KEPSQ(6)).LT.KQ(6))GO TO 100
      KQQM=MCARL*KRAP
      IF((KQQM-KEPSQ(N)).GT.KQ(N))GO TO 90
      DO 110 I=6,N
      IF((KQQM-KEPSQ(I)).GT.KQ(I))GO TO 110
      IF((KQQP+KEPSQ(I)).LT.KQ(I))GO TO 110
      IRJ(I,1)=IRJ(I,1)+1
      JJ=IRJ(I,1)
      IH(I,JJ,1)=M
      IK(I,JJ,1)=K
      IL(I,JJ,1)=MXL
  110 CONTINUE
  100 CONTINUE
   90 CONTINUE
   80 CONTINUE
 3000 RETURN
      END
      SUBROUTINE CUDHKL1(KRAM,KRAP,K1)
      COMMON D(50),Q(50),KQ(50),TH(50),EPSIL(50),EPSQ(50),KEPSQ(50),DTH,
     1       V,VINF,VSUP,PAS,PAS2,PAS4,PAS8,PAS16,PAS32,PAS64,COEFF,RAP,
     2       WAVE2,PIDEG,PIRAD,N,MH,MH2,MK,MK2,ML,ML2,KDENS,KZ,KQT(50)
      COMMON /BAF/NT,IRJ(50,8),IH(50,50,8),IK(50,50,8),IL(50,100,8)
     1           ,NPOSS,FWOLFF
      NT=0
      KK=K1-1
      DO 5 I=1,N
      JJ=IRJ(I,KK)
      IRJ(I,K1)=0
      DO 10 J=1,JJ
      M=IH(I,J,KK)
      K=IK(I,J,KK)
      MXL=IL(I,J,KK)
      MCARL=M*M+K*K+MXL*MXL
      KQQM=MCARL*KRAP
      KQQP=MCARL*KRAM
      IF((KQQM-KEPSQ(I)).GT.KQ(I))GO TO 10
      IF((KQQP+KEPSQ(I)).LT.KQ(I))GO TO 10
      IRJ(I,K1)=IRJ(I,K1)+1
      KJ=IRJ(I,K1)
      IH(I,KJ,K1)=M
      IK(I,KJ,K1)=K
      IL(I,KJ,K1)=MXL
  10  CONTINUE
      IF(IRJ(I,K1).NE.0)GO TO 5
      NT=-1
      GO TO 55
  5   CONTINUE
 55   RETURN
      END
C     ------------------------------------------------------------------
C     |          T E T R A G O N A L   E T   H E X A G O N A L         |
C     ------------------------------------------------------------------
      SUBROUTINE TETHEX(CMIN,NC,ICHOIX)
      COMMON D(50),Q(50),KQ(50),TH(50),EPSIL(50),EPSQ(50),KEPSQ(50),DTH,
     1       V,VINF,VSUP,PAS,PAS2,PAS4,PAS8,PAS16,PAS32,PAS64,COEFF,RAP,
     2       WAVE2,PIDEG,PIRAD,N,MH,MH2,MK,MK2,ML,ML2,KDENS,KZ,KQT(50)
      COMMON QCAL(50,20),AA,BB,CC,BETA,AR,BR,CR,DR,ER,FR
      COMMON /BAF/NT,IRJ(50,8),IH(50,50,8),IK(50,50,8),IL(50,100,8)
     1           ,NPOSS,FWOLFF
      DIMENSION NDICH(7)
      DATA IR,IW/15,16/
    1 FORMAT(1H ,10X,2HA=,F8.5,10X,2HC=,F8.5,10X,33HVOLUME DE LA MAILLE
     1ELEMENTAIRE =,F10.3)
    2 FORMAT(1H ,10X,F8.5,4X,3(10X,F3.0),9X,F8.5,11X,F8.5,9X,F8.5)
    3 FORMAT(1H ,50X,9(1H-)/)
    4 FORMAT(1H ,10X,2HA=,F8.5,10X,2HC=,F8.5,10X,33HVOLUME DE LA MAILLE
     1ELEMENTAIRE =,F10.3,5X,2HZ=,I2)
    5 FORMAT(1H ,7X,14HD EXPERIMENTAL,12X,1HH,12X,1HK,12X,1HL,10X,9HD CA
     1LCULE,12X,5HDE-DC,12X,7HEPSILON/1H )
      DO 20 I=1,7
   20 NDICH(I)=0
      AA=1000.
      CC=1000.
      RA=4./3.
      CONST=COEFF
      CONTR=D(1)-EPSIL(1)
       CONTR1=CONTR
      COVOL=1.
      IF(ICHOIX)79,79,78
   78 CONST=RA*COEFF
      COVOL=0.8660254
      CONTR=CONTR/COVOL
   79 DO 1000 IA=1,NC
      AW=IA-1
      AMOI1=CMIN+(AW*PAS)
      APLU1=AMOI1+PAS
      MH=(APLU1/D(5))
      MH2=(APLU1/D(N))
      KRAM1=CONST/(AMOI1*AMOI1)
      KRAP1=CONST/(APLU1*APLU1)
      DO 1100 IC=1,NC
      CW=IC-1
      CMOI1=CMIN+(CW*PAS)
      CPLU1=CMOI1+PAS
      IF(APLU1.LT.CONTR.AND.CPLU1.LT.CONTR1)GO TO 1100
      VTESTM=AMOI1*AMOI1*CMOI1*COVOL
      IF(V.LT.VTESTM)GO TO 1000
      IF(VSUP.LT.VTESTM)GO TO 1000
      VTESTP=APLU1*APLU1*CPLU1*COVOL
      IF(VINF.GT.VTESTP)GO TO 1100
      ML=(CPLU1/D(5))
      ML2=(CPLU1/D(N))
      KRCM1=COEFF/(CMOI1*CMOI1)
      KRCP1=COEFF/(CPLU1*CPLU1)
      CALL TEDHKL(KRAM1,KRAP1,KRCM1,KRCP1,ICHOIX)
      IF(NT.EQ.-1)GO TO 1100
      DO 120 I=6,N
      IF(IRJ(I,1).EQ.0)GO TO 1100
  120 CONTINUE
      NDICH(1)=NDICH(1)+1
      DO 2000 IA2=1,2
      AW=IA2-1
      AMOI2=AMOI1+(AW*PAS2)
      APLU2=AMOI2+PAS2
      KRAM2=CONST/(AMOI2*AMOI2)
      KRAP2=CONST/(APLU2*APLU2)
      DO 2100 IC2=1,2
      CW=IC2-1
      CMOI2=CMOI1+(CW*PAS2)
      ATEST=ABS(AMOI2-AA)
      CTEST=ABS(CMOI2-CC)
      IF(ATEST.LT.0.25.AND.CTEST.LT.0.25)GO TO 2100
      VTESTM=AMOI2*AMOI2*CMOI2*COVOL
      IF(VSUP.LT.VTESTM)GO TO 2000
      CPLU2=CMOI2+PAS2
      VTESTP=APLU2*APLU2*CPLU2*COVOL
      IF(VINF.GT.VTESTP)GO TO 2100
      KRCM2=COEFF/(CMOI2*CMOI2)
      KRCP2=COEFF/(CPLU2*CPLU2)
      CALL TEDHKL1(KRAM2,KRAP2,KRCM2,KRCP2,ICHOIX,2)
      IF(NT.EQ.-1)GO TO 2100
      NDICH(2)=NDICH(2)+1
      DO 3000 IA3=1,2
      AW=IA3-1
      AMOI3=AMOI2+(AW*PAS4)
      APLU3=AMOI3+PAS4
      KRAM3=CONST/(AMOI3*AMOI3)
      KRAP3=CONST/(APLU3*APLU3)
      DO 3100 IC3=1,2
      CW=IC3-1
      CMOI3=CMOI2+(CW*PAS4)
      ATEST=ABS(AMOI3-AA)
      CTEST=ABS(CMOI3-CC)
      IF(ATEST.LT.0.25.AND.CTEST.LT.0.25)GO TO 3100
      VTESTM=AMOI3*AMOI3*CMOI3*COVOL
      IF(VSUP.LT.VTESTM)GO TO 3000
      CPLU3=CMOI3+PAS4
      VTESTP=APLU3*APLU3*CPLU3*COVOL
      IF(VINF.GT.VTESTP)GO TO 3100
      KRCM3=COEFF/(CMOI3*CMOI3)
      KRCP3=COEFF/(CPLU3*CPLU3)
      CALL TEDHKL1(KRAM3,KRAP3,KRCM3,KRCP3,ICHOIX,3)
      IF(NT.EQ.-1)GO TO 3100
      NDICH(3)=NDICH(3)+1
      DO 4000 IA4=1,2
      AW=IA4-1
      AMOI4=AMOI3+(AW*PAS8)
      APLU4=AMOI4+PAS8
      KRAM4=CONST/(AMOI4*AMOI4)
      KRAP4=CONST/(APLU4*APLU4)
      DO 4100 IC4=1,2
      CW=IC4-1
      CMOI4=CMOI3+(CW*PAS8)
      ATEST=ABS(AMOI4-AA)
      CTEST=ABS(CMOI4-CC)
      IF(ATEST.LT.0.25.AND.CTEST.LT.0.25)GO TO 4100
      VTESTM=AMOI4*AMOI4*CMOI4*COVOL
      IF(VSUP.LT.VTESTM)GO TO 4000
      CPLU4=CMOI4+PAS8
      VTESTP=APLU4*APLU4*CPLU4*COVOL
      IF(VINF.GT.VTESTP)GO TO 4100
      KRCM4=COEFF/(CMOI4*CMOI4)
      KRCP4=COEFF/(CPLU4*CPLU4)
      CALL TEDHKL1(KRAM4,KRAP4,KRCM4,KRCP4,ICHOIX,4)
      IF(NT.EQ.-1)GO TO 4100
      NDICH(4)=NDICH(4)+1
      DO 5000 IA5=1,2
      AW=IA5-1
      AMOI5=AMOI4+(AW*PAS16)
      APLU5=AMOI5+PAS16
      KRAM5=CONST/(AMOI5*AMOI5)
      KRAP5=CONST/(APLU5*APLU5)
      DO 5100 IC5=1,2
      CW=IC5-1
      CMOI5=CMOI4+(CW*PAS16)
      ATEST=ABS(AMOI5-AA)
      CTEST=ABS(CMOI5-CC)
      IF(ATEST.LT.0.25.AND.CTEST.LT.0.25)GO TO 5100
      VTESTM=AMOI5*AMOI5*CMOI5*COVOL
      IF(VSUP.LT.VTESTM)GO TO 5000
      CPLU5=CMOI5+PAS16
      VTESTP=APLU5*APLU5*CPLU5*COVOL
      IF(VINF.GT.VTESTP)GO TO 5100
      KRCM5=COEFF/(CMOI5*CMOI5)
      KRCP5=COEFF/(CPLU5*CPLU5)
      CALL TEDHKL1(KRAM5,KRAP5,KRCM5,KRCP5,ICHOIX,5)
      IF(NT.EQ.-1)GO TO 5100
      NDICH(5)=NDICH(5)+1
      DO 6000 IA6=1,2
      AW=IA6-1
      AMOI6=AMOI5+(AW*PAS32)
      APLU6=AMOI6+PAS32
      KRAM6=CONST/(AMOI6*AMOI6)
      KRAP6=CONST/(APLU6*APLU6)
      DO 6100 IC6=1,2
      CW=IC6-1
      CMOI6=CMOI5+(CW*PAS32)
      ATEST=ABS(AMOI6-AA)
      CTEST=ABS(CMOI6-CC)
      IF(ATEST.LT.0.25.AND.CTEST.LT.0.25)GO TO 6100
      VTESTM=AMOI6*AMOI6*CMOI6*COVOL
      IF(VSUP.LT.VTESTM)GO TO 6000
      CPLU6=CMOI6+PAS32
      VTESTP=APLU6*APLU6*CPLU6*COVOL
      IF(VINF.GT.VTESTP)GO TO 6100
      KRCM6=COEFF/(CMOI6*CMOI6)
      KRCP6=COEFF/(CPLU6*CPLU6)
      CALL TEDHKL1(KRAM6,KRAP6,KRCM6,KRCP6,ICHOIX,6)
      IF(NT.EQ.-1)GO TO 6100
      NDICH(6)=NDICH(6)+1
      DO 7000 IA7=1,2
      AW=IA7-1
      AMOI7=AMOI6+(AW*PAS64)
      APLU7=AMOI7+PAS64
      RAM7=1./(AMOI7*AMOI7)
      RAP7=1./(APLU7*APLU7)
      KRAM7=CONST*RAM7
      KRAP7=CONST*RAP7
      DO 7100 IC7=1,2
      CW=IC7-1
      CMOI7=CMOI6+(CW*PAS64)
      ATEST=ABS(AMOI7-AA)
      CTEST=ABS(CMOI7-CC)
      IF(ATEST.LT.0.25.AND.CTEST.LT.0.25)GO TO 7100
      VTESTM=AMOI7*AMOI7*CMOI7*COVOL
      IF(VSUP.LT.VTESTM)GO TO 7000
      CPLU7=CMOI7+PAS64
      VTESTP=APLU7*APLU7*CPLU7*COVOL
      IF(VINF.GT.VTESTP)GO TO 7100
      RCM7=1./(CMOI7*CMOI7)
      RCP7=1./(CPLU7*CPLU7)
      KRCM7=COEFF*RCM7
      KRCP7=COEFF*RCP7
      CALL TEDHKL1(KRAM7,KRAP7,KRCM7,KRCP7,ICHOIX,7)
      IF(NT.EQ.-1)GO TO 7100
      NDICH(7)=NDICH(7)+1
      AARE=AA
      CCRE=CC
      AA=AMOI7+(PAS64/2.)
      CC=CMOI7+(PAS64/2.)
      VAP=AA*AA*CC
      IF(ICHOIX.EQ.1)VAP=VAP*SQRT(3.)/2.
      DO 15 I=1,N
      DO 16 J=1,20
   16 QCAL(I,J)=-1.
   15 CONTINUE
      SAM=0.
      IF(ICHOIX.EQ.1)SAM=1.
      DO 45 I=1,N
      JJ=IRJ(I,7)
      DO 35 J=1,JJ
      NPHK=SAM*IH(I,J,7)*IK(I,J,7)
      QCAL(I,J)=(IH(I,J,7)**2+IK(I,J,7)**2+NPHK)/AA**2+(IL(I,J,7)/CC)**2
   35 CONTINUE
   45 CONTINUE
      IND=2
      NRIND=2
      IF(ICHOIX.GT.0)IND=3
      CALL AFFPAR(IND,NRIND,VAP)
      IF(FWOLFF.NE.-1000.)GO TO 7113
      AA=AARE
      CC=CCRE
      GO TO 7100
 7113 WRITE(IW,3)
 7100 CONTINUE
 7000 CONTINUE
 6100 CONTINUE
 6000 CONTINUE
 5100 CONTINUE
 5000 CONTINUE
 4100 CONTINUE
 4000 CONTINUE
 3100 CONTINUE
 3000 CONTINUE
 2100 CONTINUE
 2000 CONTINUE
 1100 CONTINUE
 1000 CONTINUE
      WRITE(IW,10)(NDICH(I),I=1,7)
   10 FORMAT(/,10X,'ITERATION NUMBER AT EACH DICHOTOMY LEVEL :',
     S7X,7(3X,I5))
      RETURN
      END
      SUBROUTINE TEDHKL(KRAM,KRAP,KRCM,KRCP,ICHOIX)
      COMMON D(50),Q(50),KQ(50),TH(50),EPSIL(50),EPSQ(50),KEPSQ(50),DTH,
     1       V,VINF,VSUP,PAS,PAS2,PAS4,PAS8,PAS16,PAS32,PAS64,COEFF,RAP,
     2       WAVE2,PIDEG,PIRAD,N,MH,MH2,MK,MK2,ML,ML2,KDENS,KZ,KQT(50)
      COMMON /BAF/NT,IRJ(50,8),IH(50,50,8),IK(50,50,8),IL(50,100,8)
     1           ,NPOSS,FWOLFF
      NT=0
      DO 15 I=1,N
      IRJ(I,1)=0
   15 CONTINUE
      DO 10 M=0,MH
      MCARH=M*M
      DO 20 K=0,M
      MCARK=MCARH+K*K
      DO 30 MXL=0,ML
      MCARL=MXL*MXL
      IF(ICHOIX)200,200,201
  200 KQQM=MCARK*KRAP+MCARL*KRCP
      KQQP=MCARK*KRAM+MCARL*KRCM
      GO TO 202
  201 KSOM=MCARK+M*K
      KQQM=KSOM*KRAP+MCARL*KRCP
      KQQP=KSOM*KRAM+MCARL*KRCM
  202 IF((KQQM-KEPSQ(5)).GT.KQ(5))GO TO 20
      IF((KQQP+KEPSQ(1)).LT.KQ(1))GO TO 30
      DO 40 I=1,5
      IF((KQQM-KEPSQ(I)).GT.KQ(I))GO TO 40
      IF((KQQP+KEPSQ(I)).LT.KQ(I))GO TO 40
      IRJ(I,1)=IRJ(I,1)+1
      JJ=IRJ(I,1)
      IH(I,JJ,1)=M
      IK(I,JJ,1)=K
      IL(I,JJ,1)=MXL
   40 CONTINUE
   30 CONTINUE
   20 CONTINUE
   10 CONTINUE
      DO 70 I=1,5
      IF(IRJ(I,1).NE.0)GO TO 70
      NT=-1
      GO TO 3000
   70 CONTINUE
      DO 80 M=0,MH2
      MCARH=M*M
      DO 90 K=0,M
      MCARK=MCARH+K*K
      DO 100 MXL=0,ML2
      MCARL=MXL*MXL
      IF(ICHOIX)300,300,301
  300 KQQM=(MCARK*KRAP)+(MCARL*KRCP)
      KQQP=(MCARK*KRAM)+(MCARL*KRCM)
      GO TO 302
  301 KSOM=MCARK+M*K
      KQQM=(KSOM*KRAP)+(MCARL*KRCP)
      KQQP=(KSOM*KRAM)+(MCARL*KRCM)
  302 IF((KQQM-KEPSQ(N)).GT.KQ(N))GO TO 90
      IF((KQQP+KEPSQ(6)).LT.KQ(6))GO TO 100
      DO 110 I=6,N
      IF((KQQM-KEPSQ(I)).GT.KQ(I))GO TO 110
      IF((KQQP+KEPSQ(I)).LT.KQ(I))GO TO 110
      IRJ(I,1)=IRJ(I,1)+1
      JJ=IRJ(I,1)
      IH(I,JJ,1)=M
      IK(I,JJ,1)=K
      IL(I,JJ,1)=MXL
  110 CONTINUE
  100 CONTINUE
   90 CONTINUE
   80 CONTINUE
 3000 RETURN
      END
      SUBROUTINE TEDHKL1(KRAM,KRAP,KRCM,KRCP,ICHOIX,K1)
      COMMON D(50),Q(50),KQ(50),TH(50),EPSIL(50),EPSQ(50),KEPSQ(50),DTH,
     1       V,VINF,VSUP,PAS,PAS2,PAS4,PAS8,PAS16,PAS32,PAS64,COEFF,RAP,
     2       WAVE2,PIDEG,PIRAD,N,MH,MH2,MK,MK2,ML,ML2,KDENS,KZ,KQT(50)
      COMMON /BAF/NT,IRJ(50,8),IH(50,50,8),IK(50,50,8),IL(50,100,8)
     1           ,NPOSS,FWOLFF
      NT=0
      KK=K1-1
      DO 5 I=1,N
      JJ=IRJ(I,KK)
      IRJ(I,K1)=0
      DO 10 J=1,JJ
      M=IH(I,J,KK)
      K=IK(I,J,KK)
      MXL=IL(I,J,KK)
      MCARK=M*M+K*K
      MCARL=MXL*MXL
      IF(ICHOIX)200,200,201
 200  KQQM=(MCARK*KRAP)+(MCARL*KRCP)
      KQQP=(MCARK*KRAM)+(MCARL*KRCM)
      GO TO 202
 201  KSOM=MCARK+M*K
      KQQM=KSOM*KRAP+MCARL*KRCP
      KQQP=KSOM*KRAM+MCARL*KRCM
 202  IF((KQQM-KEPSQ(I)).GT.KQ(I))GO TO 10
      IF((KQQP+KEPSQ(I)).LT.KQ(I))GO TO 10
      IRJ(I,K1)=IRJ(I,K1)+1
      KJ=IRJ(I,K1)
      IH(I,KJ,K1)=M
      IK(I,KJ,K1)=K
      IL(I,KJ,K1)=MXL
  10  CONTINUE
      IF(IRJ(I,K1).NE.0)GO TO 5
      NT=-1
      GO TO 55
   5  CONTINUE
  55  RETURN
      END
C     ------------------------------------------------------------------
C     |                   O R T H O R H O M B I Q U E                  |
C     ------------------------------------------------------------------
      SUBROUTINE ORTHOR(AMIN,BMIN,CMIN,NA,NB,NC)
      COMMON D(50),Q(50),KQ(50),TH(50),EPSIL(50),EPSQ(50),KEPSQ(50),DTH,
     1       V,VINF,VSUP,PAS,PAS2,PAS4,PAS8,PAS16,PAS32,PAS64,COEFF,RAP,
     2       WAVE2,PIDEG,PIRAD,N,MH,MH2,MK,MK2,ML,ML2,KDENS,KZ,KQT(50)
      COMMON QCAL(50,20),AA,BB,CC,BETA,AR,BR,CR,DR,ER,FR
      COMMON /BAF/NT,IRJ(50,8),IH(50,50,8),IK(50,50,8),IL(50,100,8)
     1           ,NPOSS,FWOLFF
      DIMENSION NDICH(7)
      DATA IR,IW/15,16/
    1 FORMAT(1H ,10X,2HA=,F8.5,10X,2HB=,F8.5,10X,2HC=,F8.5,10X,33HVOLUME
     1 DE LA MAILLE ELEMENTAIRE =,F10.3)
    2 FORMAT(1H ,10X,F8.5,4X,3(10X,F3.0),9X,F8.5,11X,F8.5,9X,F8.5)
    3 FORMAT(1H ,50X,9(1H-)/)
    4 FORMAT(1H ,10X,2HA=,F8.5,10X,2HB=,F8.5,10X,2HC=,F8.5,10X,33HVOLUME
     1 DE LA MAILLE ELEMENTAIRE =,F10.3,5X,2HZ=,I2)
    5 FORMAT(1H ,7X,14HD EXPERIMENTAL,12X,1HH,12X,1HK,12X,1HL,10X,9HD CA
     1LCULE,12X,5HDE-DC,12X,7HEPSILON/1H )
      DO 20 I=1,7
   20 NDICH(I)=0
      AA=1000.
      BB=1000.
      CC=1000.
      DO 1000 IA=1,NA
      AW=IA-1
      AMOI1=AMIN+(AW*PAS)
      APLU1=AMOI1+PAS
      MH=(APLU1/D(5))
      MH2=(APLU1/D(N))
      KRAM1=COEFF/(AMOI1*AMOI1)
      KRAP1=COEFF/(APLU1*APLU1)
      DO 2000 IB=1,NB
      BW=IB-1
      BMOI1=BMIN+(BW*PAS)
      IF(APLU1-BMOI1)1000,800,800
  800 BPLU1=BMOI1+PAS
      MK=(BPLU1/D(5))
      MK2=(BPLU1/D(N))
      KRBM1=COEFF/(BMOI1*BMOI1)
      KRBP1=COEFF/(BPLU1*BPLU1)
      DO 2500 IC=1,NC
      CW=IC-1
      CMOI1=CMIN+(CW*PAS)
      CPLU1=CMOI1+PAS
      IF(BPLU1-CMOI1)2000,801,801
  801 IF(CPLU1.LE.CMIN)GO TO 2500
      VTESTM=AMOI1*BMOI1*CMOI1
      IF(V.LT.VTESTM)GO TO 2000
      IF(VSUP.LT.VTESTM)GO TO 2000
      VTESTP=APLU1*BPLU1*CPLU1
      IF(VINF.GT.VTESTP)GO TO 2500
      ML=(CPLU1/D(5))
      ML2=(CPLU1/D(N))
      KRCM1=COEFF/(CMOI1*CMOI1)
      KRCP1=COEFF/(CPLU1*CPLU1)
      CALL ORDHKL(KRAM1,KRAP1,KRBM1,KRBP1,KRCM1,KRCP1)
      IF(NT.EQ.-1)GO TO 2500
      DO 120 I=6,N
      IF(IRJ(I,1).EQ.0)GO TO 2500
  120 CONTINUE
      NDICH(1)=NDICH(1)+1
      DO 3000 IA2=1,2
      AW=IA2-1
      AMOI2=AMOI1+(AW*PAS2)
      APLU2=AMOI2+PAS2
      KRAM2=COEFF/(AMOI2*AMOI2)
      KRAP2=COEFF/(APLU2*APLU2)
      DO 3100 IB2=1,2
      BW=IB2-1
      BMOI2=BMOI1+(BW*PAS2)
      BPLU2=BMOI2+PAS2
      KRBM2=COEFF/(BMOI2*BMOI2)
      KRBP2=COEFF/(BPLU2*BPLU2)
      DO 3200 IC2=1,2
      CW=IC2-1
      CMOI2=CMOI1+(CW*PAS2)
      ATEST=ABS(AMOI2-AA)
      BTEST=ABS(BMOI2-BB)
      CTEST=ABS(CMOI2-CC)
      IF(ATEST.LT.0.25.AND.BTEST.LT.0.25.AND.CTEST.LT.0.25) GO TO 3200
      CPLU2=CMOI2+PAS2
      VTESTM=AMOI2*BMOI2*CMOI2
      IF(VSUP.LT.VTESTM)GO TO 3100
      VTESTP=APLU2*BPLU2*CPLU2
      IF(VINF.GT.VTESTP)GO TO 3200
      KRCM2=COEFF/(CMOI2*CMOI2)
      KRCP2=COEFF/(CPLU2*CPLU2)
      CALL ORDHKL1(KRAM2,KRAP2,KRBM2,KRBP2,KRCM2,KRCP2,2)
      IF(NT.EQ.-1)GO TO 3200
      NDICH(2)=NDICH(2)+1
      DO 4000 IA3=1,2
      AW=IA3-1
      AMOI3=AMOI2+(AW*PAS4)
      APLU3=AMOI3+PAS4
      KRAM3=COEFF/(AMOI3*AMOI3)
      KRAP3=COEFF/(APLU3*APLU3)
      DO 4100 IB3=1,2
      BW=IB3-1
      BMOI3=BMOI2+(BW*PAS4)
      BPLU3=BMOI3+PAS4
      KRBM3=COEFF/(BMOI3*BMOI3)
      KRBP3=COEFF/(BPLU3*BPLU3)
      DO 4200 IC3=1,2
      CW=IC3-1
      CMOI3=CMOI2+(CW*PAS4)
      ATEST=ABS(AMOI3-AA)
      BTEST=ABS(BMOI3-BB)
      CTEST=ABS(CMOI3-CC)
      IF(ATEST.LT.0.25.AND.BTEST.LT.0.25.AND.CTEST.LT.0.25) GO TO 4200
      CPLU3=CMOI3+PAS4
      VTESTM=AMOI3*BMOI3*CMOI3
      IF(VSUP.LT.VTESTM)GO TO 4100
      VTESTP=APLU3*BPLU3*CPLU3
      IF(VINF.GT.VTESTP)GO TO 4200
      KRCM3=COEFF/(CMOI3*CMOI3)
      KRCP3=COEFF/(CPLU3*CPLU3)
      CALL ORDHKL1(KRAM3,KRAP3,KRBM3,KRBP3,KRCM3,KRCP3,3)
      IF(NT.EQ.-1)GO TO 4200
      NDICH(3)=NDICH(3)+1
      DO 5000 IA4=1,2
      AW=IA4-1
      AMOI4=AMOI3+(AW*PAS8)
      APLU4=AMOI4+PAS8
      KRAM4=COEFF/(AMOI4*AMOI4)
      KRAP4=COEFF/(APLU4*APLU4)
      DO 5100 IB4=1,2
      BW=IB4-1
      BMOI4=BMOI3+(BW*PAS8)
      BPLU4=BMOI4+PAS8
      KRBM4=COEFF/(BMOI4*BMOI4)
      KRBP4=COEFF/(BPLU4*BPLU4)
      DO 5200 IC4=1,2
      CW=IC4-1
      CMOI4=CMOI3+(CW*PAS8)
      ATEST=ABS(AMOI4-AA)
      BTEST=ABS(BMOI4-BB)
      CTEST=ABS(CMOI4-CC)
      IF(ATEST.LT.0.25.AND.BTEST.LT.0.25.AND.CTEST.LT.0.25) GO TO 5200
      CPLU4=CMOI4+PAS8
      VTESTM=AMOI4*BMOI4*CMOI4
      IF(VSUP.LT.VTESTM)GO TO 5100
      VTESTP=APLU4*BPLU4*CPLU4
      IF(VINF.GT.VTESTP)GO TO 5200
      KRCM4=COEFF/(CMOI4*CMOI4)
      KRCP4=COEFF/(CPLU4*CPLU4)
      CALL ORDHKL1(KRAM4,KRAP4,KRBM4,KRBP4,KRCM4,KRCP4,4)
      IF(NT.EQ.-1)GO TO 5200
      NDICH(4)=NDICH(4)+1
      DO 6000 IA5=1,2
      AW=IA5-1
      AMOI5=AMOI4+(AW*PAS16)
      APLU5=AMOI5+PAS16
      KRAM5=COEFF/(AMOI5*AMOI5)
      KRAP5=COEFF/(APLU5*APLU5)
      DO 6100 IB5=1,2
      BW=IB5-1
      BMOI5=BMOI4+(BW*PAS16)
      IF(BMOI5.GT.APLU5)GO TO 6000
      BPLU5=BMOI5+PAS16
      KRBM5=COEFF/(BMOI5*BMOI5)
      KRBP5=COEFF/(BPLU5*BPLU5)
      DO 6200 IC5=1,2
      CW=IC5-1
      CMOI5=CMOI4+(CW*PAS16)
      IF(CMOI5.GT.BPLU5)GO TO 6100
      ATEST=ABS(AMOI5-AA)
      BTEST=ABS(BMOI5-BB)
      CTEST=ABS(CMOI5-CC)
      IF(ATEST.LT.0.25.AND.BTEST.LT.0.25.AND.CTEST.LT.0.25) GO TO 6200
      CPLU5=CMOI5+PAS16
      VTESTM=AMOI5*BMOI5*CMOI5
      IF(VSUP.LT.VTESTM)GO TO 6100
      VTESTP=APLU5*BPLU5*CPLU5
      IF(VINF.GT.VTESTP)GO TO 6200
      KRCM5=COEFF/(CMOI5*CMOI5)
      KRCP5=COEFF/(CPLU5*CPLU5)
      CALL ORDHKL1(KRAM5,KRAP5,KRBM5,KRBP5,KRCM5,KRCP5,5)
      IF(NT.EQ.-1)GO TO 6200
      NDICH(5)=NDICH(5)+1
      DO 7000 IA6=1,2
      AW=IA6-1
      AMOI6=AMOI5+(AW*PAS32)
      APLU6=AMOI6+PAS32
      KRAM6=COEFF/(AMOI6*AMOI6)
      KRAP6=COEFF/(APLU6*APLU6)
      DO 7100 IB6=1,2
      BW=IB6-1
      BMOI6=BMOI5+(BW*PAS32)
      BPLU6=BMOI6+PAS32
      KRBM6=COEFF/(BMOI6*BMOI6)
      KRBP6=COEFF/(BPLU6*BPLU6)
      DO 7200 IC6=1,2
      CW=IC6-1
      CMOI6=CMOI5+(CW*PAS32)
      ATEST=ABS(AMOI6-AA)
      BTEST=ABS(BMOI6-BB)
      CTEST=ABS(CMOI6-CC)
      IF(ATEST.LT.0.25.AND.BTEST.LT.0.25.AND.CTEST.LT.0.25) GO TO 7200
      CPLU6=CMOI6+PAS32
      VTESTM=AMOI6*BMOI6*CMOI6
      IF(VSUP.LT.VTESTM)GO TO 7100
      VTESTP=APLU6*BPLU6*CPLU6
      IF(VINF.GT.VTESTP)GO TO 7200
      KRCM6=COEFF/(CMOI6*CMOI6)
      KRCP6=COEFF/(CPLU6*CPLU6)
      CALL ORDHKL1(KRAM6,KRAP6,KRBM6,KRBP6,KRCM6,KRCP6,6)
      IF(NT.EQ.-1)GO TO 7200
      NDICH(6)=NDICH(6)+1
      DO 8000 IA7=1,2
      AW=IA7-1
      AMOI7=AMOI6+(AW*PAS64)
      APLU7=AMOI7+PAS64
      RAM7=1./(AMOI7*AMOI7)
      RAP7=1./(APLU7*APLU7)
      KRAM7=COEFF*RAM7
      KRAP7=COEFF*RAP7
      DO 8100 IB7=1,2
      BW=IB7-1
      BMOI7=BMOI6+(BW*PAS64)
      BPLU7=BMOI7+PAS64
      RBM7=1./(BMOI7*BMOI7)
      RBP7=1./(BPLU7*BPLU7)
      KRBM7=COEFF*RBM7
      KRBP7=COEFF*RBP7
      DO 8200 IC7=1,2
      CW=IC7-1
      CMOI7=CMOI6+(CW*PAS64)
      ATEST=ABS(AMOI7-AA)
      BTEST=ABS(BMOI7-BB)
      CTEST=ABS(CMOI7-CC)
      IF(ATEST.LT.0.25.AND.BTEST.LT.0.25.AND.CTEST.LT.0.25)GO TO 8200
      CPLU7=CMOI7+PAS64
      VTESTM=AMOI7*BMOI7*CMOI7
      IF(VSUP.LT.VTESTM)GO TO 8100
      VTESTP=APLU7*BPLU7*CPLU7
      IF(VINF.GT.VTESTP)GO TO 8200
      RCM7=1./(CMOI7*CMOI7)
      RCP7=1./(CPLU7*CPLU7)
      KRCM7=COEFF*RCM7
      KRCP7=COEFF*RCP7
      CALL ORDHKL1(KRAM7,KRAP7,KRBM7,KRBP7,KRCM7,KRCP7,7)
      IF(NT.EQ.-1)GO TO 8200
      NDICH(7)=NDICH(7)+1
      AARE=AA
      BBRE=BB
      CCRE=CC
      AA=AMOI7+(PAS64/2.)
      BB=BMOI7+(PAS64/2.)
      CC=CMOI7+(PAS64/2.)
      VAP=AA*BB*CC
      DO 15 I=1,N
      DO 16 J=1,20
   16 QCAL(I,J)=-1.
   15 CONTINUE
C
C
      DO 45 I=1,N
      JJ=IRJ(I,7)
      DO 35 J=1,JJ
      QCAL(I,J)=(IH(I,J,7)/AA)**2+(IK(I,J,7)/BB)**2+(IL(I,J,7)/CC)**2
  35  CONTINUE
  45  CONTINUE
      IND=4
      NRIND=3
      CALL AFFPAR(IND,NRIND,VAP)
      IF(FWOLFF.NE.-1000.)GO TO 8213
      AA=AARE
      BB=BBRE
      CC=CCRE
      GO TO 8200
 8213 WRITE(IW,3)
 8200 CONTINUE
 8100 CONTINUE
 8000 CONTINUE
 7200 CONTINUE
 7100 CONTINUE
 7000 CONTINUE
 6200 CONTINUE
 6100 CONTINUE
 6000 CONTINUE
 5200 CONTINUE
 5100 CONTINUE
 5000 CONTINUE
 4200 CONTINUE
 4100 CONTINUE
 4000 CONTINUE
 3200 CONTINUE
 3100 CONTINUE
 3000 CONTINUE
 2500 CONTINUE
 2000 CONTINUE
 1000 CONTINUE
      WRITE(IW,10)(NDICH(I),I=1,7)
   10 FORMAT(/,10X,'ITERATION NUMBER AT EACH DICHOTOMY LEVEL :',
     S7X,7(3X,I5))
      RETURN
      END
      SUBROUTINE ORDHKL(KRAM,KRAP,KRBM,KRBP,KRCM,KRCP)
      COMMON D(50),Q(50),KQ(50),TH(50),EPSIL(50),EPSQ(50),KEPSQ(50),DTH,
     1       V,VINF,VSUP,PAS,PAS2,PAS4,PAS8,PAS16,PAS32,PAS64,COEFF,RAP,
     2       WAVE2,PIDEG,PIRAD,N,MH,MH2,MK,MK2,ML,ML2,KDENS,KZ,KQT(50)
      COMMON /BAF/NT,IRJ(50,8),IH(50,50,8),IK(50,50,8),IL(50,100,8)
     1           ,NPOSS,FWOLFF
      NT=0
      DO 15 I=1,N
      IRJ(I,1)=0
   15 CONTINUE
      DO 10 M=0,MH
      MCARH=M*M
      M1=MCARH*KRAM
      MP1=MCARH*KRAP
      DO 20 K=0,MK
      MCARK=K*K
      M2=(MCARK*KRBM)+M1
      MP2=(MCARK*KRBP)+MP1
      KBORIN=MP2-KEPSQ(5)
      IF(KBORIN.GT.KQ(5))GO TO 10
      DO 30 MXL=0,ML
      MCARL=MXL*MXL
      KQQM=(MCARL*KRCP)+MP2
      KBORIN=KQQM-KEPSQ(5)
      IF(KBORIN.GT.KQ(5))GO TO 20
      KQQP=(MCARL*KRCM)+M2
      KBORSU=KQQP+KEPSQ(1)
      IF(KQ(1).GT.KBORSU)GO TO 30
      DO 40 I=1,5
      KBORIN=KQQM-KEPSQ(I)
      IF(KBORIN.GT.KQ(I))GO TO 40
      KBORSU=KQQP+KEPSQ(I)
      IF(KBORSU.LT.KQ(I))GO TO 40
      IRJ(I,1)=IRJ(I,1)+1
      JJ=IRJ(I,1)
      IH(I,JJ,1)=M
      IK(I,JJ,1)=K
      IL(I,JJ,1)=MXL
   40 CONTINUE
   30 CONTINUE
   20 CONTINUE
   10 CONTINUE
      DO 70 I=1,5
      IF(IRJ(I,1).NE.0)GO TO 70
      NT=-1
      GO TO 3000
   70 CONTINUE
      DO 80 M=0,MH2
      MCARH=M*M
      M1=MCARH*KRAM
      MP1=MCARH*KRAP
      DO 90 K=0,MK2
      MCARK=K*K
      M2=(MCARK*KRBM)+M1
      MP2=(MCARK*KRBP)+MP1
      KBORIN=MP2-KEPSQ(N)
      IF(KBORIN.GT.KQ(N))GO TO 80
      DO 100 MXL=0,ML2
      MCARL=MXL*MXL
      KQQP=(MCARL*KRCM)+M2
      KBORSU=KQQP+KEPSQ(6)
      IF(KBORSU.LT.KQ(6))GO TO 100
      KQQM=(MCARL*KRCP)+MP2
      KBORIN=KQQM-KEPSQ(N)
      IF(KBORIN.GT.KQ(N))GO TO 90
      DO 110 I=6,N
      KBORIN=KQQM-KEPSQ(I)
      IF(KBORIN.GT.KQ(I))GO TO 110
      KBORSU=KQQP+KEPSQ(I)
      IF(KBORSU.LT.KQ(I))GO TO 110
      IRJ(I,1)=IRJ(I,1)+1
      JJ=IRJ(I,1)
      IH(I,JJ,1)=M
      IK(I,JJ,1)=K
      IL(I,JJ,1)=MXL
  110 CONTINUE
  100 CONTINUE
   90 CONTINUE
   80 CONTINUE
 3000 RETURN
      END
      SUBROUTINE ORDHKL1(KRAM,KRAP,KRBM,KRBP,KRCM,KRCP,K1)
      COMMON D(50),Q(50),KQ(50),TH(50),EPSIL(50),EPSQ(50),KEPSQ(50),DTH,
     1       V,VINF,VSUP,PAS,PAS2,PAS4,PAS8,PAS16,PAS32,PAS64,COEFF,RAP,
     2       WAVE2,PIDEG,PIRAD,N,MH,MH2,MK,MK2,ML,ML2,KDENS,KZ,KQT(50)
      COMMON /BAF/NT,IRJ(50,8),IH(50,50,8),IK(50,50,8),IL(50,100,8)
     1           ,NPOSS,FWOLFF
      NT=0
      KK=K1-1
      DO 5 I=1,N
      JJ=IRJ(I,KK)
      IRJ(I,K1)=0
      DO 10 J=1,JJ
      M=IH(I,J,KK)
      K=IK(I,J,KK)
      MXL=IL(I,J,KK)
      KQQP=MXL*MXL*KRCM+K*K*KRBM+M*M*KRAM
      KQQM=MXL*MXL*KRCP+K*K*KRBP+M*M*KRAP
      IF((KQQM-KEPSQ(I)).GT.KQ(I))GO TO 10
      IF((KQQP+KEPSQ(I)).LT.KQ(I))GO TO 10
      IRJ(I,K1)=IRJ(I,K1)+1
      KJ=IRJ(I,K1)
      IH(I,KJ,K1)=M
      IK(I,KJ,K1)=K
      IL(I,KJ,K1)=MXL
  10  CONTINUE
      IF(IRJ(I,K1).NE.0)GO TO 5
      NT=-1
      GO TO 55
  5   CONTINUE
 55   RETURN
      END
C     ------------------------------------------------------------------
C     |                    M O N O C L I N I Q U E                     |
C     ------------------------------------------------------------------
      SUBROUTINE MONOC(AMIN,NA,NB,CMIN,NC,BEMAX,BEMIN,BMIN)
      DIMENSION NDICH(7)
      COMMON D(50),Q(50),KQ(50),TH(50),EPSIL(50),EPSQ(50),KEPSQ(50),DTH,
     1       V,VINF,VSUP,PAS,PAS2,PAS4,PAS8,PAS16,PAS32,PAS64,COEFF,RAP,
     2       WAVE2,PIDEG,PIRAD,N,MH,MH2,MK,MK2,ML,ML2,KDENS,KZ,KQT(50)
      COMMON QCAL(50,20),AA,BB,CC,BETA,AR,BR,CR,DR,ER,FR
      COMMON /BAF/NT,IRJ(50,8),IH(50,50,8),IK(50,50,8),IL(50,100,8)
     1           ,NPOSS,FWOLFF
      COMMON /AMAX DD/AMAX,DD1,DD2
      COMMON /MONO/AMAXM,BMAXM,CMAXM,MC,PETIAMAX
      DATA IR,IW/15,16/
      F(NX,NY,NZ,DD,NE,NG)=NX+NY+NZ-DD*NE*NG
    3 FORMAT(1H ,50X,9(1H-)/)
    4 FORMAT(1H ,20X,21HANGLE RANGE SCANNED :,5X,13HBETA MINIMUM=,F10.
     12,6HDEGRES,5X,13HBETA MAXIMUM=,F10.2,6HDEGRES)
      DO 20 I=1,7
   20 NDICH(I)=0
      AA=100.
      BB=100.
      CC=100.
      BEBE=3.
      PASBE=PIRAD*5.00
      NBETA=((BEMAX-BEMIN)/PASBE)
      PAB2=PASBE/2.
      PAB4=PASBE/4.
      PAB8=PASBE/8.
      PAB16=PASBE/16.
      PAB32=PASBE/32.
      PAB64=PASBE/64.
      DO 1000 IBET1=0,NBETA
      BEINF1=BEMIN+(IBET1*PASBE)
      IF(BEINF1.GE.(BEMAX-1E-12))GO TO 1007
      BESUP1=BEINF1+PASBE
      BBETM=BEINF1*PIDEG
      BBETP=BESUP1*PIDEG
      WRITE(IW,4)BBETM,BBETP
      CSM1=COS(BEINF1)
      CSP1=COS(BESUP1)
      DO 1100 IA=0,NA
      AMOI1=AMIN+(IA*PAS)
      IF((AMOI1/SIN(BEINF1)).GE.PETIAMAX)GO TO 1000
      APLU1=AMOI1+PAS
      DO 21 I=1,10
      PP=I
      DA=APLU1/PP
      IF(D(5)-DA)21,22,22
   21 CONTINUE
   22 MH=I
      DO 55 J=I,20
      PP=J
      DA=APLU1/PP
      IF(D(N)-DA)55,51,51
   55 CONTINUE
   51 MH2=J
      KRAP1=COEFF/(AMOI1*AMOI1)
      KRAM1=COEFF/(APLU1*APLU1)
      DO 1200 IB=0,NB
      BMOI1=BMIN+(IB*PAS)
      BPLU1=BMOI1+PAS
      IF(APLU1.LT.DD1.AND.BPLU1.LT.DD1)GO TO 1200
      DO 23 I=1,10
      PP=I
      DB=BPLU1/PP
      IF(D(5)-DB)23,24,24
   23 CONTINUE
   24 MK=I
      DO 85 J=I,20
      PP=J
      DB=BPLU1/PP
      IF(D(N)-DB)85,61,61
   85 CONTINUE
   61 MK2=J
      KRBM1=COEFF/(BPLU1*BPLU1)
      KRBP1=COEFF/(BMOI1*BMOI1)
      DO 1300 IC=0,NC
      CMOI1=CMIN+(IC*PAS)
      IF(CMOI1.GE.APLU1)GO TO 1200
      ATEST=ABS(AMOI1-AA)
      BTEST=ABS(BMOI1-BB)
      CTEST=ABS(CMOI1-CC)
      BETEST=ABS(BEINF1-BEBE)
      IF(ATEST.LT.0.1.AND.BTEST.LT.0.1.AND.CTEST.LT.0.1.AND.BETEST.LT.0.
     +009)GO TO 1300
      CPLU1=CMOI1+PAS
      IF(MC.EQ.0.AND.APLU1.LE.AMAXM.AND.BPLU1.LE.BMAXM.AND.CPLU1.
     +LE.CMAXM)GO TO 1300
      IF(CPLU1.LE.DD2.AND.BPLU1.LE.DD2)
     1GO TO 1300
      VTESTM=AMOI1*BMOI1*CMOI1/SIN(BEINF1)
      IF(V.LT.VTESTM)GO TO 1200
      IF(VSUP.LT.VTESTM)GO TO 1200
      VTESTP=APLU1*BPLU1*CPLU1/SIN(BESUP1)
      IF(VINF.GT.VTESTP)GO TO 1300
      DO 25 I=1,10
      PP=I
      DC=CPLU1/PP
      IF(D(5)-DC)25,26,26
   25 CONTINUE
   26 ML=I
      DO 75 J=I,20
      PP=J
      DC=CPLU1/PP
      IF(D(N)-DC)75,71,71
   75 CONTINUE
   71 ML2=J
      KRCM=COEFF/(CPLU1*CPLU1)
      KRCP=COEFF/(CMOI1*CMOI1)
      KRACMS=2.*COEFF/(APLU1*CPLU1)
      KRACPS=2.*COEFF/(AMOI1*CMOI1)
      KRACM=2.*COEFF*CSM1/(APLU1*CPLU1)
      KRACP=2.*COEFF*CSP1/(AMOI1*CMOI1)
      KAMCP=2.*COEFF/(AMOI1*CPLU1)
      KAPCM=2.*COEFF/(APLU1*CMOI1)
40001 CALL MODHKL(KRAM1,KRAP1,KRCM,KRCP,KRACM,KRACP,KRBM1,KRBP1,
     1KAMCP,KAPCM,CSM1,CSP1,AMOI1,APLU1,CMOI1,CPLU1,KRACMS,KRACPS)
      IF(NT.EQ.-1)GO TO 1300
      DO 120 I=6,N
      IF(IRJ(I,1).EQ.0)GO TO 1300
  120 CONTINUE
      DO 30001 I=1,N-1
      J=N+1
      IF(IRJ(I,1).EQ.1.AND.IRJ(J,1).EQ.1.AND.IH(I,1,1).EQ.IH(J,1,1).AND.
     +IK(I,1,1).EQ.IK(J,1,1).AND.IL(I,1,1).EQ.IL(J,1,1))GO TO 1300
30001 CONTINUE
      NDICH(1)=NDICH(1)+1
      DO 2000 IBET2=0,1
      BEINF2=BEINF1+(IBET2*PAB2)
      BESUP2=BEINF2+PAB2
      CSM2=COS(BEINF2)
      CSP2=COS(BESUP2)
      DO 2100 IA2=0,1
      AMOI2=AMOI1+(IA2*PAS2)
      IF((AMOI2/SIN(BEINF2)).GE.PETIAMAX)GO TO 2000
C     IF(AMOI2.GE.AMAX)GO TO 2000
      APLU2=AMOI2+PAS2
      KRAM2=COEFF/(APLU2*APLU2)
      KRAP2=COEFF/(AMOI2*AMOI2)
      DO 2200 IB2=0,1
      BMOI2=BMOI1+(IB2*PAS2)
      BPLU2=BMOI2+PAS2
      KRBM2=COEFF/(BPLU2*BPLU2)
      KRBP2=COEFF/(BMOI2*BMOI2)
      DO 2300 IC2=0,1
      CMOI2=CMOI1+(IC2*PAS2)
      IF(CMOI2.GE.APLU2)GO TO 2200
      ATEST=ABS(AMOI2-AA)
      BTEST=ABS(BMOI2-BB)
      CTEST=ABS(CMOI2-CC)
      BETEST=ABS(BEINF2-BEBE)
      IF(ATEST.LT.0.1.AND.BTEST.LT.0.1.AND.CTEST.LT.0.1.AND.BETEST.LT.0.
     +009)GO TO 2300
      VTESTM=AMOI2*BMOI2*CMOI2/SIN(BEINF2)
      IF(VTESTM.GT.VSUP)GO TO 2200
      CPLU2=CMOI2+PAS2
      IF(MC.EQ.0.AND.APLU2.LE.AMAXM.AND.BPLU2.LE.BMAXM.AND.CPLU2.
     +LE.CMAXM)GO TO 2300
      VTESTP=APLU2*BPLU2*CPLU2/SIN(BESUP2)
      IF(VINF.GT.VTESTP)GO TO 2300
      KRCM=COEFF/(CPLU2*CPLU2)
      KRCP=COEFF/(CMOI2*CMOI2)
      KRACMS=2.*COEFF/(APLU2*CPLU2)
      KRACPS=2.*COEFF/(AMOI2*CMOI2)
      KRACM=2.*COEFF*CSM2/(APLU2*CPLU2)
      KRACP=2.*COEFF*CSP2/(AMOI2*CMOI2)
      KAMCP=2.*COEFF/(AMOI2*CPLU2)
      KAPCM=2.*COEFF/(APLU2*CMOI2)
40002 CALL MODHKL1(KRAM2,KRAP2,KRCM,KRCP,KRACM,KRACP,KRBM2,KRBP2,
     1KAMCP,KAPCM,CSM2,CSP2,AMOI2,APLU2,CMOI2,CPLU2,KRACMS,KRACPS,2)
      IF(NT.EQ.-1)GO TO 2300
      DO 30002 I=1,N-1
      J=N+1
      IF(IRJ(I,2).EQ.1.AND.IRJ(J,2).EQ.1.AND.IH(I,1,2).EQ.IH(J,1,2).AND.
     +IK(I,1,2).EQ.IK(J,1,2).AND.IL(I,1,2).EQ.IL(J,1,2))GO TO 2300
30002 CONTINUE
      NDICH(2)=NDICH(2)+1
      DO 3000 IBET3=0,1
      BEINF3=BEINF2+(IBET3*PAB4)
      BESUP3=BEINF3+PAB4
      CSM3=COS(BEINF3)
      CSP3=COS(BESUP3)
      DO 3100 IA3=0,1
      AMOI3=AMOI2+(IA3*PAS4)
C     IF(AMOI3.GE.AMAX)GO TO 3000
      APLU3=AMOI3+PAS4
      KRAM3=COEFF/(APLU3*APLU3)
      KRAP3=COEFF/(AMOI3*AMOI3)
      DO 3200 IB3=0,1
      BMOI3=BMOI2+(IB3*PAS4)
      BPLU3=BMOI3+PAS4
      KRBM3=COEFF/(BPLU3*BPLU3)
      KRBP3=COEFF/(BMOI3*BMOI3)
      DO 3300 IC3=0,1
      CMOI3=CMOI2+(IC3*PAS4)
      IF(CMOI3.GE.APLU3)GO TO 3200
      ATEST=ABS(AMOI3-AA)
      BTEST=ABS(BMOI3-BB)
      CTEST=ABS(CMOI3-CC)
      BETEST=ABS(BEINF3-BEBE)
      IF(ATEST.LT.0.1.AND.BTEST.LT.0.1.AND.CTEST.LT.0.1.AND.BETEST.LT.0.
     +009)GO TO 3300
      VTESTM=AMOI3*BMOI3*CMOI3/SIN(BEINF3)
      IF(VTESTM.GT.VSUP)GO TO 3200
      CPLU3=CMOI3+PAS4
      IF(MC.EQ.0.AND.APLU3.LE.AMAXM.AND.BPLU3.LE.BMAXM.AND.CPLU3.
     +LE.CMAXM)GO TO 3300
      VTESTP=APLU3*BPLU3*CPLU3/SIN(BESUP3)
      IF(VINF.GT.VTESTP)GO TO 3300
      KRCM=COEFF/(CPLU3*CPLU3)
      KRCP=COEFF/(CMOI3*CMOI3)
      KRACMS=2.*COEFF/(APLU3*CPLU3)
      KRACPS=2.*COEFF/(AMOI3*CMOI3)
      KRACM=2.*COEFF*CSM3/(APLU3*CPLU3)
      KRACP=2.*COEFF*CSP3/(AMOI3*CMOI3)
      KAMCP=2.*COEFF/(AMOI3*CPLU3)
      KAPCM=2.*COEFF/(APLU3*CMOI3)
40003 CALL MODHKL1(KRAM3,KRAP3,KRCM,KRCP,KRACM,KRACP,KRBM3,KRBP3,
     1KAMCP,KAPCM,CSM3,CSP3,AMOI3,APLU3,CMOI3,CPLU3,KRACMS,KRACPS,3)
      IF(NT.EQ.-1)GO TO 3300
      DO 30003 I=1,N-1
      J=N+1
      IF(IRJ(I,3).EQ.1.AND.IRJ(J,3).EQ.1.AND.IH(I,1,3).EQ.IH(J,1,3).AND.
     +IK(I,1,3).EQ.IK(J,1,3).AND.IL(I,1,3).EQ.IL(J,1,3))GO TO 3300
30003 CONTINUE
      NDICH(3)=NDICH(3)+1
      DO 4000 IBET4=0,1
      BEINF4=BEINF3+(IBET4*PAB8)
      BESUP4=BEINF4+PAB8
      CSM4=COS(BEINF4)
      CSP4=COS(BESUP4)
      DO 4100 IA4=0,1
      AMOI4=AMOI3+(IA4*PAS8)
C     IF(AMOI4.GE.AMAX)GO TO 4000
      APLU4=AMOI4+PAS8
      KRAM4=COEFF/(APLU4*APLU4)
      KRAP4=COEFF/(AMOI4*AMOI4)
      DO 4200 IB4=0,1
      BMOI4=BMOI3+(IB4*PAS8)
      BPLU4=BMOI4+PAS8
      KRBM4=COEFF/(BPLU4*BPLU4)
      KRBP4=COEFF/(BMOI4*BMOI4)
      DO 4300 IC4=0,1
      CMOI4=CMOI3+(IC4*PAS8)
      IF(CMOI4.GE.APLU4)GO TO 4200
      ATEST=ABS(AMOI4-AA)
      BTEST=ABS(BMOI4-BB)
      CTEST=ABS(CMOI4-CC)
      BETEST=ABS(BEINF4-BEBE)
      IF(ATEST.LT.0.1.AND.BTEST.LT.0.1.AND.CTEST.LT.0.1.AND.BETEST.LT.0.
     +009)GO TO 4300
      VTESTM=AMOI4*BMOI4*CMOI4/SIN(BEINF4)
      IF(VTESTM.GT.VSUP)GO TO 4200
      CPLU4=CMOI4+PAS8
      IF(MC.EQ.0.AND.APLU4.LE.AMAXM.AND.BPLU4.LE.BMAXM.AND.CPLU4.
     +LE.CMAXM)GO TO 4300
      VTESTP=APLU4*BPLU4*CPLU4/SIN(BESUP4)
      IF(VINF.GT.VTESTP)GO TO 4300
      KRCM=COEFF/(CPLU4*CPLU4)
      KRCP=COEFF/(CMOI4*CMOI4)
      KRACMS=2.*COEFF/(APLU4*CPLU4)
      KRACPS=2.*COEFF/(AMOI4*CMOI4)
      KRACM=2.*COEFF*CSM4/(APLU4*CPLU4)
      KRACP=2.*COEFF*CSP4/(AMOI4*CMOI4)
      KAMCP=2.*COEFF/(AMOI4*CPLU4)
      KAPCM=2.*COEFF/(APLU4*CMOI4)
40004 CALL MODHKL1(KRAM4,KRAP4,KRCM,KRCP,KRACM,KRACP,KRBM4,KRBP4,
     1KAMCP,KAPCM,CSM4,CSP4,AMOI4,APLU4,CMOI4,CPLU4,KRACMS,KRACPS,4)
      IF(NT.EQ.-1)GO TO 4300
      DO 30004 I=1,N-1
      J=N+1
      IF(IRJ(I,4).EQ.1.AND.IRJ(J,4).EQ.1.AND.IH(I,1,4).EQ.IH(J,1,4).AND.
     +IK(I,1,4).EQ.IK(J,1,4).AND.IL(I,1,4).EQ.IL(J,1,4))GO TO 4300
30004 CONTINUE
      NDICH(4)=NDICH(4)+1
      DO 5000 IBET5=0,1
      BEINF5=BEINF4+(IBET5*PAB16)
      BESUP5=BEINF5+PAB16
      CSM5=COS(BEINF5)
      CSP5=COS(BESUP5)
      DO 5100 IA5=0,1
      AMOI5=AMOI4+(IA5*PAS16)
C     IF(AMOI5.GE.AMAX)GO TO 5000
      APLU5=AMOI5+PAS16
      KRAM5=COEFF/(APLU5*APLU5)
      KRAP5=COEFF/(AMOI5*AMOI5)
      DO 5200 IB5=0,1
      BMOI5=BMOI4+(IB5*PAS16)
      BPLU5=BMOI5+PAS16
      KRBM5=COEFF/(BPLU5*BPLU5)
      KRBP5=COEFF/(BMOI5*BMOI5)
      DO 5300 IC5=0,1
      CMOI5=CMOI4+(IC5*PAS16)
      IF(CMOI5.GE.APLU5)GO TO 5200
      ATEST=ABS(AMOI5-AA)
      BTEST=ABS(BMOI5-BB)
      CTEST=ABS(CMOI5-CC)
      BETEST=ABS(BEINF5-BEBE)
      IF(ATEST.LT.0.1.AND.BTEST.LT.0.1.AND.CTEST.LT.0.1.AND.BETEST.LT.0.
     +009)GO TO 5300
      VTESTM=AMOI5*BMOI5*CMOI5/SIN(BEINF5)
      IF(VTESTM.GT.VSUP)GO TO 5200
      CPLU5=CMOI5+PAS16
      IF(MC.EQ.0.AND.APLU5.LE.AMAXM.AND.BPLU5.LE.BMAXM.AND.CPLU5.
     +LE.CMAXM)GO TO 5300
      VTESTP=APLU5*BPLU5*CPLU5/SIN(BESUP5)
      IF(VINF.GT.VTESTP)GO TO 5300
      KRCM=COEFF/(CPLU5*CPLU5)
      KRCP=COEFF/(CMOI5*CMOI5)
      KRACMS=2.*COEFF/(APLU5*CPLU5)
      KRACPS=2.*COEFF/(AMOI5*CMOI5)
      KRACM=2.*COEFF*CSM5/(APLU5*CPLU5)
      KRACP=2.*COEFF*CSP5/(AMOI5*CMOI5)
      KAMCP=2.*COEFF/(AMOI5*CPLU5)
      KAPCM=2.*COEFF/(APLU5*CMOI5)
40005 CALL MODHKL1(KRAM5,KRAP5,KRCM,KRCP,KRACM,KRACP,KRBM5,KRBP5,
     1KAMCP,KAPCM,CSM5,CSP5,AMOI5,APLU5,CMOI5,CPLU5,KRACMS,KRACPS,5)
      IF(NT.EQ.-1)GO TO 5300
      DO 30005 I=1,N-1
      J=N+1
      IF(IRJ(I,5).EQ.1.AND.IRJ(J,5).EQ.1.AND.IH(I,1,5).EQ.IH(J,1,5).AND.
     +IK(I,1,5).EQ.IK(J,1,5).AND.IL(I,1,5).EQ.IL(J,1,5))GO TO 5300
30005 CONTINUE
      NDICH(5)=NDICH(5)+1
      DO 6000 IBET6 =0,1
      BEINF6=BEINF5+(IBET6*PAB32)
      BESUP6=BEINF6+PAB32
      CSM6=COS(BEINF6)
      CSP6=COS(BESUP6)
      DO 6100 IA6=0,1
      AMOI6=AMOI5+(IA6*PAS32)
C     IF(AMOI6.GE.AMAX)GO TO 6000
      APLU6=AMOI6+PAS32
      KRAM6=COEFF/(APLU6*APLU6)
      KRAP6=COEFF/(AMOI6*AMOI6)
      DO 6200 IB6=0,1
      BMOI6=BMOI5+(IB6*PAS32)
      BPLU6=BMOI6+PAS32
      KRBM6=COEFF/(BPLU6*BPLU6)
      KRBP6=COEFF/(BMOI6*BMOI6)
      DO 6300 IC6=0,1
      CMOI6=CMOI5+(IC6*PAS32)
      IF(CMOI6.GE.APLU6)GO TO 6200
      ATEST=ABS(AMOI6-AA)
      BTEST=ABS(BMOI6-BB)
      CTEST=ABS(CMOI6-CC)
      BETEST=ABS(BEINF6-BEBE)
      IF(ATEST.LT.0.1.AND.BTEST.LT.0.1.AND.CTEST.LT.0.1.AND.BETEST.LT.0.
     +009)GO TO 6300
      VTESTM=AMOI6*BMOI6*CMOI6/SIN(BEINF6)
      IF(VTESTM.GT.VSUP)GO TO 6200
      CPLU6=CMOI6+PAS32
      IF(MC.EQ.0.AND.APLU6.LE.AMAXM.AND.BPLU6.LE.BMAXM.AND.CPLU6.
     +LE.CMAXM)GO TO 6300
      VTESTP=APLU6*BPLU6*CPLU6/SIN(BESUP6)
      IF(VINF.GT.VTESTP)GO TO 6300
      KRCM=COEFF/(CPLU6*CPLU6)
      KRCP=COEFF/(CMOI6*CMOI6)
      KRACMS=2.*COEFF/(APLU6*CPLU6)
      KRACPS=2.*COEFF/(AMOI6*CMOI6)
      KRACM=2.*COEFF*CSM6/(APLU6*CPLU6)
      KRACP=2.*COEFF*CSP6/(AMOI6*CMOI6)
      KAMCP=2.*COEFF/(AMOI6*CPLU6)
      KAPCM=2.*COEFF/(APLU6*CMOI6)
40006 CALL MODHKL1(KRAM6,KRAP6,KRCM,KRCP,KRACM,KRACP,KRBM6,KRBP6,
     1KAMCP,KAPCM,CSM6,CSP6,AMOI6,APLU6,CMOI6,CPLU6,KRACMS,KRACPS,6)
      IF(NT.EQ.-1)GO TO 6300
      DO 30006 I=1,N-1
      J=N+1
      IF(IRJ(I,6).EQ.1.AND.IRJ(J,6).EQ.1.AND.IH(I,1,6).EQ.IH(J,1,6).AND.
     +IK(I,1,6).EQ.IK(J,1,6).AND.IL(I,1,6).EQ.IL(J,1,6))GO TO 6300
30006 CONTINUE
      NDICH(6)=NDICH(6)+1
      DO 7000 IBET7=0,1
      BEINF7=BEINF6+(IBET7*PAB64)
      BESUP7=BEINF7+PAB64
      CSM7=COS(BEINF7)
      CSP7=COS(BESUP7)
      DO 7100 IA7=0,1
      AMOI7=AMOI6+(IA7*PAS64)
C     IF(AMOI7.GE.AMAX)GO TO 7000
      APLU7=AMOI7+PAS64
      KRAM7=COEFF/(APLU7*APLU7)
      KRAP7=COEFF/(AMOI7*AMOI7)
      DO 7200 IB7=0,1
      BMOI7=BMOI6+(IB7*PAS64)
      BPLU7=BMOI7+PAS64
      KRBM7=COEFF/(BPLU7*BPLU7)
      KRBP7=COEFF/(BMOI7*BMOI7)
      DO 7300 IC7=0,1
      CMOI7=CMOI6+(IC7*PAS64)
      IF(CMOI7.GE.APLU7)GO TO 7200
      ATEST=ABS(AMOI7-AA)
      BTEST=ABS(BMOI7-BB)
      CTEST=ABS(CMOI7-CC)
      BETEST=ABS(BEINF7-BEBE)
      IF(ATEST.LT.0.1.AND.BTEST.LT.0.1.AND.CTEST.LT.0.1.AND.BETEST.LT.0.
     +009)GO TO 7300
      VTESTM=AMOI7*BMOI7*CMOI7/SIN(BEINF7)
      IF(VTESTM.GT.VSUP)GO TO 7200
      CPLU7=CMOI7+PAS64
      IF(MC.EQ.0.AND.APLU7.LE.AMAXM.AND.BPLU7.LE.BMAXM.AND.CPLU7.
     +LE.CMAXM)GO TO 7300
      VTESTP=APLU7*BPLU7*CPLU7/SIN(BESUP7)
      IF(VINF.GT.VTESTP)GO TO 7300
      KRCM=COEFF/(CPLU7*CPLU7)
      KRCP=COEFF/(CMOI7*CMOI7)
      KRACMS=2.*COEFF/(APLU7*CPLU7)
      KRACPS=2.*COEFF/(AMOI7*CMOI7)
      KRACM=2.*COEFF*CSM7/(APLU7*CPLU7)
      KRACP=2.*COEFF*CSP7/(AMOI7*CMOI7)
      KAMCP=2.*COEFF/(AMOI7*CPLU7)
      KAPCM=2.*COEFF/(APLU7*CMOI7)
40007 CALL MODHKL1(KRAM7,KRAP7,KRCM,KRCP,KRACM,KRACP,KRBM7,KRBP7,
     1KAMCP,KAPCM,CSM7,CSP7,AMOI7,APLU7,CMOI7,CPLU7,KRACMS,KRACPS,7)
      IF(NT.EQ.-1)GO TO 7300
      DO 30007 I=1,N-1
      J=N+1
      IF(IRJ(I,7).EQ.1.AND.IRJ(J,7).EQ.1.AND.IH(I,1,7).EQ.IH(J,1,7).AND.
     +IK(I,1,7).EQ.IK(J,1,7).AND.IL(I,1,7).EQ.IL(J,1,7))GO TO 7300
30007 CONTINUE
      NDICH(7)=NDICH(7)+1
      AARE=AA
      BBRE=BB
      CCRE=CC
      BEBERE=BEBE
      AA=AMOI7+(PAS64/2.)
      BB=BMOI7+(PAS64/2.)
      CC=CMOI7+(PAS64/2.)
      BEBE=BEINF7+(PAB64/2.)
      VAP=AA*BB*CC/SIN(BEBE)
      BETA=PIDEG*BEBE
C
C
      DO 15 I=1,N
      DO 16 J=1,20
      QCAL(I,J)=-1.
   16 CONTINUE
   15 CONTINUE
      DO 45 I=1,N
      JJ=IRJ(I,7)
      DO 35 J=1,JJ
      CARH=IH(I,J,7)*IH(I,J,7)
      CARK=IK(I,J,7)*IK(I,J,7)
      CARL=IL(I,J,7)*IL(I,J,7)
      PROHL=IL(I,J,7)*IH(I,J,7)
      QCAL(I,J)=CARH/AA**2+CARK/BB**2+CARL/CC**2-2*PROHL
     S*COS(BEBE)/AA/CC
   35 CONTINUE
   45 CONTINUE
      IND=5
      NRIND=4
      CALL AFFPAR(IND,NRIND,VAP)
      AA=AA*SIN(BEBE)
      CC=CC*SIN(BEBE)
      IF(FWOLFF.NE.-1000.)GO TO 7313
      AA=AARE
      BB=BBRE
      CC=CCRE
      BEBE=BEBERE
      GO TO 7300
 7313 WRITE(IW,3)
 7300 CONTINUE
 7200 CONTINUE
 7100 CONTINUE
 7000 CONTINUE
 6300 CONTINUE
 6200 CONTINUE
 6100 CONTINUE
 6000 CONTINUE
 5300 CONTINUE
 5200 CONTINUE
 5100 CONTINUE
 5000 CONTINUE
 4300 CONTINUE
 4200 CONTINUE
 4100 CONTINUE
 4000 CONTINUE
 3300 CONTINUE
 3200 CONTINUE
 3100 CONTINUE
 3000 CONTINUE
 2300 CONTINUE
 2200 CONTINUE
 2100 CONTINUE
 2000 CONTINUE
 1300 CONTINUE
 1200 CONTINUE
 1100 CONTINUE
 1000 CONTINUE
 1007 CONTINUE
      WRITE(IW,10)(NDICH(I),I=1,7)
   10 FORMAT(/,10X,'ITERATION NUMBER AT EACH DICHOTOMY LEVEL :',
     S7X,7(3X,I5))
      RETURN
      END
      SUBROUTINE MODHKL(KRAM,KRAP,KRCM,KRCP,KRACM,KRACP,KRBM,KRBP,
     1KAMCP,KAPCM,CSM,CSP,AMOI,APLU,CMOI,CPLU,KRACMS,KRACPS)
      COMMON D(50),Q(50),KQ(50),TH(50),EPSIL(50),EPSQ(50),KEPSQ(50),DTH,
     1       V,VINF,VSUP,PAS,PAS2,PAS4,PAS8,PAS16,PAS32,PAS64,COEFF,RAP,
     2       WAVE2,PIDEG,PIRAD,N,MH,MH2,MK,MK2,ML,ML2,KDENS,KZ,KQT(50)
      COMMON /BAF/NT,IRJ(50,8),IH(50,50,8),IK(50,50,8),IL(50,100,8)
     1           ,NPOSS,FWOLFF
      F(NX,NY,NZ,DD,NE,NG)=NX+NY+NZ-DD*NE*NG
      NT=0
      DO 15 I=1,N
      IRJ(I,1)=0
   15 CONTINUE
      DO 10 M=0,MH
      MDL=ML*MOD(1,M+1)
      MCARH=M*M
      MM1=MCARH*KRAM
      MP1=MCARH*KRAP
      DO 20 K=0,MK
      MCARK=K*K
      MM2=(MCARK*KRBM)
      MP2=(MCARK*KRBP)
      DO 30 MXL=-MDL,ML
      MCARL=MXL*MXL
      MPROHL=M*MXL
      MM3=(MCARL*KRCM)
      MP3=(MCARL*KRCP)
      IF(MPROHL)50,51,51
   50 LC0=0
      LA1=0
      IF((M*CPLU/(AMOI*MXL)).GT.CSP)GO TO 710
      LC0=1
      IF((M*CPLU/(MXL*APLU)).LT.CSP)GO TO 710
      KQQM=(COEFF*(MXL/CPLU)**2*(1-CSP**2))+MM2
      GO TO 760
  710 IF((M*CMOI/(MXL*AMOI)).GT.CSP)GO TO 720
      IF((M*CMOI/(MXL*APLU)).LT.CSP)GO TO 720
      KQQM=F(MM1,MM2,MM3,CSP,KRACMS,MPROHL)
      GO TO 760
  720 IF((MXL*APLU/(M*CMOI)).GT.CSP)GO TO 730
      LA1=1
      IF((MXL*APLU/(M*CPLU)).LT.CSP)GO TO 730
      KQQM=(COEFF*(M/APLU)**2*(1-CSP**2))+MM2
      GO TO 750
  730 IF((MXL*AMOI/(M*CMOI)).GT.CSP.OR.(MXL*AMOI/(M*CPLU)).LT.CSP)GO
     1TO 770
      KQQM=F(MM1,MM2,MM3,CSP,KRACMS,MPROHL)
  750 IF((1./CMOI+1./CPLU).GE.((2.*M*CSM)/(MXL*AMOI)))GO TO 765
      KQQP=F(MP1,MP2,MM3,CSM,KAMCP,MPROHL)
      GO TO 52
  760 IF((1./AMOI+1./APLU).GE.(2.*MXL*CSM/(M*CMOI)))GO TO 765
      KQQP=F(MM1,MP2,MP3,CSM,KAPCM,MPROHL)
      GO TO 52
  765 KQQP=F(MP1,MP2,MP3,CSM,KRACPS,MPROHL)
      GO TO 52
  770 IF(CSM.GE.(MXL*APLU/(M*CMOI)))GO TO 775
      KQQP=F(MP1,MP2,MM3,CSM,KAMCP,MPROHL)
      KQQM=F(MM1,MM2,MP3,CSP,KAPCM,MPROHL)
      GO TO 52
  775 IF(CSM.LE.(MXL*AMOI/(M*CPLU)))GO TO 790
      IF(LC0.EQ.0)GO TO 780
      KQQP=F(MP1,MP2,MP3,CSM,KRACPS,MPROHL)
      IF(LA1.EQ.0)GO TO 785
      KQQM=F(MM1,MM2,MM3,CSP,KRACMS,MPROHL)
      GO TO 760
  785 KQQM=F(MM1,MM2,MP3,CSP,KAPCM,MPROHL)
      GO TO 52
  780 KQQM=F(MP1,MM2,MM3,CSP,KAMCP,MPROHL)
      GO TO 760
  790 KQQM=F(MM1,MM2,MP3,CSP,KAPCM,MPROHL)
      KQQP=F(MP1,MP2,MM3,CSM,KAMCP,MPROHL)
      GO TO 52
   52 IF((KQQM-KEPSQ(5)).GT.KQ(5))GO TO 30
      GO TO 53
   51 KQQM=MM1+MM2+MM3-(MPROHL*KRACM)
      IF((KQQM-KEPSQ(5)).LE.KQ(5))GO TO 59
      IF(MXL)30,20,20
   59 KQQP=MP1+MP2+MP3-(MPROHL*KRACP)
   53 IF((KQQP+KEPSQ(1)).LT.KQ(1))GO TO 30
      DO 40 I=1,5
      KBORIN=KQQM-KEPSQ(I)
      IF(KBORIN.GT.KQ(I))GO TO 40
      KBORSU=KQQP+KEPSQ(I)
      IF(KBORSU.LT.KQ(I))GO TO 40
      IRJ(I,1)=IRJ(I,1)+1
      JJ=IRJ(I,1)
      IH(I,JJ,1)=M
      IK(I,JJ,1)=K
      IL(I,JJ,1)=MXL
   40 CONTINUE
   30 CONTINUE
   20 CONTINUE
   10 CONTINUE
      DO 70 I=1,5
      IF(IRJ(I,1).NE.0.)GO TO 70
      NT=-1
      GO TO 3000
   70 CONTINUE
      DO 80 M=0,MH2
      MDL2=ML2*MOD(1,M+1)
      MCARH=M*M
      MM1=MCARH*KRAM
      MP1=MCARH*KRAP
      DO 90 K=0,MK2
      MCARK=K*K
      MM2=(MCARK*KRBM)
      MP2=(MCARK*KRBP)
      DO 100 MXL=-MDL2,ML2
      MCARL=MXL*MXL
      MPROHL=M*MXL
      MM3=(MCARL*KRCM)
      MP3=(MCARL*KRCP)
      IF(MPROHL)101,102,102
  101 LC0=0
      LA1=0
      IF((M*CPLU/(AMOI*MXL)).GT.CSP)GO TO 810
      LC0=1
      IF((M*CPLU/(MXL*APLU)).LT.CSP)GO TO 810
      KQQM=(COEFF*(MXL/CPLU)**2*(1-CSP**2))+MM2
      GO TO 860
  810 IF((M*CMOI/(MXL*AMOI)).GT.CSP)GO TO 820
      IF((M*CMOI/(MXL*APLU)).LT.CSP)GO TO 820
      KQQM=F(MM1,MM2,MM3,CSP,KRACMS,MPROHL)
      GO TO 850
  820 IF((MXL*APLU/(M*CMOI)).GT.CSP)GO TO 830
      LA1=1
      IF((MXL*APLU/(M*CPLU)).LT.CSP)GO TO 830
      KQQM=COEFF*(M/APLU)**2*(1-CSP**2)+MM2
      GO TO 860
  830 IF((MXL*AMOI/(M*CMOI)).GT.CSP.OR.(MXL*AMOI/(M*CPLU)).LT.CSP)GO
     1TO 870
      KQQM=F(MM1,MM2,MM3,CSP,KRACMS,MPROHL)
  850 IF((1./CMOI+1./CPLU).GE.((2.*M*CSM)/(MXL*AMOI)))GO TO 865
      KQQP=F(MP1,MP2,MM3,CSM,KAMCP,MPROHL)
      GO TO 103
  860 IF((1./AMOI+1./APLU).GE.(2.*MXL*CSM/(M*CMOI)))GO TO 865
      KQQP=F(MM1,MP2,MP3,CSM,KAPCM,MPROHL)
      GO TO 103
  865 KQQP=F(MP1,MP2,MP3,CSM,KRACPS,MPROHL)
      GO TO 103
  870 IF(CSM.GE.(MXL*APLU/(M*CMOI)))GO TO 875
      KQQP=F(MP1,MP2,MM3,CSM,KAMCP,MPROHL)
      KQQM=F(MM1,MM2,MP3,CSP,KAPCM,MPROHL)
      GO TO 103
  875 IF(CSM.LE.(MXL*AMOI/(M*CPLU)))GO TO 890
      IF(LC0.EQ.0)GO TO 880
      KQQP=F(MP1,MP2,MP3,CSM,KRACPS,MPROHL)
      IF(LA1.EQ.0)GO TO 885
      KQQM=F(MM1,MM2,MM3,CSP,KRACMS,MPROHL)
      GO TO 860
  885 KQQM=F(MM1,MM2,MP3,CSP,KAPCM,MPROHL)
      GO TO 103
  880 KQQM=F(MP1,MM2,MM3,CSP,KAMCP,MPROHL)
      GO TO 860
  890 KQQM=F(MM1,MM2,MP3,CSP,KAPCM,MPROHL)
      KQQP=F(MP1,MP2,MM3,CSM,KAMCP,MPROHL)
      GO TO 103
  103 IF((KQQM-KEPSQ(N)).GT.KQ(N))GO TO 100
      GO TO 104
  102 KQQM=MM1+MM2+MM3-(MPROHL*KRACM)
      IF((KQQM-KEPSQ(N)).LE.KQ(N))GO TO 109
      IF(MXL)100,90,90
  109 KQQP=MP1+MP2+MP3-(MPROHL*KRACP)
  104 IF((KQQP+KEPSQ(6)).LT.KQ(6))GO TO 100
      DO 110 I=6,N
      KBORIN=KQQM-KEPSQ(I)
      IF(KBORIN.GT.KQ(I))GO TO 110
      KBORSU=KQQP+KEPSQ(I)
      IF(KBORSU.LT.KQ(I))GO TO 110
      IRJ(I,1)=IRJ(I,1)+1
      JJ=IRJ(I,1)
      IH(I,JJ,1)=M
      IK(I,JJ,1)=K
      IL(I,JJ,1)=MXL
  110 CONTINUE
  100 CONTINUE
   90 CONTINUE
   80 CONTINUE
 3000 RETURN
      END
      SUBROUTINE MODHKL1(KRAM,KRAP,KRCM,KRCP,KRACM,KRACP,KRBM,KRBP,
     1KAMCP,KAPCM,CSM,CSP,AMOI,APLU,CMOI,CPLU,KRACMS,KRACPS,K1)
      COMMON D(50),Q(50),KQ(50),TH(50),EPSIL(50),EPSQ(50),KEPSQ(50),DTH,
     1       V,VINF,VSUP,PAS,PAS2,PAS4,PAS8,PAS16,PAS32,PAS64,COEFF,RAP,
     2       WAVE2,PIDEG,PIRAD,N,MH,MH2,MK,MK2,ML,ML2,KDENS,KZ,KQT(50)
      COMMON /BAF/NT,IRJ(50,8),IH(50,50,8),IK(50,50,8),IL(50,100,8)
     1           ,NPOSS,FWOLFF
      F(NX,NY,NZ,DD,NE,NG)=NX+NY+NZ-DD*NE*NG
      NT=0
      KK=K1-1
      DO 5 I=1,N
      JJ=IRJ(I,KK)
      IRJ(I,K1)=0
      DO 10 J=1,JJ
      M=IH(I,J,KK)
      K=IK(I,J,KK)
      MXL=IL(I,J,KK)
      MCARH=M*M
      MCARK=K*K
      MCARL=MXL*MXL
      MPROHL=M*MXL
      MM1=MCARH*KRAM
      MP1=MCARH*KRAP
      MM2=MCARK*KRBM
      MP2=MCARK*KRBP
      MM3=MCARL*KRCM
      MP3=MCARL*KRCP
      IF(MPROHL)50,51,51
   50 LC0=0
      LA1=0
      IF((M*CPLU/(AMOI*MXL)).GT.CSP)GO TO 710
      LC0=1
      IF((M*CPLU/(MXL*APLU)).LT.CSP)GO TO 710
      KQQM=(COEFF*(MXL/CPLU)**2*(1-CSP**2))+MM2
      GO TO 760
  710 IF((M*CMOI/(MXL*AMOI)).GT.CSP)GO TO 720
      IF((M*CMOI/(MXL*APLU)).LT.CSP)GO TO 720
      KQQM=F(MM1,MM2,MM3,CSP,KRACMS,MPROHL)
      GO TO 760
  720 IF((MXL*APLU/(M*CMOI)).GT.CSP)GO TO 730
      LA1=1
      IF((MXL*APLU/(M*CPLU)).LT.CSP)GO TO 730
      KQQM=(COEFF*(M/APLU)**2*(1-CSP**2))+MM2
      GO TO 750
  730 IF((MXL*AMOI/(M*CMOI)).GT.CSP.OR.(MXL*AMOI/(M*CPLU)).LT.CSP)GO
     1TO 770
      KQQM=F(MM1,MM2,MM3,CSP,KRACMS,MPROHL)
  750 IF((1./CMOI+1./CPLU).GE.((2.*M*CSM)/(MXL*AMOI)))GO TO 765
      KQQP=F(MP1,MP2,MM3,CSM,KAMCP,MPROHL)
      GO TO 52
  760 IF((1./AMOI+1./APLU).GE.(2.*MXL*CSM/(M*CMOI)))GO TO 765
      KQQP=F(MM1,MP2,MP3,CSM,KAPCM,MPROHL)
      GO TO 52
  765 KQQP=F(MP1,MP2,MP3,CSM,KRACPS,MPROHL)
      GO TO 52
  770 IF(CSM.GE.(MXL*APLU/(M*CMOI)))GO TO 775
      KQQP=F(MP1,MP2,MM3,CSM,KAMCP,MPROHL)
      KQQM=F(MM1,MM2,MP3,CSP,KAPCM,MPROHL)
      GO TO 52
  775 IF(CSM.LE.(MXL*AMOI/(M*CPLU)))GO TO 790
      IF(LC0.EQ.0)GO TO 780
      KQQP=F(MP1,MP2,MP3,CSM,KRACPS,MPROHL)
      IF(LA1.EQ.0)GO TO 785
      KQQM=F(MM1,MM2,MM3,CSP,KRACMS,MPROHL)
      GO TO 760
  785 KQQM=F(MM1,MM2,MP3,CSP,KAPCM,MPROHL)
      GO TO 52
  780 KQQM=F(MP1,MM2,MM3,CSP,KAMCP,MPROHL)
      GO TO 760
  790 KQQM=F(MM1,MM2,MP3,CSP,KAPCM,MPROHL)
      KQQP=F(MP1,MP2,MM3,CSM,KAMCP,MPROHL)
      GO TO 52
   51 KQQM=MM1+MM2+MM3-(MPROHL*KRACM)
      KQQP=MP1+MP2+MP3-(MPROHL*KRACP)
   52 IF((KQQM-KEPSQ(I)).GT.KQ(I))GO TO 10
      IF((KQQP+KEPSQ(I)).LT.KQ(I))GO TO 10
      IRJ(I,K1)=IRJ(I,K1)+1
      KJ=IRJ(I,K1)
      IH(I,KJ,K1)=M
      IK(I,KJ,K1)=K
      IL(I,KJ,K1)=MXL
   10 CONTINUE
      IF(IRJ(I,K1).NE.0)GO TO 5
      NT=-1
      GO TO 55
    5 CONTINUE
   55 RETURN
      END
C     ------------------------------------------------------------------
C     |                     T R I C L I N I Q U E                      |
C     ------------------------------------------------------------------
      SUBROUTINE TRICLINI(VMIN,VMAX)
      DIMENSION NDICH(8),AAS(100),BBS(100),
     +CCS(100),DDS(100),EES(100),FFS(100),ARX(50),BRX(50),
     +CRX(50),DRX(50),ERX(50),FRX(50)
      COMMON D(50),Q(50),KQ(50),TH(50),EPSIL(50),EPSQ(50),KEPSQ(50),DTH,
     1       V,VINF,VSUP,PAS,PAS2,PAS4,PAS8,PAS16,PAS32,PAS64,COEFF,RAP,
     2       WAVE2,PIDEG,PIRAD,N,MH,MH2,MK,MK2,ML,ML2,KDENS,KZ,KQT(50)
      COMMON QCAL(50,20),AA,BB,CC,BETA,AR,BR,CR,DR,ER,FR
      COMMON /BAF/NT,IRJ(50,8),IH(50,50,8),IK(50,50,8),IL(50,100,8)
     1           ,NPOSS,FWOLFF
      COMMON /AMAX DD/AMAX,DD1,DD2
      COMMON /AMAX/AMIN,BMAX,BMIN,KAMAX,KAMIN,KBMAX,KBMIN
     +,DMIN,DMAX,AMN,BMN,KDMIN,KDMAX,DMN
      COMMON /N5/N5,N6
      COMMON /MT/MT,IRJB(50),IHB(50,50),IKB(50,50),ILB(50,50)
      COMMON /VMAX/VMAX2,VMIN2,VR2,LY,VMOII
      COMMON /IIDD/IIDD,CMAX,JRD,RMIN(50),RMAX(50),RCMAX(50)
      COMMON /DMOI/DMOI1,DMOI2,DMOI3,DMOI4,DMOI5,DMOI6,DMOI7,DPLU1,DPLU2
     +,DPLU3,DPLU4,DPLU5,DPLU6,DPLU7,KDMOI1,KDMOI2,KDMOI3,KDMOI4,KDMOI5,
     +KDMOI6,KDMOI7,KDPLU1,KDPLU2,KDPLU3,KDPLU4,KDPLU5,KDPLU6,KDPLU7
      DATA IR,IW/15,16/
      CXA=.01
      CXB=.01
      CXC=.01
      CXD=.01
      CXE=.01
      CXF=.01
      CYA=.0015
      PASA=.0027
      PASB=.0027
      PASC=.0027
      PASD=.0027
      PASE=.0027
      PASF=.0027
      N5=5
      N6=6
      ZCO=1.98296076
      KT=1
      KTP=0
      JB=0
      VMAX2=VMAX**2
      VMIN2=VMIN*VMIN
      VR2=V*V
      AAS(1)=1000.
      BBS(1)=1000.
      CCS(1)=1000.
      DDS(1)=1000.
      EES(1)=1000.
      FFS(1)=1000.
      PASA2=PASA/2.
      PASA4=PASA/4.
      PASA8=PASA/8.
      PASA16=PASA/16.
      PASA32=PASA/32.
      PASA64=PASA/64.
      PASB2=PASB/2.
      PASB4=PASB/4.
      PASB8=PASB/8.
      PASB16=PASB/16.
      PASB32=PASB/32.
      PASB64=PASB/64.
      PASC2=PASC/2.
      PASC4=PASC/4.
      PASC8=PASC/8.
      PASC16=PASC/16.
      PASC32=PASC/32.
      PASC64=PASC/64.
      PASD2=PASD/2.
      PASD4=PASD/4.
      PASD8=PASD/8.
      PASD16=PASD/16.
      PASD32=PASD/32.
      PASD64=PASD/64.
      PASE2=PASE/2.
      PASE4=PASE/4.
      PASE8=PASE/8.
      PASE16=PASE/16.
      PASE32=PASE/32.
      PASE64=PASE/64.
      PASF2=PASF/2.
      PASF4=PASF/4.
      PASF8=PASF/8.
      PASF16=PASF/16.
      PASF32=PASF/32.
      PASF64=PASF/64.
      MH=4
      MK=4
      ML=4
      MH2=6
      MK2=6
      ML2=6
      DO 8500 IJX=1,14
      DO 20 I=1,8
  20  NDICH(I)=0
      DMIN=.0
      EMIN=.0
      FMIN=.0
      JRD=0
      JRT=1
      AMIN=Q(1)/4.-EPSQ(1)/4.
      AMAX=Q(4)+EPSQ(4)
      CMIN=AMIN
      CMAX=AMAX
      BMIN=AMIN
      BMAX=AMAX
      FMAX=SQRT(ZCO)*AMAX
      EMAX=2.*AMAX
      DMAX=FMAX
      MD12=0
      MD34=0
      MD56=0
      MD7=0
      MD89=0
      JJAA=1
      JJBB=1
      JJCC=1
      JJDD=1
      JJEE=1
      JJFF=1
      IIAA=1
      IIBB=1
      IICC=1
      IIDD=1
      IIEE=1
      IIFF=1
      GO TO(2101,2102,2104,2106,2107,2109,2110,2111,2112,2113,2118,2
     +120,2121,2123),IJX
C 1
2101  IRJ(1,7)=1
      AMAX=Q(1)+EPSQ(1)
      AMIN=Q(1)-EPSQ(1)
      BMAX=Q(2)+EPSQ(2)
      BMIN=Q(2)-EPSQ(2)
      KAMIN=COEFF*AMIN
      KAMAX=COEFF*AMAX
      KBMIN=COEFF*BMIN
      KBMAX=COEFF*BMAX
      AMN=(AMIN+AMAX)/2.
      BMN=(BMIN+BMAX)/2.
      AMOI1=AMIN
      AMOI2=AMIN
      AMOI3=AMIN
      AMOI4=AMIN
      AMOI5=AMIN
      AMOI6=AMIN
      AMOI7=AMIN
      APLU1=AMAX
      APLU2=AMAX
      APLU3=AMAX
      APLU4=AMAX
      APLU5=AMAX
      APLU6=AMAX
      APLU7=AMAX
      BMOI1=BMIN
      BMOI2=BMIN
      BMOI3=BMIN
      BMOI4=BMIN
      BMOI5=BMIN
      BMOI6=BMIN
      BMOI7=BMIN
      BPLU1=BMAX
      BPLU2=BMAX
      BPLU3=BMAX
      BPLU4=BMAX
      BPLU5=BMAX
      BPLU6=BMAX
      BPLU7=BMAX
      KAMOI1=KAMIN
      KAMOI2=KAMIN
      KAMOI3=KAMIN
      KAMOI4=KAMIN
      KAMOI5=KAMIN
      KAMOI6=KAMIN
      KAMOI7=KAMIN
      KAPLU1=KAMAX
      KAPLU2=KAMAX
      KAPLU3=KAMAX
      KAPLU4=KAMAX
      KAPLU5=KAMAX
      KAPLU6=KAMAX
      KAPLU7=KAMAX
      KBMOI1=KBMIN
      KBMOI2=KBMIN
      KBMOI3=KBMIN
      KBMOI4=KBMIN
      KBMOI5=KBMIN
      KBMOI6=KBMIN
      KBMOI7=KBMIN
      KBPLU1=KBMAX
      KBPLU2=KBMAX
      KBPLU3=KBMAX
      KBPLU4=KBMAX
      KBPLU5=KBMAX
      KBPLU6=KBMAX
      KBPLU7=KBMAX
      JJAA=0
      JJBB=0
      IIAA=0
      IIBB=0
      DMAX=AMIN 1(SQRT(ZCO*AMAX*BMAX),DMAX)
      FMAX=AMIN 1(SQRT(ZCO*AMAX*CMAX),FMAX)
      CMIN=AMAX 1(CMIN,EMIN*EMIN/BMAX/2.)
      CMIN=AMAX 1(CMIN,FMIN*FMIN/AMAX/ZCO)
      EMAX=AMIN 1(2*SQRT(BMAX*CMAX),EMAX)
      DMI=DMIN
      DMA=DMAX
      CMA=CMAX
      CMI=CMIN
      FMA=FMAX
      EMA=EMAX
      CALL RELSPE
      IF(JRD.EQ.0)GO TO 2038
      JJDD=0
 4001 DMIN=RMIN(JRT)
      DMAX=RMAX(JRT)
      DMAX=AMIN 1(SQRT(ZCO*AMAX*BMAX),DMAX)
      KDMIN=COEFF*DMIN
      KDMAX=COEFF*DMAX
      DMOI1=DMIN
      DMOI2=DMIN
      DMOI3=DMIN
      DMOI4=DMIN
      DMOI5=DMIN
      DMOI6=DMIN
      DMOI7=DMIN
      DPLU1=DMAX
      DPLU2=DMAX
      DPLU3=DMAX
      DPLU4=DMAX
      DPLU5=DMAX
      DPLU6=DMAX
      DPLU7=DMAX
      KDMOI1=KDMIN
      KDMOI2=KDMIN
      KDMOI3=KDMIN
      KDMOI4=KDMIN
      KDMOI5=KDMIN
      KDMOI6=KDMIN
      KDMOI7=KDMIN
      KDPLU1=KDMAX
      KDPLU2=KDMAX
      KDPLU3=KDMAX
      KDPLU4=KDMAX
      KDPLU5=KDMAX
      KDPLU6=KDMAX
      KDPLU7=KDMAX
      CMAX=RCMAX(JRT)
      EMAX=AMIN 1(2*SQRT(BMAX*CMAX),EMAX)
      FMAX=AMIN 1(SQRT(ZCO*AMAX*CMAX),FMAX)
      CMIN=AMAX 1(CMIN,EMIN*EMIN/BMAX/2.)
      CMIN=AMAX 1(CMIN,FMIN*FMIN/AMAX/ZCO)
      ASSIGN 4001 TO IAS
      GO TO 2038
C 2
2102  IRJ(1,7)=1
      AMIN=Q(1)-EPSQ(1)
      AMAX=Q(1)+EPSQ(1)
      KAMIN=COEFF*AMIN
      KAMAX=COEFF*AMAX
      AMOI1=AMIN
      AMOI2=AMIN
      AMOI3=AMIN
      AMOI4=AMIN
      AMOI5=AMIN
      AMOI6=AMIN
      AMOI7=AMIN
      APLU1=AMAX
      APLU2=AMAX
      APLU3=AMAX
      APLU4=AMAX
      APLU5=AMAX
      APLU6=AMAX
      APLU7=AMAX
      KAMOI1=KAMIN
      KAMOI2=KAMIN
      KAMOI3=KAMIN
      KAMOI4=KAMIN
      KAMOI5=KAMIN
      KAMOI6=KAMIN
      KAMOI7=KAMIN
      KAPLU1=KAMAX
      KAPLU2=KAMAX
      KAPLU3=KAMAX
      KAPLU4=KAMAX
      KAPLU5=KAMAX
      KAPLU6=KAMAX
      KAPLU7=KAMAX
      S1=Q(2)-Q(1)
      ERRD=EPSQ(1)+EPSQ(2)
      MD12=1
      IIAA=0
      IIDD=0
      JJAA=0
      JJDD=0
      DMAX=AMIN 1(SQRT(ZCO*AMAX*BMAX),DMAX)
      BMIN=AMAX 1((S1-ERRD-DMAX),BMIN)
      EMAX=AMIN 1(2*SQRT(BMAX*CMAX),EMAX)
      FMAX=AMIN 1(SQRT(ZCO*AMAX*CMAX),FMAX)
      GO TO 2038
C 4
2104  AMIN=Q(1)-EPSQ(1)
      AMAX=Q(1)+EPSQ(1)
      KAMIN=COEFF*AMIN
      KAMAX=COEFF*AMAX
      AMOI1=AMIN
      AMOI2=AMIN
      AMOI3=AMIN
      AMOI4=AMIN
      AMOI5=AMIN
      AMOI6=AMIN
      AMOI7=AMIN
      APLU1=AMAX
      APLU2=AMAX
      APLU3=AMAX
      APLU4=AMAX
      APLU5=AMAX
      APLU6=AMAX
      APLU7=AMAX
      KAMOI1=KAMIN
      KAMOI2=KAMIN
      KAMOI3=KAMIN
      KAMOI4=KAMIN
      KAMOI5=KAMIN
      KAMOI6=KAMIN
      KAMOI7=KAMIN
      KAPLU1=KAMAX
      KAPLU2=KAMAX
      KAPLU3=KAMAX
      KAPLU4=KAMAX
      KAPLU5=KAMAX
      KAPLU6=KAMAX
      KAPLU7=KAMAX
      S8=Q(2)
      ERRE=EPSQ(2)
      JJEE=0
      JJAA=0
      IIEE=0
      IIAA=0
      MD89=1
      DMAX=AMIN 1(SQRT(ZCO*AMAX*BMAX),DMAX)
      FMAX=AMIN 1(SQRT(ZCO*AMAX*CMAX),FMAX)
      AMABC=2.*(AMAX 1(BMAX,CMAX))
      CMIN=AMAX 1(S8-BMAX-ERRE-AMABC,CMIN)
      BMIN=AMAX 1(S8-CMAX-ERRE-AMABC,BMIN)
      CMAX=AMIN 1(CMAX,S8-BMIN+AMABC+ERRE)
      BMAX=AMIN 1(BMAX,S8-CMIN+AMABC+ERRE)
      DMAX=AMIN 1(SQRT(ZCO*AMAX*BMAX),DMAX)
      FMAX=AMIN 1(SQRT(ZCO*AMAX*CMAX),FMAX)
      EMAX=AMIN 1(2*SQRT(BMAX*CMAX),EMAX)
      GO TO 2038
C 6
2106  IRJ(1,7)=1
      AMAX=Q(1)+EPSQ(1)
      AMIN=Q(1)-EPSQ(1)
      BMAX=Q(2)/4.+EPSQ(2)/4.
      BMIN=Q(2)/4.-EPSQ(2)/4.
      KAMIN=COEFF*AMIN
      KAMAX=COEFF*AMAX
      KBMIN=COEFF*BMIN
      KBMAX=COEFF*BMAX
      AMN=(AMIN+AMAX)/2.
      BMN=(BMIN+BMAX)/2.
      AMOI1=AMIN
      AMOI2=AMIN
      AMOI3=AMIN
      AMOI4=AMIN
      AMOI5=AMIN
      AMOI6=AMIN
      AMOI7=AMIN
      APLU1=AMAX
      APLU2=AMAX
      APLU3=AMAX
      APLU4=AMAX
      APLU5=AMAX
      APLU6=AMAX
      APLU7=AMAX
      BMOI1=BMIN
      BMOI2=BMIN
      BMOI3=BMIN
      BMOI4=BMIN
      BMOI5=BMIN
      BMOI6=BMIN
      BMOI7=BMIN
      BPLU1=BMAX
      BPLU2=BMAX
      BPLU3=BMAX
      BPLU4=BMAX
      BPLU5=BMAX
      BPLU6=BMAX
      BPLU7=BMAX
      KAMOI1=KAMIN
      KAMOI2=KAMIN
      KAMOI3=KAMIN
      KAMOI4=KAMIN
      KAMOI5=KAMIN
      KAMOI6=KAMIN
      KAMOI7=KAMIN
      KAPLU1=KAMAX
      KAPLU2=KAMAX
      KAPLU3=KAMAX
      KAPLU4=KAMAX
      KAPLU5=KAMAX
      KAPLU6=KAMAX
      KAPLU7=KAMAX
      KBMOI1=KBMIN
      KBMOI2=KBMIN
      KBMOI3=KBMIN
      KBMOI4=KBMIN
      KBMOI5=KBMIN
      KBMOI6=KBMIN
      KBMOI7=KBMIN
      KBPLU1=KBMAX
      KBPLU2=KBMAX
      KBPLU3=KBMAX
      KBPLU4=KBMAX
      KBPLU5=KBMAX
      KBPLU6=KBMAX
      KBPLU7=KBMAX
      IIAA=0
      IIBB=0
      JJAA=0
      JJBB=0
      DMAX=AMIN 1(SQRT(ZCO*AMAX*BMAX),DMAX)
      FMAX=AMIN 1(SQRT(ZCO*AMAX*CMAX),FMAX)
      CMIN=AMAX 1(CMIN,EMIN*EMIN/BMAX/2.)
      CMIN=AMAX 1(CMIN,FMIN*FMIN/AMAX/ZCO)
      EMAX=AMIN 1(2*SQRT(BMAX*CMAX),EMAX)
      DMI=DMIN
      DMA=DMAX
      CMA=CMAX
      CMI=CMIN
      FMA=FMAX
      EMA=EMAX
      CALL RELSPE
      IF(JRD.EQ.0)GO TO 2038
      JJDD=0
 4006 DMIN=RMIN(JRT)
      DMAX=RMAX(JRT)
      DMAX=AMIN 1(SQRT(ZCO*AMAX*BMAX),DMAX)
      KDMIN=COEFF*DMIN
      KDMAX=COEFF*DMAX
      DMOI1=DMIN
      DMOI2=DMIN
      DMOI3=DMIN
      DMOI4=DMIN
      DMOI5=DMIN
      DMOI6=DMIN
      DMOI7=DMIN
      DPLU1=DMAX
      DPLU2=DMAX
      DPLU3=DMAX
      DPLU4=DMAX
      DPLU5=DMAX
      DPLU6=DMAX
      DPLU7=DMAX
      KDMOI1=KDMIN
      KDMOI2=KDMIN
      KDMOI3=KDMIN
      KDMOI4=KDMIN
      KDMOI5=KDMIN
      KDMOI6=KDMIN
      KDMOI7=KDMIN
      KDPLU1=KDMAX
      KDPLU2=KDMAX
      KDPLU3=KDMAX
      KDPLU4=KDMAX
      KDPLU5=KDMAX
      KDPLU6=KDMAX
      KDPLU7=KDMAX
      CMAX=RCMAX(JRT)
      FMAX=AMIN 1(SQRT(ZCO*AMAX*CMAX),FMAX)
      CMIN=AMAX 1(CMIN,EMIN*EMIN/BMAX/2.)
      CMIN=AMAX 1(CMIN,FMIN*FMIN/AMAX/ZCO)
      EMAX=AMIN 1(2*SQRT(BMAX*CMAX),EMAX)
      ASSIGN 4006 TO IAS
      GO TO 2038
C 7
2107  IRJ(1,7)=1
      BMIN=Q(2)-EPSQ(2)
      BMAX=Q(2)+EPSQ(2)
      KBMIN=COEFF*BMIN
      KBMAX=COEFF*BMAX
      BMOI1=BMIN
      BMOI2=BMIN
      BMOI3=BMIN
      BMOI4=BMIN
      BMOI5=BMIN
      BMOI6=BMIN
      BMOI7=BMIN
      BPLU1=BMAX
      BPLU2=BMAX
      BPLU3=BMAX
      BPLU4=BMAX
      BPLU5=BMAX
      BPLU6=BMAX
      BPLU7=BMAX
      KBMOI1=KBMIN
      KBMOI2=KBMIN
      KBMOI3=KBMIN
      KBMOI4=KBMIN
      KBMOI5=KBMIN
      KBMOI6=KBMIN
      KBMOI7=KBMIN
      KBPLU1=KBMAX
      KBPLU2=KBMAX
      KBPLU3=KBMAX
      KBPLU4=KBMAX
      KBPLU5=KBMAX
      KBPLU6=KBMAX
      KBPLU7=KBMAX
      MD34=1
      S3=Q(1)-Q(2)
      ERRD=EPSQ(1)+EPSQ(2)
      JJBB=0
      JJDD=0
      IIBB=0
      IIDD=0
      DMAX=AMIN 1(SQRT(ZCO*AMAX*BMAX),DMAX)
      AMIN=AMAX 1(AMIN,(S3-ERRD-DMAX))
      EMAX=AMIN 1(2*SQRT(BMAX*CMAX),EMAX)
      CMIN=AMAX 1(CMIN,EMIN*EMIN/BMAX/2.)
      CMIN=AMAX 1(CMIN,FMIN*FMIN/AMAX/ZCO)
      GO TO 2038
C 9
2109  IRJ(1,7)=1
      AMAX=Q(1)/4.+EPSQ(1)/4.
      AMIN=Q(1)/4.-EPSQ(1)/4.
      BMAX=Q(2)+EPSQ(2)
      BMIN=Q(2)-EPSQ(2)
      KAMIN=COEFF*AMIN
      KAMAX=COEFF*AMAX
      KBMIN=COEFF*BMIN
      KBMAX=COEFF*BMAX
      AMN=(AMIN+AMAX)/2.
      BMN=(BMIN+BMAX)/2.
      AMOI1=AMIN
      AMOI2=AMIN
      AMOI3=AMIN
      AMOI4=AMIN
      AMOI5=AMIN
      AMOI6=AMIN
      AMOI7=AMIN
      APLU1=AMAX
      APLU2=AMAX
      APLU3=AMAX
      APLU4=AMAX
      APLU5=AMAX
      APLU6=AMAX
      APLU7=AMAX
      BMOI1=BMIN
      BMOI2=BMIN
      BMOI3=BMIN
      BMOI4=BMIN
      BMOI5=BMIN
      BMOI6=BMIN
      BMOI7=BMIN
      BPLU1=BMAX
      BPLU2=BMAX
      BPLU3=BMAX
      BPLU4=BMAX
      BPLU5=BMAX
      BPLU6=BMAX
      BPLU7=BMAX
      KAMOI1=KAMIN
      KAMOI2=KAMIN
      KAMOI3=KAMIN
      KAMOI4=KAMIN
      KAMOI5=KAMIN
      KAMOI6=KAMIN
      KAMOI7=KAMIN
      KAPLU1=KAMAX
      KAPLU2=KAMAX
      KAPLU3=KAMAX
      KAPLU4=KAMAX
      KAPLU5=KAMAX
      KAPLU6=KAMAX
      KAPLU7=KAMAX
      KBMOI1=KBMIN
      KBMOI2=KBMIN
      KBMOI3=KBMIN
      KBMOI4=KBMIN
      KBMOI5=KBMIN
      KBMOI6=KBMIN
      KBMOI7=KBMIN
      KBPLU1=KBMAX
      KBPLU2=KBMAX
      KBPLU3=KBMAX
      KBPLU4=KBMAX
      KBPLU5=KBMAX
      KBPLU6=KBMAX
      KBPLU7=KBMAX
      JJAA=0
      JJBB=0
      IIAA=0
      IIBB=0
      DMAX=AMIN 1(SQRT(ZCO*AMAX*BMAX),DMAX)
      FMAX=AMIN 1(SQRT(ZCO*AMAX*CMAX),FMAX)
      CMIN=AMAX 1(CMIN,EMIN*EMIN/BMAX/2.)
      CMIN=AMAX 1(CMIN,FMIN*FMIN/AMAX/ZCO)
      EMAX=AMIN 1(2*SQRT(BMAX*CMAX),EMAX)
      DMI=DMIN
      DMA=DMAX
      CMA=CMAX
      CMI=CMIN
      FMA=FMAX
      EMA=EMAX
      CALL RELSPE
      IF(JRD.EQ.0)GO TO 2038
      JJDD=0
 4009 DMIN=RMIN(JRT)
      DMAX=RMAX(JRT)
      DMAX=AMIN 1(SQRT(ZCO*AMAX*BMAX),DMAX)
      KDMIN=COEFF*DMIN
      KDMAX=COEFF*DMAX
      DMOI1=DMIN
      DMOI2=DMIN
      DMOI3=DMIN
      DMOI4=DMIN
      DMOI5=DMIN
      DMOI6=DMIN
      DMOI7=DMIN
      DPLU1=DMAX
      DPLU2=DMAX
      DPLU3=DMAX
      DPLU4=DMAX
      DPLU5=DMAX
      DPLU6=DMAX
      DPLU7=DMAX
      KDMOI1=KDMIN
      KDMOI2=KDMIN
      KDMOI3=KDMIN
      KDMOI4=KDMIN
      KDMOI5=KDMIN
      KDMOI6=KDMIN
      KDMOI7=KDMIN
      KDPLU1=KDMAX
      KDPLU2=KDMAX
      KDPLU3=KDMAX
      KDPLU4=KDMAX
      KDPLU5=KDMAX
      KDPLU6=KDMAX
      KDPLU7=KDMAX
      CMAX=RCMAX(JRT)
      EMAX=AMIN 1(2*SQRT(BMAX*CMAX),EMAX)
      FMAX=AMIN 1(SQRT(ZCO*AMAX*CMAX),FMAX)
      CMIN=AMAX 1(CMIN,EMIN*EMIN/BMAX/2.)
      CMIN=AMAX 1(CMIN,FMIN*FMIN/AMAX/ZCO)
      ASSIGN 4009 TO IAS
      GO TO 2038
C 10
2110  IRJ(1,7)=1
      CMIN=Q(2)-EPSQ(2)
      CMAX=Q(2)+EPSQ(2)
      KCMIN=COEFF*CMIN
      KCMAX=COEFF*CMAX
      CMOI1=CMIN
      CMOI2=CMIN
      CMOI3=CMIN
      CMOI4=CMIN
      CMOI5=CMIN
      CMOI6=CMIN
      CMOI7=CMIN
      CPLU1=CMAX
      CPLU2=CMAX
      CPLU3=CMAX
      CPLU4=CMAX
      CPLU5=CMAX
      CPLU6=CMAX
      CPLU7=CMAX
      KCMOI1=KCMIN
      KCMOI2=KCMIN
      KCMOI3=KCMIN
      KCMOI4=KCMIN
      KCMOI5=KCMIN
      KCMOI6=KCMIN
      KCMOI7=KCMIN
      KCPLU1=KCMAX
      KCPLU2=KCMAX
      KCPLU3=KCMAX
      KCPLU4=KCMAX
      KCPLU5=KCMAX
      KCPLU6=KCMAX
      KCPLU7=KCMAX
      S5=Q(1)
      ERRD=EPSQ(1)
      JJCC=0
      JJDD=0
      IICC=0
      IIDD=0
      MD56=1
      AMAX=AMIN 1(S5+ERRD-BMIN+DMAX,AMAX)
      BMAX=AMIN 1(S5+ERRD-AMIN+DMAX,BMAX)
      AMAAB=SQRT(4.)*AMAX 1(AMAX,BMAX)
      AMIN=AMAX 1(AMIN,S5-ERRD-BMAX-AMAAB)
      BMIN=AMAX 1(BMIN,S5-ERRD-AMAX-AMAAB)
      DMAX=AMIN 1(SQRT(ZCO*AMAX*BMAX),DMAX)
      FMAX=AMIN 1(SQRT(ZCO*AMAX*CMAX),FMAX)
      EMAX=AMIN 1(2*SQRT(BMAX*CMAX),EMAX)
      GO TO 2038
C 11
2111  IRJ(1,7)=1
      AMIN=Q(1)/4.-EPSQ(1)/4.
      AMAX=Q(1)/4.+EPSQ(1)/4.
      KAMIN=COEFF*AMIN
      KAMAX=COEFF*AMAX
      AMOI1=AMIN
      AMOI2=AMIN
      AMOI3=AMIN
      AMOI4=AMIN
      AMOI5=AMIN
      AMOI6=AMIN
      AMOI7=AMIN
      APLU1=AMAX
      APLU2=AMAX
      APLU3=AMAX
      APLU4=AMAX
      APLU5=AMAX
      APLU6=AMAX
      APLU7=AMAX
      KAMOI1=KAMIN
      KAMOI2=KAMIN
      KAMOI3=KAMIN
      KAMOI4=KAMIN
      KAMOI5=KAMIN
      KAMOI6=KAMIN
      KAMOI7=KAMIN
      KAPLU1=KAMAX
      KAPLU2=KAMAX
      KAPLU3=KAMAX
      KAPLU4=KAMAX
      KAPLU5=KAMAX
      KAPLU6=KAMAX
      KAPLU7=KAMAX
      S1=Q(2)-Q(1)/4.
      ERRD=EPSQ(2)+EPSQ(1)/4.
      JJAA=0
      JJDD=0
      IIAA=0
      IIDD=0
      MD12=1
      BMIN=AMAX 1(BMIN,(S1-ERRD-AMAX-DMAX))
      EMAX=AMIN 1(2*SQRT(BMAX*CMAX),EMAX)
      DMAX=AMIN 1(SQRT(ZCO*AMAX*BMAX),DMAX)
      FMAX=SQRT(ZCO*AMAX*CMAX)
      CMIN=AMAX 1(CMIN,EMIN*EMIN/BMAX/2.)
      CMIN=AMAX 1(CMIN,FMIN*FMIN/AMAX/ZCO)
      GO TO 2038
C 12
2112  IRJ(1,7)=1
      CMIN=Q(2)/4.-EPSQ(2)
      CMAX=Q(2)/4.+EPSQ(2)
      KCMIN=COEFF*CMIN
      KCMAX=COEFF*CMAX
      CMOI1=CMIN
      CMOI2=CMIN
      CMOI3=CMIN
      CMOI4=CMIN
      CMOI5=CMIN
      CMOI6=CMIN
      CMOI7=CMIN
      CPLU1=CMAX
      CPLU2=CMAX
      CPLU3=CMAX
      CPLU4=CMAX
      CPLU5=CMAX
      CPLU6=CMAX
      CPLU7=CMAX
      KCMOI1=KCMIN
      KCMOI2=KCMIN
      KCMOI3=KCMIN
      KCMOI4=KCMIN
      KCMOI5=KCMIN
      KCMOI6=KCMIN
      KCMOI7=KCMIN
      KCPLU1=KCMAX
      KCPLU2=KCMAX
      KCPLU3=KCMAX
      KCPLU4=KCMAX
      KCPLU5=KCMAX
      KCPLU6=KCMAX
      KCPLU7=KCMAX
      S5=Q(1)
      ERRD=EPSQ(1)
      JJCC=0
      JJDD=0
      IICC=0
      IIDD=0
      MD56=1
      AMAX=AMIN 1(S5+ERRD-BMIN+DMAX,AMAX)
      BMAX=AMIN 1(S5+ERRD-AMIN+DMAX,BMAX)
      AMAAB=SQRT(4.)*AMAX 1(AMAX,BMAX)
      AMIN=AMAX 1(AMIN,S5-ERRD-BMAX-AMAAB)
      BMIN=AMAX 1(BMIN,S5-ERRD-AMAX-AMAAB)
      DMAX=AMIN 1(SQRT(ZCO*AMAX*BMAX),DMAX)
      FMAX=AMIN 1(SQRT(ZCO*AMAX*CMAX),FMAX)
      EMAX=AMIN 1(2*SQRT(BMAX*CMAX),EMAX)
      GO TO 2038
C 13
2113  IRJ(1,7)=1
      AMIN=Q(1)/4.-EPSQ(1)
      AMAX=Q(1)/4.+EPSQ(1)
      KAMIN=COEFF*AMIN
      KAMAX=COEFF*AMAX
      AMOI1=AMIN
      AMOI2=AMIN
      AMOI3=AMIN
      AMOI4=AMIN
      AMOI5=AMIN
      AMOI6=AMIN
      AMOI7=AMIN
      APLU1=AMAX
      APLU2=AMAX
      APLU3=AMAX
      APLU4=AMAX
      APLU5=AMAX
      APLU6=AMAX
      APLU7=AMAX
      KAMOI1=KAMIN
      KAMOI2=KAMIN
      KAMOI3=KAMIN
      KAMOI4=KAMIN
      KAMOI5=KAMIN
      KAMOI6=KAMIN
      KAMOI7=KAMIN
      KAPLU1=KAMAX
      KAPLU2=KAMAX
      KAPLU3=KAMAX
      KAPLU4=KAMAX
      KAPLU5=KAMAX
      KAPLU6=KAMAX
      KAPLU7=KAMAX
      S8=Q(2)
      ERRE=EPSQ(2)
      JJAA=0
      JJEE=0
      IIAA=0
      IIEE=0
      MD89=1
      AMABC=2.*(AMAX 1(BMAX,CMAX))
      CMIN=AMAX 1(S8-BMAX-ERRE-AMABC,CMIN)
      BMIN=AMAX 1(S8-CMAX-ERRE-AMABC,BMIN)
      CMAX=AMIN 1(CMAX,S8-BMIN+AMABC+ERRE)
      BMAX=AMIN 1(BMAX,S8-CMIN+AMABC+ERRE)
      DMAX=AMIN 1(SQRT(ZCO*AMAX*BMAX),DMAX)
      EMAX=AMIN 1(2*SQRT(BMAX*CMAX),EMAX)
      FMAX=AMIN 1(SQRT(ZCO*AMAX*CMAX),FMAX)
      GO TO 2038
C 18
2118  IRJ(1,7)=1
      BMIN=Q(2)/4.-EPSQ(2)/4.
      BMAX=Q(2)/4.+EPSQ(2)/4.
      KBMIN=COEFF*BMIN
      KBMAX=COEFF*BMAX
      BMOI1=BMIN
      BMOI2=BMIN
      BMOI3=BMIN
      BMOI4=BMIN
      BMOI5=BMIN
      BMOI6=BMIN
      BMOI7=BMIN
      BPLU1=BMAX
      BPLU2=BMAX
      BPLU3=BMAX
      BPLU4=BMAX
      BPLU5=BMAX
      BPLU6=BMAX
      BPLU7=BMAX
      KBMOI1=KBMIN
      KBMOI2=KBMIN
      KBMOI3=KBMIN
      KBMOI4=KBMIN
      KBMOI5=KBMIN
      KBMOI6=KBMIN
      KBMOI7=KBMIN
      KBPLU1=KBMAX
      KBPLU2=KBMAX
      KBPLU3=KBMAX
      KBPLU4=KBMAX
      KBPLU5=KBMAX
      KBPLU6=KBMAX
      KBPLU7=KBMAX
      S3=Q(1)-Q(2)/4.
      ERRD=EPSQ(1)+EPSQ(2)/4.
      JJBB=0
      JJDD=0
      IIBB=0
      IIDD=0
      MD34=1
      DMAX=AMIN 1(SQRT(ZCO*AMAX*BMAX),DMAX)
      AMIN=AMAX 1(AMIN,(S3-ERRD-DMAX))
      EMAX=AMIN 1(2*SQRT(BMAX*CMAX),EMAX)
      FMAX=AMIN 1(SQRT(ZCO*AMAX*CMAX),FMAX)
      GO TO 2038
C 20
2120  S5=Q(1)
      S8=Q(2)
      ERRD=EPSQ(1)
      ERRE=EPSQ(2)
      MD56=1
      JJDD=0
      JJEE=0
      IIDD=0
      MD89=1
      IIEE=0
      AMABC=2.*(AMAX 1(BMAX,CMAX))
      CMIN=AMAX 1(S8-BMAX-ERRE-AMABC,CMIN)
      BMIN=AMAX 1(S8-CMAX-ERRE-AMABC,BMIN)
      CMAX=AMIN 1(CMAX,S8-BMIN+AMABC+ERRE)
      BMAX=AMIN 1(BMAX,S8-CMIN+AMABC+ERRE)
      AMAX=AMIN 1(S5+ERRD-BMIN+DMAX,AMAX)
      BMAX=AMIN 1(S5+ERRD-AMIN+DMAX,BMAX)
      AMAAB=SQRT(4.)*AMAX 1(AMAX,BMAX)
      AMIN=AMAX 1(AMIN,S5-ERRD-BMAX-AMAAB)
      BMIN=AMAX 1(BMIN,S5-ERRD-AMAX-AMAAB)
      DMAX=AMIN 1(SQRT(ZCO*AMAX*BMAX),DMAX)
      FMAX=AMIN 1(SQRT(ZCO*AMAX*CMAX),FMAX)
      EMAX=AMIN 1(2*SQRT(BMAX*CMAX),EMAX)
      GO TO 2038
C 21
2121  IRJ(1,7)=1
      ERRB=(EPSQ(1)+EPSQ(2))/2.
      DMIN=(Q(2)-Q(1))/2.-ERRB
      DMAX=(Q(2)-Q(1))/2.+ERRB
      KDMIN=COEFF*DMIN
      KDMAX=COEFF*DMAX
      DMOI1=DMIN
      DMOI2=DMIN
      DMOI3=DMIN
      DMOI4=DMIN
      DMOI5=DMIN
      DMOI6=DMIN
      DMOI7=DMIN
      DPLU1=DMAX
      DPLU2=DMAX
      DPLU3=DMAX
      DPLU4=DMAX
      DPLU5=DMAX
      DPLU6=DMAX
      DPLU7=DMAX
      KDMOI1=KDMIN
      KDMOI2=KDMIN
      KDMOI3=KDMIN
      KDMOI4=KDMIN
      KDMOI5=KDMIN
      KDMOI6=KDMIN
      KDMOI7=KDMIN
      KDPLU1=KDMAX
      KDPLU2=KDMAX
      KDPLU3=KDMAX
      KDPLU4=KDMAX
      KDPLU5=KDMAX
      KDPLU6=KDMAX
      KDPLU7=KDMAX
      S7=(Q(1)+Q(2))/2.
      JJBB=0
      JJDD=0
      IIBB=0
      IIDD=0
      AMAX=AMIN 1(AMAX,S7+ERRB-BMIN)
      BMAX=AMIN 1(BMAX,S7+ERRB-AMIN)
      AMIN=AMAX 1(AMIN,S7-ERRD-BMAX)
      BMIN=AMAX 1(BMIN,S7-ERRD-AMAX)
      EMAX=AMIN 1(EMAX,2.*SQRT(BMAX*CMAX))
      FMAX=AMIN 1(SQRT(ZCO*AMAX*CMAX),FMAX)
      MD7=1
      GO TO 2038
C 23
2123  IRJ(1,7)=1
      AMIN=Q(1)/4.-EPSQ(1)/4.
      AMAX=Q(1)/4.+EPSQ(1)/4.
      BMIN=Q(2)/4.-EPSQ(2)/4.
      BMAX=Q(2)/4.+EPSQ(2)/4.
      AMN=(AMAX+AMIN)/2.
      BMN=(BMAX-BMIN)/2.
      JJAA=0
      JJBB=0
      IIAA=0
      IIBB=0
      AMOI1=AMIN
      AMOI2=AMIN
      AMOI3=AMIN
      AMOI4=AMIN
      AMOI5=AMIN
      AMOI6=AMIN
      AMOI7=AMIN
      APLU1=AMAX
      APLU2=AMAX
      APLU3=AMAX
      APLU4=AMAX
      APLU5=AMAX
      APLU6=AMAX
      APLU7=AMAX
      KAMOI1=KAMIN
      KAMOI2=KAMIN
      KAMOI3=KAMIN
      KAMOI4=KAMIN
      KAMOI5=KAMIN
      KAMOI6=KAMIN
      KAMOI7=KAMIN
      KAPLU1=KAMAX
      KAPLU2=KAMAX
      KAPLU3=KAMAX
      KAPLU4=KAMAX
      KAPLU5=KAMAX
      KAPLU6=KAMAX
      KAPLU7=KAMAX
      BMOI1=BMIN
      BMOI2=BMIN
      BMOI3=BMIN
      BMOI4=BMIN
      BMOI5=BMIN
      BMOI6=BMIN
      BMOI7=BMIN
      BPLU1=BMAX
      BPLU2=BMAX
      BPLU3=BMAX
      BPLU4=BMAX
      BPLU5=BMAX
      BPLU6=BMAX
      BPLU7=BMAX
      KBMOI1=KBMIN
      KBMOI2=KBMIN
      KBMOI3=KBMIN
      KBMOI4=KBMIN
      KBMOI5=KBMIN
      KBMOI6=KBMIN
      KBMOI7=KBMIN
      KBPLU1=KBMAX
      KBPLU2=KBMAX
      KBPLU3=KBMAX
      KBPLU4=KBMAX
      KBPLU5=KBMAX
      KBPLU6=KBMAX
      KBPLU7=KBMAX
      DMAX=AMIN 1(SQRT(ZCO*AMAX*BMAX),DMAX)
      FMAX=AMIN 1(SQRT(ZCO*AMAX*CMAX),FMAX)
      CMIN=AMAX 1(CMIN,EMIN*EMIN/BMAX/2.)
      CMIN=AMAX 1(CMIN,FMIN*FMIN/AMAX/ZCO)
      EMAX=AMIN 1(2*SQRT(BMAX*CMAX),EMAX)
      DMI=DMIN
      DMA=DMAX
      CMA=CMAX
      CMI=CMIN
      FMA=FMAX
      EMA=EMAX
      CALL RELSPE
      IF(JRD.EQ.0)GO TO 2038
      JJDD=0
 4023 DMIN=RMIN(JRT)
      DMAX=RMAX(JRT)
      DMAX=AMIN 1(SQRT(ZCO*AMAX*BMAX),DMAX)
      KDMIN=COEFF*DMIN
      KDMAX=COEFF*DMAX
      DMOI1=DMIN
      DMOI2=DMIN
      DMOI3=DMIN
      DMOI4=DMIN
      DMOI5=DMIN
      DMOI6=DMIN
      DMOI7=DMIN
      DPLU1=DMAX
      DPLU2=DMAX
      DPLU3=DMAX
      DPLU4=DMAX
      DPLU5=DMAX
      DPLU6=DMAX
      DPLU7=DMAX
      KDMOI1=KDMIN
      KDMOI2=KDMIN
      KDMOI3=KDMIN
      KDMOI4=KDMIN
      KDMOI5=KDMIN
      KDMOI6=KDMIN
      KDMOI7=KDMIN
      KDPLU1=KDMAX
      KDPLU2=KDMAX
      KDPLU3=KDMAX
      KDPLU4=KDMAX
      KDPLU5=KDMAX
      KDPLU6=KDMAX
      KDPLU7=KDMAX
      CMAX=RCMAX(JRT)
      EMAX=AMIN 1(2*SQRT(BMAX*CMAX),EMAX)
      FMAX=AMIN 1(SQRT(ZCO*AMAX*CMAX),FMAX)
      CMIN=AMAX 1(CMIN,EMIN*EMIN/BMAX/2.)
      CMIN=AMAX 1(CMIN,FMIN*FMIN/AMAX/ZCO)
      ASSIGN 4023 TO IAS
2038  NA=(AMAX-AMIN)/PASA
      NB=(BMAX-BMIN)/PASB
      NC=(CMAX-CMIN)/PASC
      ND=(DMAX-DMIN)/PASD
      NE=(EMAX-EMIN)/PASE
      NF=(FMAX-FMIN)/PASF
      IF(NA.LT.0.OR.NB.LT.0.OR.NC.LT.0.OR.ND.LT.0.OR.NE.LT.0.OR.NF.LT.0)
     +GO TO 80008
      IF(IIAA.EQ.0)GO TO 20012
      IA=0
1010  AMOI1=AMIN+IA*PASA
      APLU1=AMOI1+PASA
      KAMOI1=COEFF*AMOI1
      KAPLU1=COEFF*APLU1
      IF(MD34.EQ.1)THEN
      DMOI1=AMOI1-S3-ERRD
      DPLU1=APLU1-S3+ERRD
      DMOIM=AMIN 1(ABS(DMOI1),ABS(DPLU1))
      IF(DMOIM.GT.DMAX.AND.DMOI1.GE.0.)GO TO 80008
      IF(DMOIM.GT.DMAX)GO TO 1000
      IF(DMOI1.LT.(-DMAX))DMOI1=-DMAX
      IF(DPLU1.GT.DMAX)DPLU1=DMAX
      ZZO=ZCO
      IF(DMOIM*DMOIM.GE.ZZO*APLU1*BPLU1.AND.DMOI1.GE.0.)GO TO 80008
      IF(DMOIM*DMOIM.GE.ZZO*APLU1*BPLU1)GO TO 1000
      KDMOI1=COEFF*DMOI1
      KDPLU1=COEFF*DPLU1
      ELSE
      IF(MD7.EQ.0)GO TO 20012
      BMOI1=S7-APLU1-ERRB
      IF(BMOI1.GT.BMAX)GO TO 1000
      BPLU1=S7-AMOI1+ERRB
      IF(BMOI1.GT.APLU1)GO TO 1000
      IF(BPLU1.LE.0)GO TO 80008
      IF(BMOI1.LE.0)BMOI1=BMIN
      IF((DMOI1-1E-8)**2.GE.ZCO*APLU1*BPLU1)GO TO 1000
      KBMOI1=COEFF*BMOI1
      KBPLU1=COEFF*BPLU1
      ENDIF
20012 IF(IIBB.EQ.0)GO TO 20013
      IB=0
1110  BMOI1=BMIN+IB*PASB
      BPLU1=BMOI1+PASB
      KBMOI1=COEFF*BMOI1
      KBPLU1=COEFF*BPLU1
      IF(MD12.EQ.1)THEN
      DMOI1=(BMOI1-S1)-ERRD
      DPLU1=(BPLU1-S1)+ERRD
      DMOIM=AMIN 1(ABS(DMOI1),ABS(DPLU1))
      IF(DMOIM.GT.DMAX.AND.DMOI1.GE.0.)GO TO 80008
      IF(DMOIM.GT.DMAX)GO TO 1100
      IF(DMOI1.LT.(-DMAX))DMOI1=-DMAX
      IF(DPLU1.GT.DMAX)DPLU1=DMAX
      ZZO=ZCO
      IF(DMOIM*DMOIM.GE.ZZO*APLU1*BPLU1.AND.DMOI1.GE.0.)GO TO 80008
      IF(DMOIM*DMOIM.GE.ZZO*APLU1*BPLU1)GO TO 1100
      KDMOI1=COEFF*DMOI1
      KDPLU1=COEFF*DPLU1
      ELSE
      IF(MD56.EQ.0)GO TO 20013
      DMOI1=(S5-APLU1-BPLU1)-ERRD
      DPLU1=(S5-AMOI1-BMOI1)+ERRD
      DMOIM=AMIN 1(ABS(DMOI1),ABS(DPLU1))
      IF(DMOIM.GT.DMAX.AND.DPLU1.LE.0.)GO TO 1000
      IF(DMOIM.GT.DMAX)GO TO 1100
      IF(DMOI1.LT.(-DMAX))DMOI1=-DMAX
      IF(DPLU1.GT.DMAX)DPLU1=DMAX
      ZZO=ZCO
      IF(DMOIM*DMOIM.GE.ZZO*APLU1*BPLU1.AND.DPLU1.LE.0.)GO TO 1000
      IF(DMOIM*DMOIM.GE.ZZO*APLU1*BPLU1)GO TO 1100
      KDMOI1=COEFF*DMOI1
      KDPLU1=COEFF*DPLU1
      ENDIF
20013 IF(IICC.EQ.0)GO TO 20014
      IC=0
1210  CMOI1=CMIN+IC*PASC
      IF(CMOI1.GT.(CMAX+1E-8))GO TO 11000
      CPLU1=CMOI1+PASC
      KCMOI1=COEFF*CMOI1
      KCPLU1=COEFF*CPLU1
      IF(MD89.EQ.0)GO TO 20014
      EMOI1=(BMOI1+CMOI1-S8)-ERRE
      EPLU1=(BPLU1+CPLU1-S8)+ERRE
      EMMA1=AMIN 1(ABS(EMOI1),ABS(EPLU1))
      IF(EMMA1.GT.EMAX.AND.EMOI1.GE.0)GO TO 1100
      IF(EMMA1.GT.EMAX)GO TO 1200
      IF(EMOI1.LT.(-EMAX))EMOI1=-EMAX
      IF(EPLU1.GT.EMAX)EPLU1=EMAX
      ZZO=ZCO
      IF(EMOI1.LT.0.)ZZO=4.
      EMMI1=AMIN 1(ABS(EMOI1),ABS(EPLU1))
      IF(EMMI1*EMMI1.GE.ZZO*BPLU1*CPLU1.AND.EMOI1.GE.0)GO TO 1100
      IF(EMMI1*EMMI1.GE.ZZO*BPLU1*CPLU1)GO TO 1200
      KEMOI1=COEFF*EMOI1
      KEPLU1=COEFF*EPLU1
20014 IF(IIDD.EQ.0)GO TO 20015
      ID=0
1310  DMOI1=DMIN+ID*PASD
      IF(DMOI1.GT.(DMAX+1E-8))GO TO 12000
      DPLU1=DMOI1+PASD
      IF((DMOI1-1E-8)**2.GT.ZCO*APLU1*BPLU1)GO TO 12000
      KDMOI1=COEFF*DMOI1
      KDPLU1=COEFF*DPLU1
20015 IF(IIEE.EQ.0)GO TO 20016
      IE=-NE
1410  EMOI1=EMIN+IE*PASE
      EPLU1=EMOI1+PASE
      IF(EMOI1.LT.0.)GO TO 3230
      IF(EMOI1.GT.(EMAX+1E-8))GO TO 13000
      IF((EMOI1-1E-8).GT.SQRT(ZCO*BPLU1*CPLU1))GO TO 13000
      GO TO 3231
3230  IF(ABS(EPLU1-1E-8).GT.EMAX)GO TO 1400
      IF(ABS(EPLU1-1E-8).GT.SQRT(4.*BPLU1*CPLU1))GO TO 1400
3231  KEMOI1=COEFF*EMOI1
      KEPLU1=COEFF*EPLU1
20016 IF(IIFF.EQ.0)GO TO 14000
      MT=-1
      DMOIR1=AMIN 1(ABS(DMOI1),ABS(DPLU1))
      DPLUR1=AMAX 1(ABS(DMOI1),ABS(DPLU1))
      DO 1500 IMF=0,NF
      FMOI1=FMIN+IMF*PASF
      IF(FMOI1.GT.(FMAX-1E-8))GO TO 14000
      FPLU1=FMOI1+PASF
      IF((FMOI1-1E-8)**2.GT.ZCO*APLU1*CPLU1)GO TO 14000
      IF(FPLU1**2.GE.4.*AMOI1*CMOI1.OR.DPLUR1**2.GT.4.*AMOI1
     +*BMOI1)GO TO 20091
      XABC=2*APLU1*EPLU1-DMOI1*FMOI1
      XABC=XABC+SQRT(16*APLU1**2*BPLU1*CPLU1-4*APLU1*BPLU1
     +*FMOI1**2-4*APLU1*CPLU1*DMOIR1**2+(DPLUR1*FMOI1)**2)
      IF((XABC-1E-8).LT.0.)GO TO 15000
      IF(SQRT(BPLU1/CPLU1)*FPLU1.GE.DMOI1.AND.SQRT(BPLU1/CPLU1)
     +*FMOI1.LE.DPLU1)THEN
      ALET5=-2*EMOI1*APLU1+4*SQRT(CPLU1*BPLU1)*APLU1
      ELSE
      ALET0=16*APLU1*APLU1*BPLU1*CPLU1
      ALET1=DPLUR1*FMOI1-2*EMOI1*AMOI1+SQRT(ALET0-4*APLU1*BPLU1*FMOI1
     +**2-4*APLU1*CPLU1*DMOIR1**2+(DPLUR1*FMOI1)**2)
      AMBD1=0.
      IF(4*APLU1*CPLU1.GE.FPLU1**2)AMBD1=SQRT(ALET0
     +-4*APLU1*BPLU1*FPLU1**2-4*APLU1*CPLU1*DMOIR1**2+(DPLUR1*FPLU1)**2)
      ALET2=DPLUR1*FPLU1-2*EMOI1*AMOI1+AMBD1
      AMBD2=0.
      IF(4*APLU1*BPLU1.LT.DPLUR1**2)AMBD2=SQRT(ALET0
     +-4*APLU1*BPLU1*FMOI1**2-4*APLU1*CPLU1*DMOIR1**2+(DPLUR1*FMOI1)**2)
      ALET3=DPLUR1*FMOI1-2*EMOI1*AMOI1+AMBD2
      ABD=AMBD1*AMBD2
      ALET4=DPLUR1*FPLU1-2*EMOI1*AMOI1
      IF(ABD.NE.0.)ALET4=ALET4+SQRT(ALET0-4*APLU1*BPLU1*FPLU1**2
     +-4*APLU1*CPLU1*DMOIR1**2+(DPLUR1*FPLU1)**2)
      ALET5=AMAX 1(ALET1,ALET2,ALET3,ALET4)
      ENDIF
      IF((ALET5-1E-8).LT.0.)GO TO 15000
      GO TO 20092
20091 IF((2.*EPLU1*APLU1-DMOI1*FMOI1+SQRT(16.*APLU1*APLU1*BPLU1*CPLU1-4.
     +*AMOI1*BMOI1*FMOI1**2-4.*AMOI1*CMOI1*DMOIR1**2+(DPLUR1*FPLU1)**
     +2)).LT.0.)GO TO 15000
      IF((DPLU1*FPLU1-2.*EMOI1*AMOI1+SQRT(16.*APLU1*APLU1*BPLU1*CPLU1-4.
     +*AMOI1*BMOI1*FMOI1**2-4.*AMOI1*CMOI1*DMOIR1**2+(DPLUR1*FPLU1)**
     +2)).LT.0.)GO TO 14000
20092 KFMOI1=COEFF*FMOI1
      KFPLU1=COEFF*FPLU1
      DO 30011 I=1,KT
      ATEST=ABS(AMOI1-AAS(I))
      BTEST=ABS(BMOI1-BBS(I))
      CTEST=ABS(CMOI1-CCS(I))
      DTEST=ABS(DMOI1-DDS(I))
      ETEST=ABS(EMOI1-EES(I))
      ETESTY=ABS(ABS(EMOI1)-ABS(EES(I)))
      FTEST=ABS(FMOI1-FFS(I))
      IF(ATEST.LT.CXA.AND.BTEST.LT.CXB.AND.CTEST.LT.CXC.AND.DTEST.LT.
     +CXD.AND.ETEST.LT.CXE.AND.FTEST.LT.CXF)GO TO 15000
      IF(ATEST.LT.CYA.AND.BTEST.LT.CYA.AND.CTEST.LT.CYA.AND.DTEST.LT.
     +CYA.AND.ETESTY.LT.CYA.AND.FTEST.LT.CYA)GO TO 15000
30011 CONTINUE
      CALL VOLUME(AMOI1,BMOI1,CMOI1,DMOI1,EMOI1,FMOI1,
     +APLU1,BPLU1,CPLU1,DPLU1,EPLU1,FPLU1)
      IF(LY.EQ.-1)GO TO 15000
      IF(MT.EQ.-1)GO TO 2052
      CALL TRIHKLB(KAMOI1,KBMOI1,KCMOI1,KDMOI1,KEMOI1,KFMOI1,
     +KAPLU1,KBPLU1,KCPLU1,KDPLU1,KEPLU1,KFPLU1)
      IF(NT.EQ.-1)GO TO 15000
      GO TO 1001
2052  CALL TRIHKL(KAMOI1,KBMOI1,KCMOI1,KDMOI1,KEMOI1,KFMOI1,
     +KAPLU1,KBPLU1,KCPLU1,KDPLU1,KEPLU1,KFPLU1)
      IF(MT.EQ.-1)GO TO 15000
      DO 120 I=N6,N
      IF(IRJ(I,1).EQ.0)THEN
      MT=-1
      GO TO 15000
      ENDIF
 120  CONTINUE
      GO TO 2077
1001  CONTINUE
      DO 127 I=1,N
      JJL1=IRJ(I,1)
      JJL=IRJ(I,1)+IRJB(I)
      IRJ(I,1)=JJL
      IF(JJL.EQ.0)GO TO 15000
      IF(IRJB(I).EQ.0)GO TO 127
      DO 222 J=JJL1+1,JJL
      JJ=J-JJL1
      IH(I,J,1)=IHB(I,JJ)
      IK(I,J,1)=IKB(I,JJ)
      IL(I,J,1)=ILB(I,JJ)
222   CONTINUE
127   CONTINUE
2077  NDICH(1)=NDICH(1)+1
      DO 30001 I=1,N-1
      J=I+1
      IF(IRJ(I,1).EQ.1.AND.IRJ(J,1).EQ.1.AND.IH(I,1,1).EQ.IH(J,1,1).AND.
     +IK(I,1,1).EQ.IK(J,1,1).AND.IL(I,1,1).EQ.IL(J,1,1))GO TO 15000
30001 CONTINUE
      IF(IIAA.EQ.0)GO TO 20022
      IA2=0
2010  AMOI2=AMOI1+IA2*PASA2
      APLU2=AMOI2+PASA2
      KAMOI2=COEFF*AMOI2
      KAPLU2=COEFF*APLU2
      IF(MD34.EQ.1)THEN
      DMOI2=AMOI2-S3-ERRD
      DPLU2=APLU2-S3+ERRD
      DMOIM=AMIN 1(ABS(DMOI2),ABS(DPLU2))
      IF(DMOIM.GT.DMAX.AND.DMOI2.GE.0.)GO TO 15000
      IF(DMOIM.GT.DMAX)GO TO 2000
      IF(DMOI2.LT.(-DMAX))DMOI2=-DMAX
      IF(DPLU2.GT.DMAX)DPLU2=DMAX
      ZZO=ZCO
      IF(DMOIM*DMOIM.GE.ZZO*APLU2*BPLU2.AND.DMOI2.GE.0.)GO TO 15000
      IF(DMOIM*DMOIM.GE.ZZO*APLU2*BPLU2)GO TO 2000
      KDMOI2=COEFF*DMOI2
      KDPLU2=COEFF*DPLU2
      ELSE
      IF(MD7.EQ.0)GO TO 20022
      BMOI2=S7-APLU2-ERRB
      IF(BMOI2.GT.BMAX)GO TO 2000
      BPLU2=S7-AMOI2+ERRB
      IF(BMOI2.GT.APLU2)GO TO 2000
      IF(BPLU2.LE.0)GO TO 15000
      IF(BMOI2.LE.0)BMOI2=BMIN
      IF((DMOI2-1E-8)**2.GE.ZCO*APLU2*BPLU2)GO TO 2000
      KBMOI2=COEFF*BMOI2
      KBPLU2=COEFF*BPLU2
      ENDIF
20022 IF(IIBB.EQ.0)GO TO 20023
      IB2=0
21010 BMOI2=BMOI1+IB2*PASB2
      BPLU2=BMOI2+PASB2
      KBMOI2=COEFF*BMOI2
      KBPLU2=COEFF*BPLU2
      IF(MD12.EQ.1)THEN
      DMOI2=(BMOI2-S1)-ERRD
      DPLU2=(BPLU2-S1)+ERRD
      DMOIM=AMIN 1(ABS(DMOI2),ABS(DPLU2))
      IF(DMOIM.GT.DMAX.AND.DMOI2.GE.0.)GO TO 15000
      IF(DMOIM.GT.DMAX)GO TO 2100
      IF(DMOI2.LT.(-DMAX))DMOI2=-DMAX
      IF(DPLU2.GT.DMAX)DPLU2=DMAX
      ZZO=ZCO
      IF(DMOIM*DMOIM.GE.ZZO*APLU2*BPLU2.AND.DMOI2.GE.0.)GO TO 15000
      IF(DMOIM*DMOIM.GE.ZZO*APLU2*BPLU2)GO TO 2100
      KDMOI2=COEFF*DMOI2
      KDPLU2=COEFF*DPLU2
      ELSE
      IF(MD56.EQ.0)GO TO 20023
      DMOI2=(S5-APLU2-BPLU2)-ERRD
      DPLU2=(S5-AMOI2-BMOI2)+ERRD
      DMOIM=AMIN 1(ABS(DMOI2),ABS(DPLU2))
      IF(DMOIM.GT.DMAX)GO TO 2100
      IF(DMOIM.GT.DMAX.AND.DPLU2.LE.0.)GO TO 2000
      IF(DMOIM.GT.DMAX)GO TO 2100
      IF(DMOI2.LT.(-DMAX))DMOI2=-DMAX
      IF(DPLU2.GT.DMAX)DPLU2=DMAX
      ZZO=ZCO
      IF(DMOIM*DMOIM.GE.ZZO*APLU2*BPLU2.AND.DPLU2.LE.0.)GO TO 2000
      IF(DMOIM*DMOIM.GE.ZZO*APLU2*BPLU2)GO TO 2100
      KDMOI2=COEFF*DMOI2
      KDPLU2=COEFF*DPLU2
      ENDIF
20023 IF(IICC.EQ.0)GO TO 20024
      IC2=0
2210  CMOI2=CMOI1+IC2*PASC2
      CPLU2=CMOI2+PASC2
      KCMOI2=COEFF*CMOI2
      KCPLU2=COEFF*CPLU2
      IF(MD89.EQ.0)GO TO 20024
      EMOI2=(BMOI2+CMOI2-S8)-ERRE
      EPLU2=(BPLU2+CPLU2-S8)+ERRE
      EMMA2=AMIN 1(ABS(EMOI2),ABS(EPLU2))
      IF(EMMA2.GT.EMAX.AND.EMOI2.GT.0)GO TO 2100
      IF(EMMA2.GT.EMAX)GO TO 2200
      IF(EMOI2.LT.(-EMAX))EMOI2=-EMAX
      IF(EPLU2.GT.EMAX)EPLU2=EMAX
      ZZO=ZCO
      IF(EMOI2.LT.0.)ZZO=4.
      EMMI2=AMIN 1(ABS(EMOI2),ABS(EPLU2))
      IF(EMMI2*EMMI2.GE.ZZO*BPLU2*CPLU2.AND.EMOI2.GT.0)GO TO 2100
      IF(EMMI2*EMMI2.GE.ZZO*BPLU2*CPLU2)GO TO 2200
      KEMOI2=COEFF*EMOI2
      KEPLU2=COEFF*EPLU2
20024 IF(IIDD.EQ.0)GO TO 20025
      ID2=0
2310  DMOI2=DMOI1+ID2*PASD2
      DPLU2=DMOI2+PASD2
      IF((DMOI2-1E-8)**2.GT.ZCO*APLU2*BPLU2)GO TO 22000
      KDMOI2=COEFF*DMOI2
      KDPLU2=COEFF*DPLU2
20025 IF(IIEE.EQ.0)GO TO 20026
      IE2=0
2410  EMOI2=EMOI1+IE2*PASE2
      EPLU2=EMOI2+PASE2
      IF(EMOI2.LT.0.)GO TO 3330
      IF((EMOI2-1E-8).GT.SQRT(ZCO*BPLU2*CPLU2))GO TO 23000
      GO TO 3331
3330  IF(ABS(EPLU2-1E-8).GT.SQRT(4.*BPLU2*CPLU2))GO TO 2400
3331  KEMOI2=COEFF*EMOI2
      KEPLU2=COEFF*EPLU2
20026 IF(IIFF.EQ.0)GO TO 24000
      DO 2500 IMF2=0,1
      FMOI2=FMOI1+IMF2*PASF2
      DO 30012 I=1,KT
      ATEST=ABS(AMOI2-AAS(I))
      BTEST=ABS(BMOI2-BBS(I))
      CTEST=ABS(CMOI2-CCS(I))
      DTEST=ABS(DMOI2-DDS(I))
      ETEST=ABS(EMOI2-EES(I))
      ETESTY=ABS(ABS(EMOI2)-ABS(EES(I)))
      FTEST=ABS(FMOI2-FFS(I))
      IF(ATEST.LT.CXA.AND.BTEST.LT.CXB.AND.CTEST.LT.CXC.AND.DTEST.LT.
     +CXD.AND.ETEST.LT.CXE.AND.FTEST.LT.CXF)GO TO 25000
      IF(ATEST.LT.CYA.AND.BTEST.LT.CYA.AND.CTEST.LT.CYA.AND.DTEST.LT.
     +CYA.AND.ETESTY.LT.CYA.AND.FTEST.LT.CYA)GO TO 25000
30012 CONTINUE
      FPLU2=FMOI2+PASF2
      IF((FMOI2-1E-8)**2.GT.ZCO*APLU2*CPLU2)GO TO 24000
      KFMOI2=COEFF*FMOI2
      KFPLU2=COEFF*FPLU2
      CALL VOLUME(AMOI2,BMOI2,CMOI2,DMOI2,EMOI2,FMOI2,
     +APLU2,BPLU2,CPLU2,DPLU2,EPLU2,FPLU2)
      IF(LY.EQ.-1)GO TO 25000
      CALL TRIHKL1(KAMOI2,KBMOI2,KCMOI2,KDMOI2,KEMOI2,KFMOI2,
     +KAPLU2,KBPLU2,KCPLU2,KDPLU2,KEPLU2,KFPLU2,2)
1002  CONTINUE
      IF(NT.EQ.-1)GO TO 25000
      NDICH(2)=NDICH(2)+1
      DO 30002 I=1,N-1
      J=I+1
      IF(IRJ(I,2).EQ.1.AND.IRJ(J,2).EQ.1.AND.IH(I,1,2).EQ.IH(J,1,2).AND.
     +IK(I,1,2).EQ.IK(J,1,2).AND.IL(I,1,2).EQ.IL(J,1,2))GO TO 25000
30002 CONTINUE
      IF(IIAA.EQ.0)GO TO 20032
      IA3=0
3010  AMOI3=AMOI2+IA3*PASA4
      APLU3=AMOI3+PASA4
      KAMOI3=COEFF*AMOI3
      KAPLU3=COEFF*APLU3
      IF(MD34.EQ.1)THEN
      DMOI3=AMOI3-S3-ERRD
      DPLU3=APLU3-S3+ERRD
      DMOIM=AMIN 1(ABS(DMOI3),ABS(DPLU3))
      IF(DMOIM.GT.DMAX.AND.DMOI3.GE.0.)GO TO 25000
      IF(DMOIM.GT.DMAX)GO TO 3000
      IF(DMOI3.LT.(-DMAX))DMOI3=-DMAX
      IF(DPLU3.GT.DMAX)DPLU3=DMAX
      ZZO=ZCO
      IF(DMOIM*DMOIM.GE.ZZO*APLU3*BPLU3.AND.DMOI3.GE.0.)GO TO 25000
      IF(DMOIM*DMOIM.GE.ZZO*APLU3*BPLU3)GO TO 3000
      KDMOI3=COEFF*DMOI3
      KDPLU3=COEFF*DPLU3
      ELSE
      IF(MD7.EQ.0)GO TO 20032
      BMOI3=S7-APLU3-ERRB
      IF(BMOI3.GT.BMAX)GO TO 3000
      BPLU3=S7-AMOI3+ERRB
      IF(BMOI3.GT.APLU3)GO TO 3000
      IF(BPLU3.LE.0)GO TO 25000
      IF(BMOI3.LE.0)BMOI3=BMIN
      IF((DMOI3-1E-8)**2.GE.ZCO*APLU3*BPLU3)GO TO 3000
      KBMOI3=COEFF*BMOI3
      KBPLU3=COEFF*BPLU3
      ENDIF
20032 IF(IIBB.EQ.0)GO TO 20033
      IB3=0
3110  BMOI3=BMOI2+IB3*PASB4
      BPLU3=BMOI3+PASB4
      KBMOI3=COEFF*BMOI3
      KBPLU3=COEFF*BPLU3
      IF(MD12.EQ.1)THEN
      DMOI3=(BMOI3-S1)-ERRD
      DPLU3=(BPLU3-S1)+ERRD
      DMOIM=AMIN 1(ABS(DMOI3),ABS(DPLU3))
      IF(DMOIM.GT.DMAX.AND.DMOI3.GE.0.)GO TO 25000
      IF(DMOIM.GT.DMAX)GO TO 3100
      IF(DMOI3.LT.(-DMAX))DMOI3=-DMAX
      IF(DPLU3.GT.DMAX)DPLU3=DMAX
      ZZO=ZCO
      IF(DMOIM*DMOIM.GE.ZZO*APLU3*BPLU3.AND.DMOI3.GE.0.)GO TO 25000
      IF(DMOIM*DMOIM.GE.ZZO*APLU3*BPLU3)GO TO 3100
      KDMOI3=COEFF*DMOI3
      KDPLU3=COEFF*DPLU3
      ELSE
      IF(MD56.EQ.0)GO TO 20033
      DMOI3=(S5-APLU3-BPLU3)-ERRD
      DPLU3=(S5-AMOI3-BMOI3)+ERRD
      DMOIM=AMIN 1(ABS(DMOI3),ABS(DPLU3))
      IF(DMOIM.GT.DMAX.AND.DPLU3.LE.0.)GO TO 3000
      IF(DMOIM.GT.DMAX)GO TO 3100
      IF(DMOI3.LT.(-DMAX))DMOI3=-DMAX
      IF(DPLU3.GT.DMAX)DPLU3=DMAX
      ZZO=ZCO
      IF(DMOIM*DMOIM.GE.ZZO*APLU3*BPLU3.AND.DPLU3.LE.0)GO TO 3000
      IF(DMOIM*DMOIM.GE.ZZO*APLU3*BPLU3)GO TO 3100
      KDMOI3=COEFF*DMOI3
      KDPLU3=COEFF*DPLU3
      ENDIF
20033 IF(IICC.EQ.0)GO TO 20034
      IC3=0
3210  CMOI3=CMOI2+IC3*PASC4
      CPLU3=CMOI3+PASC4
      KCMOI3=COEFF*CMOI3
      KCPLU3=COEFF*CPLU3
      IF(MD89.EQ.0)GO TO 20034
      EMOI3=(BMOI3+CMOI3-S8)-ERRE
      EPLU3=(BPLU3+CPLU3-S8)+ERRE
      EMMA3=AMIN 1(ABS(EMOI3),ABS(EPLU3))
      IF(EMMA3.GT.EMAX.AND.EMOI3.GT.0)GO TO 3100
      IF(EMMA3.GT.EMAX)GO TO 3200
      IF(EMOI3.LT.(-EMAX))EMOI3=-EMAX
      IF(EPLU3.GT.EMAX)EPLU3=EMAX
      ZZO=ZCO
      IF(EMOI3.LT.0.)ZZO=4.
      EMMI3=AMIN 1(ABS(EMOI3),ABS(EPLU3))
      IF(EMMI3*EMMI3.GE.ZZO*BPLU3*CPLU3.AND.EMOI3.GT.0)GO TO 3100
      IF(EMMI3*EMMI3.GE.ZZO*BPLU3*CPLU3)GO TO 3200
      KEMOI3=COEFF*EMOI3
      KEPLU3=COEFF*EPLU3
20034 IF(IIDD.EQ.0)GO TO 20035
      ID3=0
3310  DMOI3=DMOI2+ID3*PASD4
      DPLU3=DMOI3+PASD4
      IF((DMOI3-1E-8)**2.GT.ZCO*APLU3*BPLU3)GO TO 32000
      KDMOI3=COEFF*DMOI3
      KDPLU3=COEFF*DPLU3
20035 IF(IIEE.EQ.0)GO TO 20036
      IE3=0
3410  EMOI3=EMOI2+IE3*PASE4
      EPLU3=EMOI3+PASE4
      IF(EMOI3.LT.0.)GO TO 3430
      IF((EMOI3-1E-8).GT.SQRT(ZCO*BPLU3*CPLU3))GO TO 33000
      GO TO 3431
3430  IF(ABS(EPLU3-1E-8).GT.SQRT(4.*BPLU3*CPLU3))GO TO 3400
3431  KEMOI3=COEFF*EMOI3
      KEPLU3=COEFF*EPLU3
20036 IF(IIFF.EQ.0)GO TO 34000
      DO 3500 IMF3=0,1
      FMOI3=FMOI2+IMF3*PASF4
      DO 30013 I=1,KT
      ATEST=ABS(AMOI3-AAS(I))
      BTEST=ABS(BMOI3-BBS(I))
      CTEST=ABS(CMOI3-CCS(I))
      DTEST=ABS(DMOI3-DDS(I))
      ETEST=ABS(EMOI3-EES(I))
      ETESTY=ABS(ABS(EMOI3)-ABS(EES(I)))
      FTEST=ABS(FMOI3-FFS(I))
      IF(ATEST.LT.CXA.AND.BTEST.LT.CXB.AND.CTEST.LT.CXC.AND.DTEST.LT.
     +CXD.AND.ETEST.LT.CXE.AND.FTEST.LT.CXF)GO TO 35000
      IF(ATEST.LT.CYA.AND.BTEST.LT.CYA.AND.CTEST.LT.CYA.AND.DTEST.LT.
     +CYA.AND.ETESTY.LT.CYA.AND.FTEST.LT.CYA)GO TO 35000
30013 CONTINUE
      FPLU3=FMOI3+PASF4
      IF((FMOI3-1E-8)**2.GT.ZCO*APLU3*CPLU3)GO TO 34000
      KFMOI3=COEFF*FMOI3
      KFPLU3=COEFF*FPLU3
      CALL VOLUME(AMOI3,BMOI3,CMOI3,DMOI3,EMOI3,FMOI3,
     +APLU3,BPLU3,CPLU3,DPLU3,EPLU3,FPLU3)
      IF(LY.EQ.-1)GO TO 35000
      CALL TRIHKL1(KAMOI3,KBMOI3,KCMOI3,KDMOI3,KEMOI3,KFMOI3,
     +KAPLU3,KBPLU3,KCPLU3,KDPLU3,KEPLU3,KFPLU3,3)
1003  CONTINUE
      IF(NT.EQ.-1)GO TO 35000
      NDICH(3)=NDICH(3)+1
      DO 30003 I=1,N-1
      J=I+1
      IF(IRJ(I,3).EQ.1.AND.IRJ(J,3).EQ.1.AND.IH(I,1,3).EQ.IH(J,1,3).AND.
     +IK(I,1,3).EQ.IK(J,1,3).AND.IL(I,1,3).EQ.IL(J,1,3))GO TO 35000
30003 CONTINUE
      IF(IIAA.EQ.0)GO TO 20042
      IA4=0
4010  AMOI4=AMOI3+IA4*PASA8
      APLU4=AMOI4+PASA8
      KAMOI4=COEFF*AMOI4
      KAPLU4=COEFF*APLU4
      IF(MD34.EQ.1)THEN
      DMOI4=AMOI4-S3-ERRD
      DPLU4=APLU4-S3+ERRD
      DMOIM=AMIN 1(ABS(DMOI4),ABS(DPLU4))
      IF(DMOIM.GT.DMAX.AND.DMOI4.GE.0.)GO TO 35000
      IF(DMOIM.GT.DMAX)GO TO 4000
      IF(DMOI4.LT.(-DMAX))DMOI4=-DMAX
      IF(DPLU4.GT.DMAX)DPLU4=DMAX
      ZZO=ZCO
      IF(DMOIM*DMOIM.GE.ZZO*APLU4*BPLU4.AND.DMOI4.GE.0.)GO TO 35000
      IF(DMOIM*DMOIM.GE.ZZO*APLU4*BPLU4)GO TO 4000
      KDMOI4=COEFF*DMOI4
      KDPLU4=COEFF*DPLU4
      ELSE
      IF(MD7.EQ.0)GO TO 20042
      BMOI4=S7-APLU4-ERRB
      IF(BMOI4.GT.BMAX)GO TO 4000
      BPLU4=S7-AMOI4+ERRB
      IF(BMOI4.GT.APLU4)GO TO 4000
      IF(BPLU4.LE.0)GO TO 35000
      IF(BMOI4.LE.0)BMOI4=BMIN
      IF((DMOI4-1E-8)**2.GE.ZCO*APLU4*BPLU4)GO TO 4000
      KBMOI4=COEFF*BMOI4
      KBPLU4=COEFF*BPLU4
      ENDIF
20042 IF(IIBB.EQ.0)GO TO 20043
      IB4=0
4110  BMOI4=BMOI3+IB4*PASB8
      BPLU4=BMOI4+PASB8
      KBMOI4=COEFF*BMOI4
      KBPLU4=COEFF*BPLU4
      IF(MD12.EQ.1)THEN
      DMOI4=(BMOI4-S1)-ERRD
      DPLU4=(BPLU4-S1)+ERRD
      DMOIM=AMIN 1(ABS(DMOI4),ABS(DPLU4))
      IF(DMOIM.GT.DMAX.AND.DMOI4.GE.0.)GO TO 35000
      IF(DMOIM.GT.DMAX)GO TO 4100
      IF(DMOI4.LT.(-DMAX))DMOI4=-DMAX
      IF(DPLU4.GT.DMAX)DPLU4=DMAX
      ZZO=ZCO
      IF(DMOIM*DMOIM.GE.ZZO*APLU4*BPLU4.AND.DMOI4.GE.0.)GO TO 35000
      IF(DMOIM*DMOIM.GE.ZZO*APLU4*BPLU4)GO TO 4100
      KDMOI4=COEFF*DMOI4
      KDPLU4=COEFF*DPLU4
      ELSE
      IF(MD56.EQ.0)GO TO 20043
      DMOI4=(S5-APLU4-BPLU4)-ERRD
      DPLU4=(S5-AMOI4-BMOI4)+ERRD
      DMOIM=AMIN 1(ABS(DMOI4),ABS(DPLU4))
      IF(DMOIM.GT.DMAX.AND.DPLU4.LE.0.)GO TO 4000
      IF(DMOIM.GT.DMAX)GO TO 4100
      IF(DMOI4.LT.(-DMAX))DMOI4=-DMAX
      IF(DPLU4.GT.DMAX)DPLU4=DMAX
      ZZO=ZCO
      IF(DMOIM*DMOIM.GE.ZZO*APLU4*BPLU4.AND.DPLU4.LE.0.)GO TO 4000
      IF(DMOIM*DMOIM.GE.ZZO*APLU4*BPLU4)GO TO 4100
      KDMOI4=COEFF*DMOI4
      KDPLU4=COEFF*DPLU4
      ENDIF
20043 IF(IICC.EQ.0)GO TO 20044
      IC4=0
4210  CMOI4=CMOI3+IC4*PASC8
      CPLU4=CMOI4+PASC8
      KCMOI4=COEFF*CMOI4
      KCPLU4=COEFF*CPLU4
      IF(MD89.EQ.0)GO TO 20044
      EMOI4=(BMOI4+CMOI4-S8)-ERRE
      EPLU4=(BPLU4+CPLU4-S8)+ERRE
      EMMA4=AMIN 1(ABS(EMOI4),ABS(EPLU4))
      IF(EMMA4.GT.EMAX.AND.EMOI4.GT.0)GO TO 4100
      IF(EMMA4.GT.EMAX)GO TO 4200
      IF(EMOI4.LT.(-EMAX))EMOI4=-EMAX
      IF(EPLU4.GT.EMAX)EPLU4=EMAX
      ZZO=ZCO
      IF(EMOI4.LT.0.)ZZO=4.
      EMMI4=AMIN 1(ABS(EMOI4),ABS(EPLU4))
      IF(EMMI4*EMMI4.GE.ZZO*BPLU4*CPLU4.AND.EMOI4.GT.0)GO TO 4100
      IF(EMMI4*EMMI4.GE.ZZO*BPLU4*CPLU4)GO TO 4200
      KEMOI4=COEFF*EMOI4
      KEPLU4=COEFF*EPLU4
20044 IF(IIDD.EQ.0)GO TO 20045
      ID4=0
4310  DMOI4=DMOI3+ID4*PASD8
      DPLU4=DMOI4+PASD8
      IF((DMOI4-1E-8)**2.GT.ZCO*APLU4*BPLU4)GO TO 42000
      KDMOI4=COEFF*DMOI4
      KDPLU4=COEFF*DPLU4
20045 IF(IIEE.EQ.0)GO TO 20046
      IE4=0
4410  EMOI4=EMOI3+IE4*PASE8
      EPLU4=EMOI4+PASE8
      IF(EMOI4.LT.0.)GO TO 3530
      IF((EMOI4-1E-8).GT.SQRT(ZCO*BPLU4*CPLU4))GO TO 43000
      GO TO 3531
3530  IF(ABS(EPLU4-1E-8).GT.SQRT(4.*BPLU4*CPLU4))GO TO 4400
3531  KEMOI4=COEFF*EMOI4
      KEPLU4=COEFF*EPLU4
20046 IF(IIFF.EQ.0)GO TO 44000
      DO 4500 IMF4=0,1
      FMOI4=FMOI3+IMF4*PASF8
      DO 30014 I=1,KT
      ATEST=ABS(AMOI4-AAS(I))
      BTEST=ABS(BMOI4-BBS(I))
      CTEST=ABS(CMOI4-CCS(I))
      DTEST=ABS(DMOI4-DDS(I))
      ETEST=ABS(EMOI4-EES(I))
      ETESTY=ABS(ABS(EMOI4)-ABS(EES(I)))
      FTEST=ABS(FMOI4-FFS(I))
      IF(ATEST.LT.CXA.AND.BTEST.LT.CXB.AND.CTEST.LT.CXC.AND.DTEST.LT.
     +CXD.AND.ETEST.LT.CXE.AND.FTEST.LT.CXF)GO TO 45000
      IF(ATEST.LT.CYA.AND.BTEST.LT.CYA.AND.CTEST.LT.CYA.AND.DTEST.LT.
     +CYA.AND.ETESTY.LT.CYA.AND.FTEST.LT.CYA)GO TO 45000
30014 CONTINUE
      FPLU4=FMOI4+PASF8
      IF((FMOI4-1E-8)**2.GT.ZCO*APLU4*CPLU4)GO TO 44000
      KFMOI4=COEFF*FMOI4
      KFPLU4=COEFF*FPLU4
      CALL VOLUME(AMOI4,BMOI4,CMOI4,DMOI4,EMOI4,FMOI4,
     +APLU4,BPLU4,CPLU4,DPLU4,EPLU4,FPLU4)
      IF(LY.EQ.-1)GO TO 45000
      CALL TRIHKL1(KAMOI4,KBMOI4,KCMOI4,KDMOI4,KEMOI4,KFMOI4,
     +KAPLU4,KBPLU4,KCPLU4,KDPLU4,KEPLU4,KFPLU4,4)
1004  CONTINUE
      IF(NT.EQ.-1)GO TO 45000
      NDICH(4)=NDICH(4)+1
      DO 30004 I=1,N-1
      J=I+1
      IF(IRJ(I,4).EQ.1.AND.IRJ(J,4).EQ.1.AND.IH(I,1,4).EQ.IH(J,1,4).AND.
     +IK(I,1,4).EQ.IK(J,1,4).AND.IL(I,1,4).EQ.IL(J,1,4))GO TO 45000
30004 CONTINUE
      IF(IIAA.EQ.0)GO TO 20052
      IA5=0
5010  AMOI5=AMOI4+IA5*PASA16
      APLU5=AMOI5+PASA16
      KAMOI5=COEFF*AMOI5
      KAPLU5=COEFF*APLU5
      IF(MD34.EQ.1)THEN
      DMOI5=AMOI5-S3-ERRD
      DPLU5=APLU5-S3+ERRD
      DMOIM=AMIN 1(ABS(DMOI5),ABS(DPLU5))
      IF(DMOIM.GT.DMAX.AND.DMOI5.GE.0.)GO TO 45000
      IF(DMOIM.GT.DMAX)GO TO 5000
      IF(DMOI5.LT.(-DMAX))DMOI5=-DMAX
      IF(DPLU5.GT.DMAX)DPLU5=DMAX
      ZZO=ZCO
      IF(DMOIM*DMOIM.GE.ZZO*APLU5*BPLU5.AND.DMOI5.GE.0.)GO TO 45000
      IF(DMOIM*DMOIM.GE.ZZO*APLU5*BPLU5)GO TO 5000
      KDMOI5=COEFF*DMOI5
      KDPLU5=COEFF*DPLU5
      ELSE
      IF(MD7.EQ.0)GO TO 20052
      BMOI5=S7-APLU5-ERRB
      IF(BMOI5.GT.BMAX)GO TO 5000
      BPLU5=S7-AMOI5+ERRB
      IF(BMOI5.GT.APLU5)GO TO 5000
      IF(BPLU5.LE.0)GO TO 45000
      IF(BMOI5.LE.0)BMOI5=BMIN
      IF((DMOI5-1E-8)**2.GE.ZCO*APLU5*BPLU5)GO TO 5000
      KBMOI5=COEFF*BMOI5
      KBPLU5=COEFF*BPLU5
      ENDIF
20052 IF(IIBB.EQ.0)GO TO 20053
      IB5=0
5110  BMOI5=BMOI4+IB5*PASB16
      BPLU5=BMOI5+PASB16
      KBMOI5=COEFF*BMOI5
      KBPLU5=COEFF*BPLU5
      IF(MD12.EQ.1)THEN
      DMOI5=(BMOI5-S1)-ERRD
      DPLU5=(BPLU5-S1)+ERRD
      DMOIM=AMIN 1(ABS(DMOI5),ABS(DPLU5))
      IF(DMOIM.GT.DMAX.AND.DMOI5.GE.0.)GO TO 45000
      IF(DMOIM.GT.DMAX)GO TO 5100
      IF(DMOI5.LT.(-DMAX))DMOI5=-DMAX
      IF(DPLU5.GT.DMAX)DPLU5=DMAX
      ZZO=ZCO
      IF(DMOIM*DMOIM.GE.ZZO*APLU5*BPLU5.AND.DMOI5.GE.0.)GO TO 45000
      IF(DMOIM*DMOIM.GE.ZZO*APLU5*BPLU5)GO TO 5100
      KDMOI5=COEFF*DMOI5
      KDPLU5=COEFF*DPLU5
      ELSE
      IF(MD56.EQ.0)GO TO 20053
      DMOI5=(S5-APLU5-BPLU5)-ERRD
      DPLU5=(S5-AMOI5-BMOI5)+ERRD
      DMOIM=AMIN 1(ABS(DMOI5),ABS(DPLU5))
      IF(DMOIM.GT.DMAX.AND.DPLU5.LE.0.)GO TO 5000
      IF(DMOIM.GT.DMAX)GO TO 5100
      IF(DMOI5.LT.(-DMAX))DMOI5=-DMAX
      IF(DPLU5.GT.DMAX)DPLU5=DMAX
      ZZO=ZCO
      IF(DMOIM*DMOIM.GE.ZZO*APLU5*BPLU5.AND.DPLU5.LE.0.)GO TO 5000
      IF(DMOIM*DMOIM.GE.ZZO*APLU5*BPLU5)GO TO 5100
      KDMOI5=COEFF*DMOI5
      KDPLU5=COEFF*DPLU5
      ENDIF
20053 IF(IICC.EQ.0)GO TO 20054
      IC5=0
5210  CMOI5=CMOI4+IC5*PASC16
      CPLU5=CMOI5+PASC16
      KCMOI5=COEFF*CMOI5
      KCPLU5=COEFF*CPLU5
      IF(MD89.EQ.0)GO TO 20054
      EMOI5=(BMOI5+CMOI5-S8)-ERRE
      EPLU5=(BPLU5+CPLU5-S8)+ERRE
      EMMA5=AMIN 1(ABS(EMOI5),ABS(EPLU5))
      IF(EMMA5.GT.EMAX.AND.EMOI5.GT.0)GO TO 5100
      IF(EMMA5.GT.EMAX)GO TO 5200
      IF(EMOI5.LT.(-EMAX))EMOI5=-EMAX
      IF(EPLU5.GT.EMAX)EPLU5=EMAX
      ZZO=ZCO
      IF(EMOI5.LT.0.)ZZO=4.
      EMMI5=AMIN 1(ABS(EMOI5),ABS(EPLU5))
      IF(EMMI5*EMMI5.GE.ZZO*BPLU5*CPLU5.AND.EMOI5.GT.0)GO TO 5100
      IF(EMMI5*EMMI5.GE.ZZO*BPLU5*CPLU5)GO TO 5200
      KEMOI5=COEFF*EMOI5
      KEPLU5=COEFF*EPLU5
20054 IF(IIDD.EQ.0)GO TO 20055
      ID5=0
5310  DMOI5=DMOI4+ID5*PASD16
      DPLU5=DMOI5+PASD16
      IF((DMOI5-1E-8)**2.GT.ZCO*APLU5*BPLU5)GO TO 52000
      KDMOI5=COEFF*DMOI5
      KDPLU5=COEFF*DPLU5
20055 IF(IIEE.EQ.0)GO TO 20056
      IE5=0
5410  EMOI5=EMOI4+IE5*PASE16
      EPLU5=EMOI5+PASE16
      IF(EMOI5.LT.0.)GO TO 3630
      IF((EMOI5-1E-8).GT.SQRT(ZCO*BPLU5*CPLU5))GO TO 53000
      GO TO 3631
3630  IF(ABS(EPLU5-1E-8).GT.SQRT(4.*BPLU5*CPLU5))GO TO 5400
3631  KEMOI5=COEFF*EMOI5
      KEPLU5=COEFF*EPLU5
20056 IF(IIFF.EQ.0)GO TO 54000
      DO 5500 IMF5=0,1
      FMOI5=FMOI4+IMF5*PASF16
      DO 30015 I=1,KT
      ATEST=ABS(AMOI5-AAS(I))
      BTEST=ABS(BMOI5-BBS(I))
      CTEST=ABS(CMOI5-CCS(I))
      DTEST=ABS(DMOI5-DDS(I))
      ETEST=ABS(EMOI5-EES(I))
      ETESTY=ABS(ABS(EMOI5)-ABS(EES(I)))
      FTEST=ABS(FMOI5-FFS(I))
      IF(ATEST.LT.CXA.AND.BTEST.LT.CXB.AND.CTEST.LT.CXC.AND.DTEST.LT.
     +CXD.AND.ETEST.LT.CXE.AND.FTEST.LT.CXF)GO TO 55000
      IF(ATEST.LT.CYA.AND.BTEST.LT.CYA.AND.CTEST.LT.CYA.AND.DTEST.LT.
     +CYA.AND.ETESTY.LT.CYA.AND.FTEST.LT.CYA)GO TO 55000
30015 CONTINUE
      FPLU5=FMOI5+PASF16
      IF((FMOI5-1E-8)**2.GT.ZCO*APLU5*CPLU5)GO TO 54000
      KFMOI5=COEFF*FMOI5
      KFPLU5=COEFF*FPLU5
      CALL VOLUME(AMOI5,BMOI5,CMOI5,DMOI5,EMOI5,FMOI5,
     +APLU5,BPLU5,CPLU5,DPLU5,EPLU5,FPLU5)
      IF(LY.EQ.-1)GO TO 55000
      CALL TRIHKL1(KAMOI5,KBMOI5,KCMOI5,KDMOI5,KEMOI5,KFMOI5,
     +KAPLU5,KBPLU5,KCPLU5,KDPLU5,KEPLU5,KFPLU5,5)
1005  CONTINUE
      IF(NT.EQ.-1)GO TO 55000
      NDICH(5)=NDICH(5)+1
      DO 30005 I=1,N-1
      J=I+1
      IF(IRJ(I,5).EQ.1.AND.IRJ(J,5).EQ.1.AND.IH(I,1,5).EQ.IH(J,1,5).AND.
     +IK(I,1,5).EQ.IK(J,1,5).AND.IL(I,1,5).EQ.IL(J,1,5))GO TO 55000
30005 CONTINUE
      IF(IIAA.EQ.0)GO TO 20062
      IA6=0
6010  AMOI6=AMOI5+IA6*PASA32
      APLU6=AMOI6+PASA32
      KAMOI6=COEFF*AMOI6
      KAPLU6=COEFF*APLU6
      IF(MD34.EQ.1)THEN
      DMOI6=AMOI6-S3-ERRD
      DPLU6=APLU6-S3+ERRD
      DMOIM=AMIN 1(ABS(DMOI6),ABS(DPLU6))
      IF(DMOIM.GT.DMAX.AND.DMOI6.GE.0.)GO TO 55000
      IF(DMOIM.GT.DMAX)GO TO 6000
      IF(DMOI6.LT.(-DMAX))DMOI6=-DMAX
      IF(DPLU6.GT.DMAX)DPLU6=DMAX
      ZZO=ZCO
      IF(DMOIM*DMOIM.GE.ZZO*APLU6*BPLU6.AND.DMOI6.GE.0.)GO TO 55000
      IF(DMOIM*DMOIM.GE.ZZO*APLU6*BPLU6)GO TO 6000
      KDMOI6=COEFF*DMOI6
      KDPLU6=COEFF*DPLU6
      ELSE
      IF(MD7.EQ.0)GO TO 20062
      BMOI6=S7-APLU6-ERRB
      IF(BMOI6.GT.BMAX)GO TO 6000
      BPLU6=S7-AMOI6+ERRB
      IF(BMOI6.GT.APLU6)GO TO 6000
      IF(BPLU6.LE.0)GO TO 55000
      IF(BMOI6.LE.0)BMOI6=BMIN
      IF((DMOI6-1E-8)**2.GE.ZCO*APLU6*BPLU6)GO TO 6000
      KBMOI6=COEFF*BMOI6
      KBPLU6=COEFF*BPLU6
      ENDIF
20062 IF(IIBB.EQ.0)GO TO 20063
      IB6=0
6110  BMOI6=BMOI5+IB6*PASB32
      BPLU6=BMOI6+PASB32
      KBMOI6=COEFF*BMOI6
      KBPLU6=COEFF*BPLU6
      IF(MD12.EQ.1)THEN
      DMOI6=(BMOI6-S1)-ERRD
      DPLU6=(BPLU6-S1)+ERRD
      DMOIM=AMIN 1(ABS(DMOI6),ABS(DPLU6))
      IF(DMOIM.GT.DMAX.AND.DMOI6.GE.0.)GO TO 55000
      IF(DMOIM.GT.DMAX)GO TO 6100
      IF(DMOI6.LT.(-DMAX))DMOI6=-DMAX
      IF(DPLU6.GT.DMAX)DPLU6=DMAX
      ZZO=ZCO
      IF(DMOIM*DMOIM.GE.ZZO*APLU6*BPLU6.AND.DMOI6.GE.0.)GO TO 55000
      IF(DMOIM*DMOIM.GE.ZZO*APLU6*BPLU6)GO TO 6100
      KDMOI6=COEFF*DMOI6
      KDPLU6=COEFF*DPLU6
      ELSE
      IF(MD56.EQ.0)GO TO 20063
      DMOI6=(S5-APLU6-BPLU6)-ERRD
      DPLU6=(S5-AMOI6-BMOI6)+ERRD
      DMOIM=AMIN 1(ABS(DMOI6),ABS(DPLU6))
      IF(DMOIM.GT.DMAX.AND.DPLU6.LE.0.)GO TO 6000
      IF(DMOIM.GT.DMAX)GO TO 6100
      IF(DMOI6.LT.(-DMAX))DMOI6=-DMAX
      IF(DPLU6.GT.DMAX)DPLU6=DMAX
      ZZO=ZCO
      IF(DMOIM*DMOIM.GE.ZZO*APLU6*BPLU6.AND.DPLU6.LE.0.)GO TO 6000
      IF(DMOIM*DMOIM.GE.ZZO*APLU6*BPLU6)GO TO 6100
      KDMOI6=COEFF*DMOI6
      KDPLU6=COEFF*DPLU6
      ENDIF
20063 IF(IICC.EQ.0)GO TO 20064
      IC6=0
6210  CMOI6=CMOI5+IC6*PASC32
      CPLU6=CMOI6+PASC32
      KCMOI6=COEFF*CMOI6
      KCPLU6=COEFF*CPLU6
      IF(MD89.EQ.0)GO TO 20064
      EMOI6=(BMOI6+CMOI6-S8)-ERRE
      EPLU6=(BPLU6+CPLU6-S8)+ERRE
      EMMA6=AMIN 1(ABS(EMOI6),ABS(EPLU6))
      IF(EMMA6.GT.EMAX.AND.EMOI6.GT.0)GO TO 6100
      IF(EMMA6.GT.EMAX)GO TO 6200
      IF(EMOI6.LT.(-EMAX))EMOI6=-EMAX
      IF(EPLU6.GT.EMAX)EPLU6=EMAX
      ZZO=ZCO
      IF(EMOI6.LT.0.)ZZO=4.
      EMMI6=AMIN 1(ABS(EMOI6),ABS(EPLU6))
      IF(EMMI6*EMMI6.GE.ZZO*BPLU6*CPLU6.AND.EMOI6.GT.0)GO TO 6100
      IF(EMMI6*EMMI6.GE.ZZO*BPLU6*CPLU6)GO TO 6200
      KEMOI6=COEFF*EMOI6
      KEPLU6=COEFF*EPLU6
20064 IF(IIDD.EQ.0)GO TO 20065
      ID6=0
6310  DMOI6=DMOI5+ID6*PASD32
      DPLU6=DMOI6+PASD32
      IF((DMOI6-1E-8)**2.GT.ZCO*APLU6*BPLU6)GO TO 62000
      KDMOI6=COEFF*DMOI6
      KDPLU6=COEFF*DPLU6
20065 IF(IIEE.EQ.0)GO TO 20066
      IE6=0
6410  EMOI6=EMOI5+IE6*PASE32
      EPLU6=EMOI6+PASE32
      IF(EMOI6.LT.0.)GO TO 3730
      IF((EMOI6-1E-8).GT.SQRT(ZCO*BPLU6*CPLU6))GO TO 63000
      GO TO 3731
3730  IF(ABS(EPLU6-1E-8).GT.SQRT(4.*BPLU6*CPLU6))GO TO 6400
3731  KEMOI6=COEFF*EMOI6
      KEPLU6=COEFF*EPLU6
20066 IF(IIFF.EQ.0)GO TO 64000
      DO 6500 IMF6=0,1
      FMOI6=FMOI5+IMF6*PASF32
      DO 30016 I=1,KT
      ATEST=ABS(AMOI6-AAS(I))
      BTEST=ABS(BMOI6-BBS(I))
      CTEST=ABS(CMOI6-CCS(I))
      DTEST=ABS(DMOI6-DDS(I))
      ETEST=ABS(EMOI6-EES(I))
      ETESTY=ABS(ABS(EMOI6)-ABS(EES(I)))
      FTEST=ABS(FMOI6-FFS(I))
      IF(ATEST.LT.CXA.AND.BTEST.LT.CXB.AND.CTEST.LT.CXC.AND.DTEST.LT.
     +CXD.AND.ETEST.LT.CXE.AND.FTEST.LT.CXF)GO TO 65000
      IF(ATEST.LT.CYA.AND.BTEST.LT.CYA.AND.CTEST.LT.CYA.AND.DTEST.LT.
     +CYA.AND.ETESTY.LT.CYA.AND.FTEST.LT.CYA)GO TO 65000
30016 CONTINUE
      FPLU6=FMOI6+PASF32
      IF((FMOI6-1E-8)**2.GT.ZCO*APLU6*CPLU6)GO TO 64000
      KFMOI6=COEFF*FMOI6
      KFPLU6=COEFF*FPLU6
      CALL VOLUME(AMOI6,BMOI6,CMOI6,DMOI6,EMOI6,FMOI6,
     +APLU6,BPLU6,CPLU6,DPLU6,EPLU6,FPLU6)
      IF(LY.EQ.-1)GO TO 65000
      CALL TRIHKL1(KAMOI6,KBMOI6,KCMOI6,KDMOI6,KEMOI6,KFMOI6,
     +KAPLU6,KBPLU6,KCPLU6,KDPLU6,KEPLU6,KFPLU6,6)
1006  CONTINUE
      IF(NT.EQ.-1)GO TO 65000
      NDICH(6)=NDICH(6)+1
      DO 30006 I=1,N-1
      J=I+1
      IF(IRJ(I,6).EQ.1.AND.IRJ(J,6).EQ.1.AND.IH(I,1,6).EQ.IH(J,1,6).AND.
     +IK(I,1,6).EQ.IK(J,1,6).AND.IL(I,1,6).EQ.IL(J,1,6))GO TO 65000
30006 CONTINUE
      IF(IIAA.EQ.0)GO TO 20072
      IA7=0
7010  AMOI7=AMOI6+IA7*PASA64
      APLU7=AMOI7+PASA64
      KAMOI7=COEFF*AMOI7
      KAPLU7=COEFF*APLU7
      IF(MD34.EQ.1)THEN
      DMOI7=AMOI7-S3-ERRD
      DPLU7=APLU7-S3+ERRD
      DMOIM=AMIN 1(ABS(DMOI7),ABS(DPLU7))
      IF(DMOIM.GT.DMAX.AND.DMOI7.GE.0.)GO TO 65000
      IF(DMOIM.GT.DMAX)GO TO 7000
      IF(DMOI7.LT.(-DMAX))DMOI7=-DMAX
      IF(DPLU7.GT.DMAX)DPLU7=DMAX
      ZZO=ZCO
      IF(DMOIM*DMOIM.GE.ZZO*APLU7*BPLU7.AND.DMOI7.GE.0.)GO TO 65000
      IF(DMOIM*DMOIM.GE.ZZO*APLU7*BPLU7)GO TO 7000
      KDMOI7=COEFF*DMOI7
      KDPLU7=COEFF*DPLU7
      ELSE
      IF(MD7.EQ.0)GO TO 20072
      BMOI7=S7-APLU7-ERRB
      IF(BMOI7.GT.BMAX)GO TO 7000
      BPLU7=S7-AMOI7+ERRB
      IF(BMOI7.GT.APLU7)GO TO 7000
      IF(BPLU7.LE.0)GO TO 65000
      IF(BMOI7.LE.0)BMOI7=BMIN
      IF((DMOI7-1E-8)**2.GE.ZCO*APLU7*BPLU7)GO TO 7000
      KBMOI7=COEFF*BMOI7
      KBPLU7=COEFF*BPLU7
      ENDIF
20072 IF(IIBB.EQ.0)GO TO 20073
      IB7=0
7110  BMOI7=BMOI6+IB7*PASB64
      BPLU7=BMOI7+PASB64
      KBMOI7=COEFF*BMOI7
      KBPLU7=COEFF*BPLU7
      IF(MD12.EQ.1)THEN
      DMOI7=(BMOI7-S1)-ERRD
      DPLU7=(BPLU7-S1)+ERRD
      DMOIM=AMIN 1(ABS(DMOI7),ABS(DPLU7))
      IF(DMOIM.GT.DMAX.AND.DMOI7.GE.0.)GO TO 65000
      IF(DMOIM.GT.DMAX)GO TO 7100
      IF(DMOI7.LT.(-DMAX))DMOI7=-DMAX
      IF(DPLU7.GT.DMAX)DPLU7=DMAX
      ZZO=ZCO
      IF(DMOIM*DMOIM.GE.ZZO*APLU7*BPLU7.AND.DMOI7.GE.0.)GO TO 65000
      IF(DMOIM*DMOIM.GE.ZZO*APLU7*BPLU7)GO TO 7100
      KDMOI7=COEFF*DMOI7
      KDPLU7=COEFF*DPLU7
      ELSE
      IF(MD56.EQ.0)GO TO 20073
      DMOI7=(S5-APLU7-BPLU7)-ERRD
      DPLU7=(S5-AMOI7-BMOI7)+ERRD
      DMOIM=AMIN 1(ABS(DMOI7),ABS(DPLU7))
      IF(DMOIM.GT.DMAX.AND.DPLU7.LE.0.)GO TO 7000
      IF(DMOIM.GT.DMAX)GO TO 7100
      IF(DMOI7.LT.(-DMAX))DMOI7=-DMAX
      IF(DPLU7.GT.DMAX)DPLU7=DMAX
      ZZO=ZCO
      IF(DMOIM*DMOIM.GE.ZZO*APLU7*BPLU7.AND.DPLU7.LE.0.)GO TO 7000
      IF(DMOIM*DMOIM.GE.ZZO*APLU7*BPLU7)GO TO 7100
      KDMOI7=COEFF*DMOI7
      KDPLU7=COEFF*DPLU7
      ENDIF
20073 IF(IICC.EQ.0)GO TO 20074
      IC7=0
7210  CMOI7=CMOI6+IC7*PASC64
      CPLU7=CMOI7+PASC64
      KCMOI7=COEFF*CMOI7
      KCPLU7=COEFF*CPLU7
      IF(MD89.EQ.0)GO TO 20074
      EMOI7=(BMOI7+CMOI7-S8)-ERRE
      EPLU7=(BPLU7+CPLU7-S8)+ERRE
      EMMA7=AMIN 1(ABS(EMOI7),ABS(EPLU7))
      IF(EMMA7.GT.EMAX.AND.EMOI7.GT.0)GO TO 7100
      IF(EMMA7.GT.EMAX)GO TO 7200
      IF(EMOI7.LT.(-EMAX))EMOI7=-EMAX
      IF(EPLU7.GT.EMAX)EPLU7=EMAX
      ZZO=ZCO
      IF(EMOI7.LT.0.)ZZO=4.
      EMMI7=AMIN 1(ABS(EMOI7),ABS(EPLU7))
      IF(EMMI7*EMMI7.GE.ZZO*BPLU7*CPLU7.AND.EMOI7.GT.0)GO TO 7100
      IF(EMMI7*EMMI7.GE.ZZO*BPLU7*CPLU7)GO TO 7200
      KEMOI7=COEFF*EMOI7
      KEPLU7=COEFF*EPLU7
20074 IF(IIDD.EQ.0)GO TO 20075
      ID7=0
7310  DMOI7=DMOI6+ID7*PASD64
      DPLU7=DMOI7+PASD64
      IF((DMOI7-1E-8)**2.GT.ZCO*APLU7*BPLU7)GO TO 72000
      KDMOI7=COEFF*DMOI7
      KDPLU7=COEFF*DPLU7
20075 IF(IIEE.EQ.0)GO TO 20076
      IE7=0
7410  EMOI7=EMOI6+IE7*PASE64
      EPLU7=EMOI7+PASE64
      IF(EMOI7.LT.0.)GO TO 3830
      IF((EMOI7-1E-8).GT.SQRT(ZCO*BPLU7*CPLU7))GO TO 73000
      GO TO 3831
3830  IF(ABS(EPLU7-1E-8).GT.SQRT(4.*BPLU7*CPLU7))GO TO 7400
3831  KEMOI7=COEFF*EMOI7
      KEPLU7=COEFF*EPLU7
20076 IF(IIFF.EQ.0)GO TO 74000
      DO 7500 IMF7=0,1
      FMOI7=FMOI6+IMF7*PASF64
      DO 30017 I=1,KT
      ATEST=ABS(AMOI7-AAS(I))
      BTEST=ABS(BMOI7-BBS(I))
      CTEST=ABS(CMOI7-CCS(I))
      DTEST=ABS(DMOI7-DDS(I))
      ETEST=ABS(EMOI7-EES(I))
      ETESTY=ABS(ABS(EMOI7)-ABS(EES(I)))
      FTEST=ABS(FMOI7-FFS(I))
      IF(ATEST.LT.CXA.AND.BTEST.LT.CXB.AND.CTEST.LT.CXC.AND.DTEST.LT.
     +CXD.AND.ETEST.LT.CXE.AND.FTEST.LT.CXF)GO TO 75000
      IF(ATEST.LT.CYA.AND.BTEST.LT.CYA.AND.CTEST.LT.CYA.AND.DTEST.LT.
     +CYA.AND.ETESTY.LT.CYA.AND.FTEST.LT.CYA)GO TO 75000
30017 CONTINUE
      FPLU7=FMOI7+PASF64
      IF((FMOI7-1E-8)**2.GT.ZCO*APLU7*CPLU7)GO TO 74000
      KFMOI7=COEFF*FMOI7
      KFPLU7=COEFF*FPLU7
      CALL VOLUME(AMOI7,BMOI7,CMOI7,DMOI7,EMOI7,FMOI7,
     +APLU7,BPLU7,CPLU7,DPLU7,EPLU7,FPLU7)
      IF(LY.EQ.-1)GO TO 75000
      CALL TRIHKL1(KAMOI7,KBMOI7,KCMOI7,KDMOI7,KEMOI7,KFMOI7,
     +KAPLU7,KBPLU7,KCPLU7,KDPLU7,KEPLU7,KFPLU7,7)
1007  CONTINUE
      IF(NT.EQ.-1)GO TO 75000
      NDICH(7)=NDICH(7)+1
      DO 30007 I=1,N-1
      J=I+1
      IF(IRJ(I,7).EQ.1.AND.IRJ(J,7).EQ.1.AND.IH(I,1,7).EQ.IH(J,1,7).AND.
     +IK(I,1,7).EQ.IK(J,1,7).AND.IL(I,1,7).EQ.IL(J,1,7))GO TO 75000
30007 CONTINUE
      AMOI8=AMOI7
      APLU8=APLU7
      KAMOI8=COEFF*AMOI8
      KAPLU8=COEFF*APLU8
      BMOI8=BMOI7
      BPLU8=BPLU7
      KBMOI8=COEFF*BMOI8
      KBPLU8=COEFF*BPLU8
      CMOI8=CMOI7
      CPLU8=CPLU7
      KCMOI8=COEFF*CMOI8
      KCPLU8=COEFF*CPLU8
      DMOI8=DMOI7
      DPLU8=DPLU7
      KDMOI8=COEFF*DMOI8
      KDPLU8=COEFF*DPLU8
      EMOI8=EMOI7
      EPLU8=EPLU7
      KEMOI8=COEFF*EMOI8
      KEPLU8=COEFF*EPLU8
      FMOI8=FMOI7
      FPLU8=FPLU7
      KFMOI8=COEFF*FMOI8
      KFPLU8=COEFF*FPLU8
      IF(JJAA.EQ.1)GO TO 82201
      IF((APLU7-AMOI7).LT.PASA64)GO TO 82201
      NAP=NINT((APLU7-AMOI7)/PASA64)
      IAP7=0
82110 AMOI8=AMOI7+IAP7*PASA64
      APLU8=AMOI8+PASA64
      KAMOI8=COEFF*AMOI8
      KAPLU8=COEFF*APLU8
82201 IF(JJBB.EQ.1)GO TO 82301
      IF((BPLU7-BMOI7).LT.PASB64)GO TO 82301
      NBP=NINT((BPLU7-BMOI7)/PASB64)
      IBP7=0
82210 BMOI8=BMOI7+IBP7*PASB64
      BPLU8=BMOI8+PASB64
      KBMOI8=COEFF*BMOI8
      KBPLU8=COEFF*BPLU8
82301 IF(JJCC.EQ.1)GO TO 82401
      IF((CPLU7-CMOI7).LT.PASC64)GO TO 82401
      NCP=NINT((CPLU7-CMOI7)/PASC64)
      ICP7=0
82310 CMOI8=CMOI7+ICP7*PASC64
      CPLU8=CMOI8+PASC64
      KCMOI8=COEFF*CMOI8
      KCPLU8=COEFF*CPLU8
82401 IF(JJDD.EQ.1)GO TO 82501
      IF((DPLU7-DMOI7).LT.PASD64)GO TO 82501
      NDP=NINT((DPLU7-DMOI7)/PASD64)
      IDP7=0
82410 DMOI8=DMOI7+IDP7*PASD64
      DPLU8=DMOI8+PASD64
      KDMOI8=COEFF*DMOI8
      KDPLU8=COEFF*DPLU8
82501 IF(JJEE.EQ.1)GO TO 82601
      IF((EPLU7-EMOI7).LT.PASE64)GO TO 82601
      NEP=NINT((EPLU7-EMOI7)/PASE64)
      IEP7=0
82510 EMOI8=EMOI7+IEP7*PASE64
      EPLU8=EMOI8+PASE64
      KEMOI8=COEFF*EMOI8
      KEPLU8=COEFF*EPLU8
82601 IF(JJFF.EQ.1)GO TO 82603
      IF((FPLU7-FMOI7).LT.PASF64)GO TO 82603
      NFP=NINT((FPLU7-FMOI7)/PASF64)
      IFP7=0
82610 FMOI8=FMOI7+IFP7*PASF64
      FPLU8=FMOI8+PASF64
      KFMOI8=COEFF*FMOI8
      KFPLU8=COEFF*FPLU8
82603 CALL TRIHKL1(KAMOI8,KBMOI8,KCMOI8,KDMOI8,KEMOI8,KFMOI8,
     +KAPLU8,KBPLU8,KCPLU8,KDPLU8,KEPLU8,KFPLU8,8)
      IF(NT.EQ.-1)GO TO 82602
      NDICH(8)=NDICH(8)+1
      DO 30008 I=1,N-1
      J=I+1
      IF(IRJ(I,8).EQ.1.AND.IRJ(J,8).EQ.1.AND.IH(I,1,8).EQ.IH(J,1,8).AND.
     +IK(I,1,8).EQ.IK(J,1,8).AND.IL(I,1,8).EQ.IL(J,1,8))GO TO 82602
30008 CONTINUE
      AA=(AMOI8+APLU8)/2.
      BB=(BMOI8+BPLU8)/2.
      CC=(CMOI8+CPLU8)/2.
      DD=(DMOI8+DPLU8)/2.
      EE=(EMOI8+EPLU8)/2.
      FF=(FMOI8+FPLU8)/2.
      KTP=KTP+1
      KT=1+MOD(1,KTP)*KT
      AAS(KT)=AA
      BBS(KT)=BB
      CCS(KT)=CC
      DDS(KT)=DD
      EES(KT)=EE
      FFS(KT)=FF
      VET=.5*SQRT(4*AA*BB*CC+DD*EE*FF-AA*EE*EE-BB*FF*FF-CC*DD*DD)
      VR2=1/VET**2
      SV=SQRT(VR2)
      VAP=SV
      SINAL=SQRT(1-EE*EE/4/BB/CC)
      SINBE=SQRT(1-FF*FF/4/AA/CC)
      SINGA=SQRT(1-DD*DD/4/AA/BB)
      AR=SQRT(BB*CC)*SINAL/VET
      BR=SQRT(AA*CC)*SINBE/VET
      CR=SQRT(AA*BB)*SINGA/VET
      DR=PIDEG*ACOS((DD*FF/4/AA-EE/2)/SINBE/SINGA/SQRT(BB*CC))
      ER=PIDEG*ACOS((EE*DD/4/BB-FF/2)/SINAL/SINGA/SQRT(AA*CC))
      FR=PIDEG*ACOS((EE*FF/4/CC-DD/2)/SINAL/SINBE/SQRT(AA*BB))
      IF(KT.EQ.1)GO TO 4024
      DO 4025 J=1,KT-1
      ARTEST=ABS(AR-ARX(J))
      BRTEST=ABS(BR-BRX(J))
      CRTEST=ABS(CR-CRX(J))
      DRTEST=ABS(DR-DRX(J))
      ERTEST=ABS(ER-ERX(J))
      FRTEST=ABS(FR-FRX(J))
      DRSTEST=ABS(180.-DR-DRX(J))
      ERSTEST=ABS(180.-ER-ERX(J))
      FRSTEST=ABS(180.-FR-FRX(J))
      IF(ARTEST.LT..1.AND.BRTEST.LT..1.AND.CRTEST.LT..1.AND.(DRTEST.LT.
     +.5.OR.DRSTEST.LT..5).AND.(ERTEST.LT..5.OR.ERSTEST.LT..5).AND.(FRT
     +EST.LT..5.OR.FRSTEST.LT..5))GO TO 75000
 4025 CONTINUE
 4024 CONTINUE
      DO 15 I=1,N
      DO 16 J=1,20
      QCAL(I,J)=-1
   16 CONTINUE
   15 CONTINUE
C
C
      DO 45 I=1,N
      JJ=IRJ(I,8)
      DO 35 J=1,JJ
      QCAL(I,J)=AA*IH(I,J,8)**2+BB*IK(I,J,8)**2+CC*IL(I,J,8)**2+DD
     +*IH(I,J,8)*IK(I,J,8)+EE*IK(I,J,8)*IL(I,J,8)+FF*IH(I,J,8)*
     +IL(I,J,8)
 35   CONTINUE
 45   CONTINUE
      IND=6
      NRIND=6
      CALL AFFPAR(IND,NRIND,VAP)
      ARX(KT)=AR
      BRX(KT)=BR
      CRX(KT)=CR
      DRX(KT)=DR
      ERX(KT)=ER
      FRX(KT)=FR
      IF(FWOLFF.EQ.-1000.)THEN
      KTP=KTP-1
      IF(KT.EQ.1)THEN
      AAS(1)=1000.
      BBS(1)=1000.
      CCS(1)=1000.
      DDS(1)=1000.
      EES(1)=1000.
      FFS(1)=1000.
      ELSE
      KT=KT-1
      ENDIF
      GO TO 82602
      ENDIF
      VR2=V*V
      WRITE(IW,611)(NDICH(I),I=1,8)
 611  FORMAT(///,4X,8I8,///)
82602 IF(JJFF.EQ.1)GO TO 82502
82600 IFP7=IFP7+1
      IF(IFP7.LE.NFP)GO TO 82610
82502 IF(JJEE.EQ.1)GO TO 82402
82500 IEP7=IEP7+1
      IF(IEP7.LE.NEP)GO TO 82510
82402 IF(JJDD.EQ.1)GO TO 82302
82400 IDP7=IDP7+1
      IF(IDP7.LE.NDP)GO TO 82410
82302 IF(JJCC.EQ.1)GO TO 82202
82300 ICP7=ICP7+1
      IF(ICP7.LE.NCP)GO TO 82310
82202 IF(JJBB.EQ.1)GO TO 82102
82200 IBP7=IBP7+1
      IF(IBP7.LE.NBP)GO TO 82210
82102 IF(JJAA.EQ.1)GO TO 75000
82100 IAP7=IAP7+1
      IF(IAP7.LE.NAP)GO TO 82110
75000 IF(IIFF.EQ.0)GO TO 74000
7500  CONTINUE
74000 IF(IIEE.EQ.0)GO TO 73000
7400  IE7=IE7+1
      IF(IE7.LE.1)GO TO 7410
73000 IF(IIDD.EQ.0)GO TO 72000
7300  ID7=ID7+1
      IF(ID7.LE.1)GO TO 7310
72000 IF(IICC.EQ.0)GO TO 71000
7200  IC7=IC7+1
      IF(IC7.LE.1)GO TO 7210
71000 IF(IIBB.EQ.0)GO TO 70000
7100  IB7=IB7+1
      IF(IB7.LE.1)GO TO 7110
70000 IF(IIAA.EQ.0)GO TO 65000
7000  IA7=IA7+1
      IF(IA7.LE.1)GO TO 7010
65000 IF(IIFF.EQ.0)GO TO 64000
6500  CONTINUE
64000 IF(IIEE.EQ.0)GO TO 63000
6400  IE6=IE6+1
      IF(IE6.LE.1)GO TO 6410
63000 IF(IIDD.EQ.0)GO TO 62000
6300  ID6=ID6+1
      IF(ID6.LE.1)GO TO 6310
62000 IF(IICC.EQ.0)GO TO 61000
6200  IC6=IC6+1
      IF(IC6.LE.1)GO TO 6210
61000 IF(IIBB.EQ.0)GO TO 60000
6100  IB6=IB6+1
      IF(IB6.LE.1)GO TO 6110
60000 IF(IIAA.EQ.0)GO TO 55000
6000  IA6=IA6+1
      IF(IA6.LE.1)GO TO 6010
55000 IF(IIFF.EQ.0)GO TO 54000
5500  CONTINUE
54000 IF(IIEE.EQ.0)GO TO 53000
5400  IE5=IE5+1
      IF(IE5.LE.1)GO TO 5410
53000 IF(IIDD.EQ.0)GO TO 52000
5300  ID5=ID5+1
      IF(ID5.LE.1)GO TO 5310
52000 IF(IICC.EQ.0)GO TO 51000
5200  IC5=IC5+1
      IF(IC5.LE.1)GO TO 5210
51000 IF(IIBB.EQ.0)GO TO 50000
5100  IB5=IB5+1
      IF(IB5.LE.1)GO TO 5110
50000 IF(IIAA.EQ.0)GO TO 45000
5000  IA5=IA5+1
      IF(IA5.LE.1)GO TO 5010
45000 IF(IIFF.EQ.0)GO TO 44000
4500  CONTINUE
44000 IF(IIEE.EQ.0)GO TO 43000
4400  IE4=IE4+1
      IF(IE4.LE.1)GO TO 4410
43000 IF(IIDD.EQ.0)GO TO 42000
4300  ID4=ID4+1
      IF(ID4.LE.1)GO TO 4310
42000 IF(IICC.EQ.0)GO TO 41000
4200  IC4=IC4+1
      IF(IC4.LE.1)GO TO 4210
41000 IF(IIBB.EQ.0)GO TO 40000
4100  IB4=IB4+1
      IF(IB4.LE.1)GO TO 4110
40000 IF(IIAA.EQ.0)GO TO 35000
4000  IA4=IA4+1
      IF(IA4.LE.1)GO TO 4010
35000 IF(IIFF.EQ.0)GO TO 34000
3500  CONTINUE
34000 IF(IIEE.EQ.0)GO TO 33000
3400  IE3=IE3+1
      IF(IE3.LE.1)GO TO 3410
33000 IF(IIDD.EQ.0)GO TO 32000
3300  ID3=ID3+1
      IF(ID3.LE.1)GO TO 3310
32000 IF(IICC.EQ.0)GO TO 31000
3200  IC3=IC3+1
      IF(IC3.LE.1)GO TO 3210
31000 IF(IIBB.EQ.0)GO TO 30000
3100  IB3=IB3+1
      IF(IB3.LE.1)GO TO 3110
30000 IF(IIAA.EQ.0)GO TO 25000
3000  IA3=IA3+1
      IF(IA3.LE.1)GO TO 3010
25000 IF(IIFF.EQ.0)GO TO 24000
2500  CONTINUE
24000 IF(IIEE.EQ.0)GO TO 23000
2400  IE2=IE2+1
      IF(IE2.LE.1)GO TO 2410
23000 IF(IIDD.EQ.0)GO TO 22000
2300  ID2=ID2+1
      IF(ID2.LE.1)GO TO 2310
22000 IF(IICC.EQ.0)GO TO 21000
2200  IC2=IC2+1
      IF(IC2.LE.1)GO TO 2210
21000 IF(IIBB.EQ.0)GO TO 20000
2100  IB2=IB2+1
      IF(IB2.LE.1)GO TO 21010
20000 IF(IIAA.EQ.0)GO TO 15000
2000  IA2=IA2+1
      IF(IA2.LE.1)GO TO 2010
15000 IF(IIFF.EQ.0)GO TO 14000
1500  CONTINUE
14000 IF(IIEE.EQ.0)GO TO 13000
1400  IE=IE+1
      IF(IE.LE.NE)GO TO 1410
13000 IF(IIDD.EQ.0)GO TO 12000
1300  ID=ID+1
      IF(ID.LE.ND)GO TO 1310
12000 IF(IICC.EQ.0)GO TO 11000
1200  IC=IC+1
      IF(IC.LE.NC)GO TO 1210
11000 IF(IIBB.EQ.0)GO TO 10000
1100  IB=IB+1
      IF(IB.LE.NB)GO TO 1110
10000 IF(IIAA.EQ.0)GO TO 80008
1000  IA=IA+1
      IF(IA.LE.NA)GO TO 1010
80008 IF(JRT.GE.JRD)GO TO 80088
      JRT=JRT+1
      GO TO IAS, (4001,4006,4009,4023)
80088 IF(JRD.NE.0)THEN
      IF(NDICH(8).NE.0)GO TO 80009
      CMAX=CMA
      DMAX=DMA
      DMIN=DMI
      CMIN=CMI
      FMAX=FMA
      EMAX=EMA
      JRD=0
      JJDD=1
      IIDD=1
      GO TO 2038
      ENDIF
80009 CONTINUE
      IF(NDICH(8).NE.0)GO TO 8501
8500  CONTINUE
8501  CONTINUE
      RETURN
      END
      SUBROUTINE TRIHKL(KAMOI,KBMOI,KCMOI,KDMOI,KEMOI,KFMOI,
     +KAPLU,KBPLU,KCPLU,KDPLU,KEPLU,KFPLU)
      COMMON D(50),Q(50),KQ(50),TH(50),EPSIL(50),EPSQ(50),KEPSQ(50),DTH,
     1       V,VINF,VSUP,PAS,PAS2,PAS4,PAS8,PAS16,PAS32,PAS64,COEFF,RAP,
     2       WAVE2,PIDEG,PIRAD,N,MH,MH2,MK,MK2,ML,ML2,KDENS,KZ,KQT(50)
      COMMON /BAF/NT,IRJ(50,8),IH(50,50,8),IK(50,50,8),IL(50,100,8)
     1           ,NPOSS,FWOLFF
      COMMON /AMAX DD/AMAX,DD1,DD2
      COMMON /AMAX/AMIN,BMAX,BMIN,KAMAX,KAMIN,KBMAX,KBMIN
     +,DMIN,DMAX,AMN,BMN,KDMIN,KDMAX,DMN
      COMMON /N5/N5,N6
      COMMON /MT/MT,IRJB(50),IHB(50,50),IKB(50,50),ILB(50,50)
      MT=0
      DO 15 I=1,N
      IRJ(I,1)=0
      IRJB(I)=0
  15  CONTINUE
      DO 10 M=0,MH
      MDK=-MK*MOD(1,1+M)
      MCARHP=M*M*KAPLU
      MCARHM=M*M*KAMOI
      DO 20 K=MDK,MK
      IF((M+ABS(K)).GT.6)GO TO 20
      MDL=-ML
      IF((MOD(1,1+M)+MOD(1,1+K*K)).EQ.0)MDL=0
      MCARKP=K*K*KBPLU
      MCARKM=K*K*KBMOI
      DO 30 MXL=MDL,ML
      IF((M+ABS(K)+ABS(MXL)).GT.6)GO TO 30
      MCARLP=MXL*MXL*KCPLU
      MCARLM=MXL*MXL*KCMOI
      IF(M*K)42,44,44
 42   MCO1P=KDMOI*M*K
      MCO1M=KDPLU*M*K
      GO TO 46
 44   MCO1P=KDPLU*M*K
      MCO1M=KDMOI*M*K
 46   IF(K*MXL)72,74,74
 72   MCO2P=KEMOI*K*MXL
      MCO2M=KEPLU*K*MXL
      GO TO 76
 74   MCO2P=KEPLU*K*MXL
      MCO2M=KEMOI*K*MXL
 76   IF(M*MXL)82,84,84
 82   MCO3P=KFMOI*M*MXL
      MCO3M=KFPLU*M*MXL
      GO TO 86
 84   MCO3P=KFPLU*M*MXL
      MCO3M=KFMOI*M*MXL
 86   KQQM=MCARHM+MCARKM+MCARLM+MCO1M+MCO2M+MCO3M
      IF((KQQM-KEPSQ(N5)).GT.KQ(N5))GO TO 30
      KQQP=MCARHP+MCARKP+MCARLP+MCO1P+MCO2P+MCO3P
      IF((KQQP+KEPSQ(1)).LT.KQ(1))GO TO 30
      DO 40 I=1,N5
      IF((KQQM-KEPSQ(I)).GT.KQ(I))GO TO 40
      IF((KQQP+KEPSQ(I)).LT.KQ(I))GO TO 40
      IRJ(I,1)=IRJ(I,1)+1
      JJ=IRJ(I,1)
      IH(I,JJ,1)=M
      IK(I,JJ,1)=K
      IL(I,JJ,1)=MXL
      IF(M*MXL.NE.0)GO TO 40
      IRJB(I)=IRJB(I)+1
      JJB=IRJB(I)
      IHB(I,JJB)=M
      IKB(I,JJB)=K
      ILB(I,JJB)=MXL
 40   CONTINUE
 30   CONTINUE
 20   CONTINUE
 10   CONTINUE
      DO 70 I=1,N5
      IF(IRJ(I,1).NE.0)GO TO 70
      MT=-1
      GO TO 3000
  70  CONTINUE
      DO 80 M=0,MH2
      MDK2=-MK2*MOD(1,1+M)
      MCARHP=M*M*KAPLU
      MCARHM=M*M*KAMOI
      DO 90 K=MDK2,MK2
      IF((M+ABS(K)).GT.6)GO TO 90
      MDL2=-ML2
      IF((MOD(1,1+M)+MOD(1,1+K*K)).EQ.0)MDL2=0
      MCARKP=K*K*KBPLU
      MCARKM=K*K*KBMOI
      DO 100 MXL=MDL2,ML2
      IF((M+ABS(K)+ABS(MXL)).GT.6)GO TO 100
      MCARLP=MXL*MXL*KCPLU
      MCARLM=MXL*MXL*KCMOI
      IF(M*K)142,144,144
142   MCO1P=KDMOI*M*K
      MCO1M=KDPLU*M*K
      GO TO 146
144   MCO1P=KDPLU*M*K
      MCO1M=KDMOI*M*K
146   IF(K*MXL)172,174,174
172   MCO2P=KEMOI*K*MXL
      MCO2M=KEPLU*K*MXL
      GO TO 176
174   MCO2P=KEPLU*K*MXL
      MCO2M=KEMOI*K*MXL
176   IF(M*MXL)182,184,184
182   MCO3P=KFMOI*M*MXL
      MCO3M=KFPLU*M*MXL
      GO TO 186
184   MCO3P=KFPLU*M*MXL
      MCO3M=KFMOI*M*MXL
186   KQQM=MCARHM+MCARKM+MCARLM+MCO1M+MCO2M+MCO3M
      IF((KQQM-KEPSQ(N)).GT.KQ(N))GO TO 100
      KQQP=MCARHP+MCARKP+MCARLP+MCO1P+MCO2P+MCO3P
      IF((KQQP+KEPSQ(N6)).LT.KQ(N6))GO TO 100
      DO 110 I=N6,N
      IF((KQQM-KEPSQ(I)).GT.KQ(I))GO TO 110
      IF((KQQP+KEPSQ(I)).LT.KQ(I))GO TO 110
      IRJ(I,1)=IRJ(I,1)+1
      JJ=IRJ(I,1)
      IH(I,JJ,1)=M
      IK(I,JJ,1)=K
      IL(I,JJ,1)=MXL
      IF(M*MXL.NE.0)GO TO 110
      IRJB(I)=IRJB(I)+1
      JJB=IRJB(I)
      IHB(I,JJB)=M
      IKB(I,JJB)=K
      ILB(I,JJB)=MXL
 110  CONTINUE
 100  CONTINUE
  90  CONTINUE
  80  CONTINUE
3000  RETURN
      END
      SUBROUTINE TRIHKLB(KAMOI,KBMOI,KCMOI,KDMOI,KEMOI,KFMOI,
     +KAPLU,KBPLU,KCPLU,KDPLU,KEPLU,KFPLU)
      COMMON D(50),Q(50),KQ(50),TH(50),EPSIL(50),EPSQ(50),KEPSQ(50),DTH,
     1       V,VINF,VSUP,PAS,PAS2,PAS4,PAS8,PAS16,PAS32,PAS64,COEFF,RAP,
     2       WAVE2,PIDEG,PIRAD,N,MH,MH2,MK,MK2,ML,ML2,KDENS,KZ,KQT(50)
      COMMON /BAF/NT,IRJ(50,8),IH(50,50,8),IK(50,50,8),IL(50,100,8)
     1           ,NPOSS,FWOLFF
      COMMON /AMAX DD/AMAX,DD1,DD2
      COMMON /AMAX/AMIN,BMAX,BMIN,KAMAX,KAMIN,KBMAX,KBMIN
     +,DMIN,DMAX,AMN,BMN,KDMIN,KDMAX,DMN
      COMMON /N5/N5,N6
      COMMON /MT/MT,IRJB(50),IHB(50,50),IKB(50,50),ILB(50,50)
      NT=0
      DO 15 I=1,N
      IRJ(I,1)=0
  15  CONTINUE
      DO 10 M=1,MH
      MDK=-MK*MOD(1,1+M)
      MCARHP=M*M*KAPLU
      MCARHM=M*M*KAMOI
      DO 20 K=MDK,MK
      IF((M+ABS(K)).GT.6)GO TO 20
      MDL=-ML
      IF((MOD(1,1+M)+MOD(1,1+K*K)).EQ.0)MDL=0
      MCARKP=K*K*KBPLU
      MCARKM=K*K*KBMOI
      DO 30 MXL=MDL,ML
      IF((M+ABS(K)+ABS(MXL)).GT.6)GO TO 30
      IF(MXL.EQ.0)GO TO 30
      MCARLP=MXL*MXL*KCPLU
      MCARLM=MXL*MXL*KCMOI
      IF(M*K)42,44,44
 42   MCO1P=KDMOI*M*K
      MCO1M=KDPLU*M*K
      GO TO 46
 44   MCO1P=KDPLU*M*K
      MCO1M=KDMOI*M*K
 46   IF(K*MXL)72,74,74
 72   MCO2P=KEMOI*K*MXL
      MCO2M=KEPLU*K*MXL
      GO TO 76
 74   MCO2P=KEPLU*K*MXL
      MCO2M=KEMOI*K*MXL
 76   IF(M*MXL)82,84,84
 82   MCO3P=KFMOI*M*MXL
      MCO3M=KFPLU*M*MXL
      GO TO 86
 84   MCO3P=KFPLU*M*MXL
      MCO3M=KFMOI*M*MXL
 86   KQQM=MCARHM+MCARKM+MCARLM+MCO1M+MCO2M+MCO3M
      IF((KQQM-KEPSQ(N5)).GT.KQ(N5))GO TO 30
      KQQP=MCARHP+MCARKP+MCARLP+MCO1P+MCO2P+MCO3P
      IF((KQQP+KEPSQ(1)).LT.KQ(1))GO TO 30
      DO 40 I=1,N5
      IF((KQQM-KEPSQ(I)).GT.KQ(I))GO TO 40
      IF((KQQP+KEPSQ(I)).LT.KQ(I))GO TO 40
      IRJ(I,1)=IRJ(I,1)+1
      JJ=IRJ(I,1)
      IH(I,JJ,1)=M
      IK(I,JJ,1)=K
      IL(I,JJ,1)=MXL
 40   CONTINUE
 30   CONTINUE
 20   CONTINUE
 10   CONTINUE
      DO 70 I=1,N5
      IF((IRJ(I,1)+IRJB(I)).NE.0)GO TO 70
      NT=-1
      GO TO 3000
  70  CONTINUE
      DO 80 M=1,MH2
      MDK2=-MK2*MOD(1,1+M)
      MCARHP=M*M*KAPLU
      MCARHM=M*M*KAMOI
      DO 90 K=MDK2,MK2
      IF((M+ABS(K)).GT.6)GO TO 90
      MDL2=-ML2
      IF((MOD(1,1+M)+MOD(1,1+K*K)).EQ.0)MDL2=0
      MCARKP=K*K*KBPLU
      MCARKM=K*K*KBMOI
      DO 100 MXL=MDL2,ML2
      IF((M+ABS(K)+ABS(MXL)).GT.6)GO TO 100
      IF(MXL.EQ.0)GO TO 100
      MCARLP=MXL*MXL*KCPLU
      MCARLM=MXL*MXL*KCMOI
      IF(M*K)142,144,144
142   MCO1P=KDMOI*M*K
      MCO1M=KDPLU*M*K
      GO TO 146
144   MCO1P=KDPLU*M*K
      MCO1M=KDMOI*M*K
146   IF(K*MXL)172,174,174
172   MCO2P=KEMOI*K*MXL
      MCO2M=KEPLU*K*MXL
      GO TO 176
174   MCO2P=KEPLU*K*MXL
      MCO2M=KEMOI*K*MXL
176   IF(M*MXL)182,184,184
182   MCO3P=KFMOI*M*MXL
      MCO3M=KFPLU*M*MXL
      GO TO 186
184   MCO3P=KFPLU*M*MXL
      MCO3M=KFMOI*M*MXL
186   KQQM=MCARHM+MCARKM+MCARLM+MCO1M+MCO2M+MCO3M
      IF((KQQM-KEPSQ(N)).GT.KQ(N))GO TO 100
      KQQP=MCARHP+MCARKP+MCARLP+MCO1P+MCO2P+MCO3P
      IF((KQQP+KEPSQ(N6)).LT.KQ(N6))GO TO 100
      DO 110 I=N6,N
      IF((KQQM-KEPSQ(I)).GT.KQ(I))GO TO 110
      IF((KQQP+KEPSQ(I)).LT.KQ(I))GO TO 110
      IRJ(I,1)=IRJ(I,1)+1
      JJ=IRJ(I,1)
      IH(I,JJ,1)=M
      IK(I,JJ,1)=K
      IL(I,JJ,1)=MXL
 110  CONTINUE
 100  CONTINUE
  90  CONTINUE
  80  CONTINUE
3000  RETURN
      END
      SUBROUTINE TRIHKL1(KAMOI,KBMOI,KCMOI,KDMOI,KEMOI,KFMOI,
     +KAPLU,KBPLU,KCPLU,KDPLU,KEPLU,KFPLU,K1)
      COMMON D(50),Q(50),KQ(50),TH(50),EPSIL(50),EPSQ(50),KEPSQ(50),DTH,
     1       V,VINF,VSUP,PAS,PAS2,PAS4,PAS8,PAS16,PAS32,PAS64,COEFF,RAP,
     2       WAVE2,PIDEG,PIRAD,N,MH,MH2,MK,MK2,ML,ML2,KDENS,KZ,KQT(50)
      COMMON /BAF/NT,IRJ(50,8),IH(50,50,8),IK(50,50,8),IL(50,100,8)
     1           ,NPOSS,FWOLFF
      COMMON /AMAX DD/AMAX,DD1,DD2
      COMMON /AMAX/AMIN,BMAX,BMIN,KAMAX,KAMIN,KBMAX,KBMIN
     +,DMIN,DMAX,AMN,BMN,KDMIN,KDMAX,DMN
      NT=0
      KK=K1-1
      DO 5 I=1,N
      JJ=IRJ(I,KK)
      IRJ(I,K1)=0
      DO 10 J=1,JJ
      M=IH(I,J,KK)
      K=IK(I,J,KK)
      MXL=IL(I,J,KK)
      MCARHP=M*M*KAPLU
      MCARHM=M*M*KAMOI
      MCARKP=K*K*KBPLU
      MCARKM=K*K*KBMOI
      MCARLP=MXL*MXL*KCPLU
      MCARLM=MXL*MXL*KCMOI
      IF(M*K)42,44,44
 42   MCO1P=KDMOI*M*K
      MCO1M=KDPLU*M*K
      GO TO 46
 44   MCO1P=KDPLU*M*K
      MCO1M=KDMOI*M*K
 46   IF(K*MXL)72,74,74
 72   MCO2P=KEMOI*K*MXL
      MCO2M=KEPLU*K*MXL
      GO TO 76
 74   MCO2P=KEPLU*K*MXL
      MCO2M=KEMOI*K*MXL
 76   IF(M*MXL)82,84,84
 82   MCO3P=KFMOI*M*MXL
      MCO3M=KFPLU*M*MXL
      GO TO 86
 84   MCO3P=KFPLU*M*MXL
      MCO3M=KFMOI*M*MXL
 86   KQQM=MCARHM+MCARKM+MCARLM+MCO1M+MCO2M+MCO3M
      IF((KQQM-KEPSQ(I)).GT.KQ(I))GO TO 10
      KQQP=MCARHP+MCARKP+MCARLP+MCO1P+MCO2P+MCO3P
      IF((KQQP+KEPSQ(I)).LT.KQ(I))GO TO 10
      IRJ(I,K1)=IRJ(I,K1)+1
      KJ=IRJ(I,K1)
      IH(I,KJ,K1)=M
      IK(I,KJ,K1)=K
      IL(I,KJ,K1)=MXL
 10   CONTINUE
      IF(IRJ(I,K1).NE.0)GO TO 5
      NT=-1
      GO TO 55
  5   CONTINUE
 55   RETURN
      END
      SUBROUTINE VOLUME(AMOI,BMOI,CMOI,DMOI,EMOI,FMOI,
     +APLU,BPLU,CPLU,DPLU,EPLU,FPLU)
      COMMON /AMAX DD/AMAX,DD1,DD2
      COMMON /AMAX/AMIN,BMAX,BMIN,KAMAX,KAMIN,KBMAX,KBMIN
     +,DMIN,DMAX,AMN,BMN,KDMIN,KDMAX,DMN
      COMMON /VMAX/VMAX2,VMIN2,VR2,LY,VMOII
      FX1(X,Y,O,P1,Q1)=.25*(4.*O*P1*Q1-O*X*X-P1*Y*Y+X*X*Y*Y/
     +4./Q1)
      FX2(Z,X,Y,O,P1,Q1)=.25*(4.*O*P1*Q1-O*X*X-P1*Y*Y-Q1*Z*Z
     ++X*Y*Z)
      LY=0
      EMOIR=AMIN 1(ABS(EMOI),ABS(EPLU))
      EPLUR=AMAX 1(ABS(EMOI),ABS(EPLU))
      IF(FPLU**2.GE.4.*AMOI*CMOI.OR.DPLU**2.GE.4.*AMOI*BMOI.OR.EPLUR**2
     +.GE.4.*BMOI*CMOI)GO TO 10
      EF1=0.
      EF2=0.
      EF3=0.
      EF4=0.
      ED1=0.
      ED2=0.
      ED3=0.
      ED4=0.
      FD1=0.
      FD2=0.
      FD3=0.
      FD4=0.
      FP1=FX2(DMOI,EMOI,FMOI,APLU,BPLU,CPLU)
      FP2=FX2(DMOI,EMOI,FPLU,APLU,BPLU,CPLU)
      FP3=FX2(DMOI,EPLU,FMOI,APLU,BPLU,CPLU)
      FP4=FX2(DMOI,EPLU,FPLU,APLU,BPLU,CPLU)
      FP5=FX2(DPLU,EMOI,FMOI,APLU,BPLU,CPLU)
      FP6=FX2(DPLU,EMOI,FPLU,APLU,BPLU,CPLU)
      FP7=FX2(DPLU,EPLU,FMOI,APLU,BPLU,CPLU)
      FP8=FX2(DPLU,EPLU,FPLU,APLU,BPLU,CPLU)
      FM1=FX2(DMOI,EMOI,FMOI,AMOI,BMOI,CMOI)
      FM2=FX2(DMOI,EMOI,FPLU,AMOI,BMOI,CMOI)
      FM3=FX2(DMOI,EPLU,FMOI,AMOI,BMOI,CMOI)
      FM4=FX2(DMOI,EPLU,FPLU,AMOI,BMOI,CMOI)
      FM5=FX2(DPLU,EMOI,FMOI,AMOI,BMOI,CMOI)
      FM6=FX2(DPLU,EMOI,FPLU,AMOI,BMOI,CMOI)
      FM7=FX2(DPLU,EPLU,FMOI,AMOI,BMOI,CMOI)
      FM8=FX2(DPLU,EPLU,FPLU,AMOI,BMOI,CMOI)
      IF(DMOI.LE.EMOI*FMOI/2./CPLU.AND.DPLU.GE.EMOI*FMOI/2./
     +CPLU)EF1=FX1(EMOI,FMOI,APLU,BPLU,CPLU)
      IF(DMOI.LE.EPLU*FMOI/2./CPLU.AND.DPLU.GE.EPLU*FMOI/2./
     +CPLU)EF2=FX1(EPLU,FMOI,APLU,BPLU,CPLU)
      IF(DMOI.LE.EMOI*FPLU/2./CPLU.AND.DPLU.GE.EMOI*FPLU/2./
     +CPLU)EF3=FX1(EMOI,FPLU,APLU,BPLU,CPLU)
      IF(DMOI.LE.EPLU*FPLU/2./CPLU.AND.DPLU.GE.EPLU*FPLU/2./
     +CPLU)EF4=FX1(EPLU,FPLU,APLU,BPLU,CPLU)
      IF(EMOI.LE.FMOI*DMOI/2./APLU.AND.EPLU.GE.FMOI*DMOI/2./
     +APLU)FD1=FX1(FMOI,DMOI,BPLU,CPLU,APLU)
      IF(EMOI.LE.FPLU*DMOI/2./APLU.AND.EPLU.GE.FPLU*DMOI/2./
     +APLU)FD2=FX1(FPLU,DMOI,BPLU,CPLU,APLU)
      IF(EMOI.LE.FMOI*DPLU/2./APLU.AND.EPLU.GE.FMOI*DPLU/2./
     +APLU)FD3=FX1(FMOI,DPLU,BPLU,CPLU,APLU)
      IF(EMOI.LE.FPLU*DPLU/2./APLU.AND.EPLU.GE.FPLU*DPLU/2./
     +APLU)FD4=FX1(FPLU,DPLU,BPLU,CPLU,APLU)
      IF(FMOI.LE.EMOI*DMOI/2./BPLU.AND.FPLU.GE.EMOI*DMOI/2./
     +BPLU)ED1=FX1(EMOI,DMOI,APLU,CPLU,BPLU)
      IF(FMOI.LE.EPLU*DMOI/2./BPLU.AND.FPLU.GE.EPLU*DMOI/2./
     +BPLU)ED2=FX1(EPLU,DMOI,APLU,CPLU,BPLU)
      IF(FMOI.LE.EMOI*DPLU/2./BPLU.AND.FPLU.GE.EMOI*DPLU/2./
     +BPLU)ED3=FX1(EMOI,DPLU,APLU,CPLU,BPLU)
      IF(FMOI.LE.EPLU*DPLU/2./BPLU.AND.FPLU.GE.EPLU*DPLU/2./
     +BPLU)ED4=FX1(EPLU,DPLU,APLU,CPLU,BPLU)
      VPLU=AMAX 1(EF1,EF2,EF3,EF4,FD1,FD2,FD3,FD4,ED1,ED2,ED3,ED4,
     +FP1,FP2,FP3,FP4,FP5,FP6,FP7,FP8)
      VMOI=AMIN 1(FM1,FM2,FM3,FM4,FM5,FM6,FM7,FM8)
      IF(VPLU.LE.0.)GO TO 1500
      VMOI=AMAX 1(6.25E-8,VMOI)
      VTESTP=1/VMOI
      VTESTM=1/VPLU
      VVR=AMAX 1(ABS(SQRT(VR2)-SQRT(VTESTP)),ABS(SQRT(VR2)
     +-SQRT(VTESTM)))
      VMOII=SQRT(VMOI)
      IF(VTESTM.GT.VMAX2)GO TO 1500
      IF(VTESTP.LT.VMIN2)GO TO 1500
      IF(SQRT(VTESTM).GT.(1.02*SQRT(VR2)))GO TO 1500
      GO TO 155
  10  VMOI=.25*(4.*AMOI*BMOI*CMOI+DMOI*EMOIR*FMOI-APLU*EPLUR**2-
     +BPLU*FPLU**2-CPLU*DPLU**2)
      VPLU=.25*(4.*APLU*BPLU*CPLU+DPLU*EPLUR*FPLU-AMOI*EMOIR**2-
     +BMOI*FMOI**2-CMOI*DMOI**2)
      IF(VPLU.LE.0.)GO TO 1500
      VMOI=AMAX 1(6.25E-8,VMOI)
      VTESTP=1/VMOI
      VTESTM=1/VPLU
      VVR=AMAX 1(ABS(SQRT(VR2)-SQRT(VTESTP)),ABS(SQRT(VR2)
     +-SQRT(VTESTM)))
      VMOII=SQRT(VMOI)
      IF(VTESTM.GT.VMAX2)GO TO 1500
      IF(VTESTP.LT.VMIN2)GO TO 1500
      IF(SQRT(VTESTM).GT.(1.02*SQRT(VR2)))GO TO 1500
      GO TO 155
1500  LY=-1
155   RETURN
      END
C
C
      SUBROUTINE RELSPE
C
      COMMON D(50),Q(50),KQ(50),TH(50),EPSIL(50),EPSQ(50),KEPSQ(50),DTH,
     1       V,VINF,VSUP,PAS,PAS2,PAS4,PAS8,PAS16,PAS32,PAS64,COEFF,RAP,
     2       WAVE2,PIDEG,PIRAD,N,MH,MH2,MK,MK2,ML,ML2,KDENS,KZ,KQT(50)
      COMMON /AMAX DD/AMAX,DD1,DD2
      COMMON /AMAX/AMIN,BMAX,BMIN,KAMAX,KAMIN,KBMAX,KBMIN
     +,DMIN,DMAX,AMN,BMN,KDMIN,KDMAX,DMN
      COMMON /IIDD/IIDD,CMAX,JRD,RMIN(50),RMAX(50),RCMAX(50)
      COMMON /DMOI/DMOI1,DMOI2,DMOI3,DMOI4,DMOI5,DMOI6,DMOI7,DPLU1,DPLU2
     +,DPLU3,DPLU4,DPLU5,DPLU6,DPLU7,KDMOI1,KDMOI2,KDMOI3,KDMOI4,KDMOI5,
     +KDMOI6,KDMOI7,KDPLU1,KDPLU2,KDPLU3,KDPLU4,KDPLU5,KDPLU6,KDPLU7
      DO 2086 NH1=1,4
      DO 2087 NK1=1,4
      IF((NH1+ABS(NK1)).GT.6)GO TO 2087
      EPR1=NH1*NH1*(AMAX-AMIN)+NK1*NK1*(BMAX-BMIN)
      QAB=AMN*NH1*NH1+BMN*NK1*NK1
      J1=0
      DO 2081 I=2,N
      IF((Q(I)-EPSQ(I)).GE.QAB.OR.(Q(I+1)+EPSQ(I+1)).LE.QAB)
     +GO TO 2081
      J1=I
      J2=I+1
      GO TO 2082
2081  CONTINUE
2082  IF(J1.EQ.0)GO TO 2087
2083  EPR=EPR1+EPSQ(J1)+EPSQ(J2)
      IF((ABS(2.*QAB-Q(J1)-Q(J2)).LT.EPR).AND.
     +(J1.GE.5.OR.(ABS(NH1)+ABS(NK1)).LE.3))THEN
      JRD=JRD+1
      DAX=(Q(J2)-Q(J1))/2./NH1/NK1
      RMIN(JRD)=DAX-(EPSQ(J1)+EPSQ(J2))/2.
      IF(RMIN(JRD).LT.0.)RMIN(JRD)=0.
      RMAX(JRD)=DAX+(EPSQ(J1)+EPSQ(J2))/2.
      RMN=(RMAX(JRD)+RMIN(JRD))/2.
      J=0
      DO 2144 I=1,N
      DO 2145 JH=0,MH2
      ERI22=JH*JH*(AMAX-AMIN)/2.+EPSQ(I)
      DO 2146 JK=-MK2,MK2
      IF((JH+ABS(JK)).GT.6)GO TO 2146
      ERI2=ERI22+JK*JK*(BMAX-BMIN)/2.+JH*JK*(RMAX(JRD)-RMIN(JRD))/2.
      IF(ABS(AMN*JH*JH+BMN*JK*JK+RMN*JH*JK-Q(I)).LT.ERI2)GO TO 2144
2146  CONTINUE
2145  CONTINUE
      J=J+1
      IF(J.EQ.1)GO TO 2144
      RCMAX(JRD)=Q(I)
      GO TO 2147
2144  CONTINUE
      IF(J.LT.2)RCMAX(JRD)=Q(N)
2147  CONTINUE
      IIDD=0
      GO TO 2085
      ELSE
      IF((2.*QAB-Q(J1)-Q(J2)).LT.0.)GO TO 2085
      J2=J2+1
      IF(J2.GT.N)GO TO 2087
      GO TO 2083
      ENDIF
2085  J1=J1-1
      IF(J1.LT.3)GO TO 2087
      GO TO 2083
2087  CONTINUE
2086  CONTINUE
2088  RETURN
      END
C     ------------------------------------------------------------------
C     |       A F F I N E M E N T   D E S   P A R A M E T R E S        |
C     ------------------------------------------------------------------
      SUBROUTINE AFFPAR(IND,NRIND,VAP)
      DIMENSION JH(50),JK(50),JL(50),POIDS(50),SSQ(50),SSC(50),DIR(6)
      DIMENSION A(7,7),B(6,6),E(7),ECART(6),ERROR(6),TEM(6)
      COMMON D(50),Q(50),KQ(50),TH(50),EPSIL(50),EPSQ(50),KEPSQ(50),DTH,
     1       V,VINF,VSUP,PAS,PAS2,PAS4,PAS8,PAS16,PAS32,PAS64,COEFF,RAP,
     2       WAVE2,PIDEG,PIRAD,N,MH,MH2,MK,MK2,ML,ML2,KDENS,KZ,KQT(50)
      COMMON QCAL(50,20),AA,BB,CC,BETA,AR,BR,CR,DR,ER,FR
      COMMON /BAF/NT,IRJ(50,8),IH(50,50,8),IK(50,50,8),IL(50,100,8)
     1           ,NPOSS,FWOLFF
      COMMON /JCOUNT/JCOUNT,EPST,NINI,FOM
      COMMON /REC/REC(6),PAR(6)
      DATA IR,IW/15,16/
      NCAF=1
      DO 937 I=1,6
  937 DIR(I)=0.
      JCOUNT=JCOUNT+1
      IF(JCOUNT.GT.30)THEN
C     WRITE(*,1055)N,FOM
      WRITE(IW,1055)N,FOM
 1055 FORMAT(10X,'W A R N I N G   :  THE NUMBER OF SOLUTIONS FOUND, WITH
     + M(',I2,') .GT.',F5.2,', IS GREATER THAN 30.'/29X,'THE CALCULATION
     + HAS BEEN STOPPED.'/29X,'PLEASE, CHECK YOUR INPUT DATA !')
      STOP
      ENDIF
      DO 710 I=1,3
      ERROR(I)=0.0001
      ERROR(I+3)=.001
  710 CONTINUE
      IF(IND.NE.6)THEN
      DO 610 I=1,N
      IRJ(I,8)=IRJ(I,7)
      DO 610 J=1,IRJ(I,8)
      IH(I,J,8)=IH(I,J,7)
      IK(I,J,8)=IK(I,J,7)
      IL(I,J,8)=IL(I,J,7)
  610 CONTINUE
      ENDIF
      BIDON1=1.001
      BIDON2=1000./1001.
      BEBE=PIRAD*BETA
      IF(IND.EQ.5)AA=AA/SIN(BEBE)
      IF(IND.EQ.5)CC=CC/SIN(BEBE)
      PAR(1)=AA
      PAR(2)=BB
      PAR(3)=CC
      PAR(4)=BETA
      IF(NRIND.EQ.2)PAR(2)=CC
      IF(IND.EQ.6)THEN
      PAR(1)=AR
      PAR(2)=BR
      PAR(3)=CR
      PAR(4)=DR
      PAR(5)=ER
      PAR(6)=FR
      ENDIF
C     S E L E C T I O N   D E S   R A I E S   U N I V O Q U E S
 1623 CONTINUE
      JAF=0
      NRAIES=0
      DO 10 I=1,N
      IF(IRJ(I,8).GT.1)GO TO 10
      NRAIES=NRAIES+1
      JH(NRAIES)=IH(I,1,8)
      JK(NRAIES)=IK(I,1,8)
      JL(NRAIES)=IL(I,1,8)
      POIDS(NRAIES)=1./EPSQ(I)
      SSQ(NRAIES)=Q(I)
   10 CONTINUE
      DO 6101 I=1,NRAIES
 6101 IF(JH(I).NE.0)GO TO 6102
      FWOLFF=-1000.
      JCOUNT=JCOUNT-1
      GO TO 581
6102  CONTINUE
      DO 6103 I=1,NRAIES
 6103 IF(JK(I).NE.0)GO TO 6104
      FWOLFF=-1000.
      JCOUNT=JCOUNT-1
      GO TO 581
6104  CONTINUE
      DO 6105 I=1,NRAIES
 6105 IF(JL(I).NE.0)GO TO 6106
      FWOLFF=-1000.
      JCOUNT=JCOUNT-1
      GO TO 581
6106  CONTINUE
      RINDEP=NRAIES-NRIND
      IF (RINDEP.LE.0.0) GOTO 565
C     CAMBIO SEGUN HEXAGONAL
C*****-----------------------------------------------
C     TEST N. 1 : NOMBRE DE RAIES (PROVISOIRE)
C SUPPRIMER ?    30 IF (NRAIES.LT.NRIND) GOTO 565
      LLAVE=1
C      A F F I N E M E N T
      CALL PASAJE(IND)
      DO 210 I=1,NRIND
      PAR(I)=REC(I)
  210 CONTINUE
      CALL PASAJE(IND)
      DO 215 I=1,NRIND
      DIR(I)=REC(I)
  215 CONTINUE
      ITERAT=0
  600 ITERAT=ITERAT+1
      DO 220 J=1,NRIND+1
      DO 225 K=1,NRIND+1
      A(J,K)=0.
  225 CONTINUE
  220 CONTINUE
      DO 230 I=1,NRAIES
      PARH=2.*PAR(1)*JH(I)**2
      PARK=2.*PAR(2)*JK(I)**2
      PARL=2.*PAR(3)*JL(I)**2
      PARKL=2.*PAR(2)*JL(I)**2
      PARHK=2.*PAR(1)*JK(I)**2
      GO TO(241,242,243,244,245,246),IND
 241  E(1)=(PARH+PARHK+2.*PAR(1)*JL(I)**2)*POIDS(I)
      SSC(I)=PAR(1)**2*(JH(I)**2+JK(I)**2+JL(I)**2)
      GO TO 247
 242  E(1)=(PARH+PARHK)*POIDS(I)
      E(2)=PARKL*POIDS(I)
      SSC(I)=PAR(1)**2*(JH(I)**2+JK(I)**2)+PAR(2)**2*JL(I)**2
      GO TO 247
 243  E(1)=(PARH+PARHK+2.*PAR(1)*JH(I)*JK(I))*POIDS(I)
      E(2)=PARKL*POIDS(I)
      SSC(I)=PAR(1)**2*(JH(I)**2+JK(I)**2+JH(I)*JK(I))+(PAR(2)*JL(I))**2
      GO TO 247
 244  E(1)=PARH*POIDS(I)
      E(2)=PARK*POIDS(I)
      E(3)=PARL*POIDS(I)
      SSC(I)=PAR(1)**2*JH(I)**2+PAR(2)**2*JK(I)**2+PAR(3)**2*JL(I)**2
      GO TO 247
 245  COSB=2.*JH(I)*JL(I)*COS(PAR(4)*PIRAD)
      E(1)=(PARH+PAR(3)*COSB)*POIDS(I)
      E(2)=PARK*POIDS(I)
      E(3)=(PARL+PAR(1)*COSB)*POIDS(I)
      E(4)=(-2.*JH(I)*JL(I)*SIN(PAR(4)*PIRAD)*PAR(1)*PAR(3))*POIDS(I)
     +*PIRAD
      SSC(I)=(PAR(1)*JH(I))**2+(PAR(2)*JK(I))**2+(PAR(3)*JL(I))**2
     ++COSB*PAR(1)*PAR(3)
      GO TO 247
 246  COSA=2.*JK(I)*JL(I)*COS(PAR(4)*PIRAD)
      COSB=2.*JH(I)*JL(I)*COS(PAR(5)*PIRAD)
      COSG=2.*JH(I)*JK(I)*COS(PAR(6)*PIRAD)
      E(1)=(PARH+COSG*PAR(2)+COSB*PAR(3))*POIDS(I)
      E(2)=(PARK+COSG*PAR(1)+COSA*PAR(3))*POIDS(I)
      E(3)=(PARL+COSA*PAR(2)+COSB*PAR(1))*POIDS(I)
      E(4)=(-2.*JK(I)*JL(I)*PAR(2)*PAR(3)*SIN(PAR(4)*PIRAD))*POIDS(I)
     +*PIRAD
      E(5)=(-2.*JH(I)*JL(I)*PAR(1)*PAR(3)*SIN(PAR(5)*PIRAD))*POIDS(I)
     +*PIRAD
      E(6)=(-2.*JH(I)*JK(I)*PAR(1)*PAR(2)*SIN(PAR(6)*PIRAD))*POIDS(I)
     +*PIRAD
      SSC(I)=(PAR(1)*JH(I))**2+(PAR(2)*JK(I))**2+(PAR(3)*JL(I))**2
     ++COSA*PAR(2)*PAR(3)+COSB*PAR(1)*PAR(3)+COSG*PAR(1)*PAR(2)
 247  E(NRIND+1)=(SSQ(I)-SSC(I))*POIDS(I)
      DO 260 J=1,NRIND+1
      DO 265 K=J,NRIND+1
      A(J,K)=A(J,K)+E(J)*E(K)
  265 CONTINUE
  260 CONTINUE
  230 CONTINUE
      IF(NRIND.EQ.1)THEN
      ECART(1)=A(1,2)/A(1,1)
      A(1,1)=1./A(1,1)
      GO TO 3655
      ENDIF
  290 DO 295 I=2,NRIND
      L=I-1
      DO 300 J=1,L
      A(I,J)=0.
  300 CONTINUE
  295 CONTINUE
      A(1,1)=1./SQRT(A(1,1))
      DO 305 I=2,NRIND
      A(1,I)=A(1,I)*A(1,1)
  305 CONTINUE
      DO 310 I=2,NRIND
      L=I-1
      DO 315 J=1,L
      A(I,I)=A(I,I)-A(J,I)*A(J,I)
      IF(A(I,I).LE.0.)THEN
C     WRITE(IW,*)' CAS SPECIAL '
C     WRITE(*,*)' CAS SPECIAL '
      FWOLFF=-1000.
      JCOUNT=JCOUNT-1
      GO TO 581
      ENDIF
  315 CONTINUE
      RAIZDA=SQRT(A(I,I))
      A(I,I)=1./RAIZDA
      LL=I+1
      DO 320 J=1,L
      DO 325 K=J,L
      A(I,J)=A(I,J)-A(K,I)*A(K,J)
  325 CONTINUE
      A(I,J)=A(I,J)*A(I,I)
  320 CONTINUE
      IF(I.EQ.NRIND)GOTO 310
      DO 330 J=LL,NRIND
      DO 335 K=1,L
      A(I,J)=A(I,J)-A(K,I)*A(K,J)
  335 CONTINUE
      A(I,J)=A(I,J)*A(I,I)
  330 CONTINUE
  310 CONTINUE
      DO 337 J=1,NRIND
      A(NRIND,J)=A(NRIND,J)*A(NRIND,NRIND)
  337 CONTINUE
      DO 340 I=2,NRIND
      IS=NRIND+1-I
      DIAG=A(IS,IS)
      DO 345 J=1,IS
      L=I-1
      DO 350 K=1,L
      IM=NRIND+1-K
      A(IS,J)=A(IS,J)-A(IS,IM)*A(IM,J)
  350 CONTINUE
      A(IS,J)=A(IS,J)*DIAG
  345 CONTINUE
  340 CONTINUE
      DO 355 I=2,NRIND
      L=I-1
      DO 360 J=1,L
      A(J,I)=A(I,J)
  360 CONTINUE
  355 CONTINUE
      DO 365 I=1,NRIND
      ECART(I)=0.
      DO 370 J=1,NRIND
      ECART(I)=ECART(I)+A(I,J)*A(J,NRIND+1)
  370 CONTINUE
  365 CONTINUE
 3655 CONTINUE
  390 DO 395 I=1,NRIND
      PAR(I)=PAR(I)+ECART(I)
  395 CONTINUE
      CALL PASAJE(IND)
      DO 400 J=1,NRIND
      ECART(J)=REC(J)-DIR(J)
      DIR(J)=REC(J)
  400 CONTINUE
      DO 410 J=1,NRIND
      IF(ABS(ECART(J)).GT.ERROR(J))GO TO 600
  410 CONTINUE
      INDI=3
      IF(NRIND.LT.3)INDI=NRIND
      DO 415 I=1,INDI
      PAR(I)=PAR(I)*BIDON1
      CALL PASAJE(IND)
      PAR(I)=PAR(I)*BIDON2
      DO 420 J=1,NRIND
      B(I,J)=(REC(J)-DIR(J))*1000./PAR(I)
  420 CONTINUE
  415 CONTINUE
      IF(NRIND.LT.4)GO TO 422
      DO 421 I=4,NRIND
      PAR(I)=PAR(I)+.01
      CALL PASAJE(IND)
      PAR(I)=PAR(I)-.01
      DO 425 J=1,NRIND
      B(I,J)=(REC(J)-DIR(J))*100.
  425 CONTINUE
  421 CONTINUE
  422 CONTINUE
  450 RINDEP=A(NRIND+1,NRIND+1)/RINDEP
      DO 480 I=1,NRIND
      ECART(I)=0.
      DO 485 J=1,NRIND
      TEM(J)=0.
      DO 490 K=1,NRIND
      TEM(J)=TEM(J)+A(J,K)*B(K,I)
  490 CONTINUE
      ECART(I)=ECART(I)+TEM(J)*B(J,I)
  485 CONTINUE
      ECART(I)=SQRT(ECART(I)*RINDEP)
  480 CONTINUE
      AE=PAR(1)
      BE=PAR(2)
      CE=PAR(3)
      AE2=AE*AE
      BE2=BE*BE
      CE2=CE*CE
      NPOSS=-1
      QNC=Q(N)+EPSQ(N)
      GO TO(1538,1538,1538,1538,1530,1537),IND
 1530 COSB=COS(PIRAD*PAR(4))
      GO TO 1538
 1537 COSA=COS(PIRAD*PAR(4))
      COSB=COS(PIRAD*PAR(5))
      COSG=COS(PIRAD*PAR(6))
 1538 CONTINUE
      M1=0
      K1=0
      L1=0
      K2=MK2
      L2=ML2
      IF(N.EQ.NINI)GO TO 751
      DO 750 IR=N+1,NINI
  750 IRJ(IR,8)=0
  751 IF(IND.EQ.1)NPOSS=0
      IF(IND.EQ.1)M1=1
      DO 1080 M=M1,MH2
      IF(IND.EQ.6.AND.M.NE.0)K1=-MK2
      IF(IND.LT.4)K2=M
      IF(IND.EQ.5.AND.M.NE.0)L1=-ML2
      DO 1090 K=K1,K2
      IF(IND.EQ.1)L2=K
      IF(IND.EQ.6.AND.(M.NE.0.OR.K.NE.0))L1=-ML2
      DO 1100 MXL=L1,L2
      GO TO(1528,1529,1531,1532,1533,1534),IND
 1528 CALQ=(M*M+K*K+MXL*MXL)*AE2
      GO TO 1536
 1529 CALQ=(M*M+K*K)*AE2+MXL*MXL*BE2
      GO TO 1536
 1531 CALQ=(M*M+K*K+M*K)*AE2+MXL*MXL*BE2
      GO TO 1536
 1532 CALQ=M*M*AE2+K*K*BE2+MXL*MXL*CE2
      GO TO 1536
 1533 CALQ=M*M*AE2+K*K*BE2+MXL*MXL*CE2+2.*M*MXL*AE*CE*COSB
      GO TO 1536
 1534 CALQ=M*M*AE2+K*K*BE2+MXL*MXL*CE2+2.*M*K*AE*BE*COSG+2.*K*MXL*BE*CE*
     +COSA+2.*M*MXL*AE*CE*COSB
 1536 IF(CALQ.GT.QNC)GO TO 1100
      NPOSS=NPOSS+1
      IF(M.EQ.0.AND.K.EQ.0.AND.MXL.EQ.0)GO TO 1100
      IF(NINI.EQ.N)GO TO 1627
      DO 1624 IS=NINI,N+1,-1
      IF(IRJ(IS,8).EQ.0)GO TO 1628
      DO 1626 JS=1,IRJ(IS,8)
      IF(M.EQ.IH(IS,JS,8).AND.K.EQ.IK(IS,JS,8).AND.MXL.EQ.IL(IS,JS,8))
     SGO TO 1624
 1626 CONTINUE
 1628 CONTINUE
      XT1=2.*PIDEG*ASIN(WAVE2*SQRT(CALQ))
      XT2=2.*PIDEG*ASIN(WAVE2*SQRT(Q(IS)))
      IF(ABS(XT1-XT2).GT.EPST)GO TO 1624
      IRJ(IS,8)=IRJ(IS,8)+1
      JJS=IRJ(IS,8)
      IH(IS,JJS,8)=M
      IK(IS,JJS,8)=K
      IL(IS,JJS,8)=MXL
 1624 CONTINUE
 1627 CONTINUE
      DO 1620 I=1,N
      XT1=2.*PIDEG*ASIN(WAVE2*SQRT(CALQ))
      XT2=2.*PIDEG*ASIN(WAVE2*SQRT(Q(I)))
      IF(ABS(XT1-XT2).GT.EPST)GO TO 1620
      MRJ=IRJ(I,8)
      DO 1619 J=1,IRJ(I,8)
      IF(M.EQ.IH(I,J,8).AND.K.EQ.IK(I,J,8).AND.MXL.EQ.IL(I,J,8))GO TO
     +1620
 1619 CONTINUE
      DO 1621 J=1,IRJ(I,8)
      GO TO(928,929,931,932,933,934),IND
  928 QNAF=(IH(I,J,8)**2+IK(I,J,8)**2+IL(I,J,8)**2)*AE2
      GO TO 936
  929 QNAF=(IH(I,J,8)**2+IK(I,J,8)**2)*AE2+IL(I,J,8)**2*BE2
      GO TO 936
  931 QNAF=(IH(I,J,8)**2+IK(I,J,8)**2+IH(I,J,8)*IK(I,J,8))*AE2+IL(I,J,8)
     +**2*BE2
      GO TO 936
  932 QNAF=IH(I,J,8)**2*AE2+IK(I,J,8)**2*BE2+IL(I,J,8)**2*CE2
      GO TO 936
  933 QNAF=IH(I,J,8)**2*AE2+IK(I,J,8)**2*BE2+IL(I,J,8)**2*CE2+2.*
     +IH(I,J,8)*IL(I,J,8)*AE*CE*COSB
      GO TO 936
  934 QNAF=IH(I,J,8)**2*AE2+IK(I,J,8)**2*BE2+IL(I,J,8)**2*CE2+2.*IH(I,
     +J,8)*IK(I,J,8)*AE*BE*COSG+2.*IK(I,J,8)*IL(I,J,8)*BE*CE*COSA+2.*
     +IH(I,J,8)*IL(I,J,8)*AE*CE*COSB
  936 IF(ABS(CALQ-Q(I)).LE.ABS(QNAF-Q(I)))THEN
      MRJ=MRJ+1
      IH(I,MRJ,8)=M
      IK(I,MRJ,8)=K
      IL(I,MRJ,8)=MXL
      JAF=1
      GO TO 1618
      ENDIF
 1621 CONTINUE
 1618 IRJ(I,8)=MRJ
 1620 CONTINUE
 1100 CONTINUE
 1090 CONTINUE
 1080 CONTINUE
      IF(JAF.EQ.1)NCAF=NCAF+1
      IF(JAF.EQ.1.AND.NCAF.LT.4)THEN
      DO 1622 IAF=1,6
 1622 PAR(IAF)=DIR(IAF)
      GO TO 1623
      ENDIF
      DTHETA=0.
      DELTAQ=0.
      DO 825 I=1,N
      JJ=IRJ(I,8)
      DIFQ=100.
      J=1
  855 PH=IH(I,J,8)
      PK=IK(I,J,8)
      PL=IL(I,J,8)
      PH2=PH*PH
      PK2=PK*PK
      PL2=PL*PL
      GO TO(828,829,831,832,833,834),IND
  828 CALQ=(PH2+PK2+PL2)*AE2
      GO TO 836
  829 CALQ=(PH2+PK2)*AE2+PL2*BE2
      GO TO 836
  831 CALQ=(PH2+PK2+PH*PK)*AE2+PL2*BE2
      GO TO 836
  832 CALQ=PH2*AE2+PK2*BE2+PL2*CE2
      GO TO 836
  833 CALQ=PH2*AE2+PK2*BE2+PL2*CE2+2.*PH*PL*AE*CE*COSB
      GO TO 836
  834 CALQ=PH2*AE2+PK2*BE2+PL2*CE2+2.*PH*PK*AE*BE*COSG+2.*PK*PL*BE*CE*
     +COSA+2.*PH*PL*AE*CE*COSB
  836 DIFQ1=ABS(Q(I)-CALQ)
      DCAL=SQRT(CALQ)
      TCAL=2.*PIDEG*ASIN(WAVE2*DCAL)
      DCAL=1./DCAL
      DIFD=D(I)-DCAL
      DIFT=TH(I)-TCAL
      IF(ABS(DIFT).LE.EPST)GO TO 845
      IF(JJ.EQ.J)THEN
      JJ=JJ-1
      GO TO 838
      ELSE
      IH(I,J,8)=IH(I,JJ,8)
      IK(I,J,8)=IK(I,JJ,8)
      IL(I,J,8)=IL(I,JJ,8)
      JJ=JJ-1
      GO TO 855
      ENDIF
  845 J=J+1
      IF(J.LE.JJ)GO TO 855
  838 IRJ(I,8)=JJ
      IF(JJ.GT.0)GO TO 825
      FWOLFF=-1000.
      JCOUNT=JCOUNT-1
      GO TO 581
  825 CONTINUE
C
C     I M P R E S S I O N   D E S   R E S U L T A T S
C
      KTESTWOLFF=1
      KCLEF=1
  570 CONTINUE
  523 CONTINUE
      IF(KCLEF.EQ.1)GO TO 1738
      AE=PAR(1)
      BE=PAR(2)
      CE=PAR(3)
      AE2=AE*AE
      BE2=BE*BE
      CE2=CE*CE
C
      GO TO(1738,1738,1738,1738,1730,1737),IND
 1730 COSB=COS(PIRAD*PAR(4))
      GO TO 1738
 1737 COSA=COS(PIRAD*PAR(4))
      COSB=COS(PIRAD*PAR(5))
      COSG=COS(PIRAD*PAR(6))
 1738 CONTINUE
C
      GOTO(571,572,573,574,575,552),IND
  571 VOL=DIR(1)**3
      DELTAV=3.*ECART(1)/DIR(1)*VOL
C     IF(KTESTWOLFF.EQ.0)WRITE(IW,1)DIR(1),VOL
C     IF(KTESTWOLFF.EQ.0.AND.KCLEF.EQ.1)WRITE(IW,142)ECART(1),DELTAV
      IF(KTESTWOLFF.EQ.0)WRITE(IW,1)DIR(1),VOL
      IF(KTESTWOLFF.EQ.0.AND.KCLEF.EQ.1)WRITE(IW,142)ECART(1)
      GOTO 576
  572 VOL=DIR(1)**2*DIR(2)
      DELTAV=VOL*(2.*ECART(1)/DIR(1)+ECART(2)/DIR(2))
C     IF(KTESTWOLFF.EQ.0)WRITE(IW,2)DIR(1),DIR(2),VOL
C     IF(KTESTWOLFF.EQ.0.AND.KCLEF.EQ.1)WRITE(IW,249)ECART(1),ECART(2),
C    SDELTAV
      IF(KTESTWOLFF.EQ.0)WRITE(IW,2)DIR(1),DIR(2),VOL
      IF(KTESTWOLFF.EQ.0.AND.KCLEF.EQ.1)WRITE(IW,249)ECART(1),ECART(2)
      GOTO 576
  573 VOL=DIR(1)**2*DIR(2)*SQRT(3.)/2.
      DELTAV=VOL*(2.*ECART(1)/DIR(1)+ECART(2)/DIR(2))
C     IF(KTESTWOLFF.EQ.0)WRITE(IW,3)DIR(1),DIR(2),VOL
C     IF(KTESTWOLFF.EQ.0.AND.KCLEF.EQ.1)WRITE(IW,342)ECART(1),ECART(2),
C    SDELTAV
      IF(KTESTWOLFF.EQ.0)WRITE(IW,3)DIR(1),DIR(2),VOL
      IF(KTESTWOLFF.EQ.0.AND.KCLEF.EQ.1)WRITE(IW,342)ECART(1),ECART(2)
      GOTO 576
  574 VOL=DIR(1)*DIR(2)*DIR(3)
      DELTAV=(ECART(1)/DIR(1)+ECART(2)/DIR(2)+ECART(3)/DIR(3))*VOL
C     IF(KTESTWOLFF.EQ.0)WRITE(IW,4)(DIR(I),I=1,3),VOL
C     IF(KTESTWOLFF.EQ.0.AND.KCLEF.EQ.1)WRITE(IW,442)(ECART(I),I=1,3),
C    SDELTAV
      IF(KTESTWOLFF.EQ.0)WRITE(IW,4)(DIR(I),I=1,3),VOL
      IF(KTESTWOLFF.EQ.0.AND.KCLEF.EQ.1)WRITE(IW,442)(ECART(I),I=1,3)
      GOTO 576
  575 VOL=DIR(1)*DIR(2)*DIR(3)*SIN(DIR(4)*PIRAD)
      DELTAV=(ECART(1)/DIR(1)+ECART(2)/DIR(2)+ECART(3)/DIR(3)
     S+ECART(4)*PIRAD/ABS(TAN(DIR(4)*PIRAD)))*VOL
C     IF(KTESTWOLFF.EQ.0)WRITE(IW,5)(DIR(I),I=1,4),VOL
C     IF(KTESTWOLFF.EQ.0.AND.KCLEF.EQ.1)WRITE(IW,547)(ECART(I),I=1,4),
C    SDELTAV
      IF(KTESTWOLFF.EQ.0)WRITE(IW,5)(DIR(I),I=1,4),VOL
      IF(KTESTWOLFF.EQ.0.AND.KCLEF.EQ.1)WRITE(IW,547)(ECART(I),I=1,4)
      GOTO 576
  552 COA=COS(DIR(4)*PIRAD)
      COB=COS(DIR(5)*PIRAD)
      COG=COS(DIR(6)*PIRAD)
      SIA=SIN(DIR(4)*PIRAD)
      SIB=SIN(DIR(5)*PIRAD)
      SIG=SIN(DIR(6)*PIRAD)
      SVOL=SQRT(1.-COA*COA-COB*COB-COG*COG+2.*COA*COB*COG)
      VOL=DIR(1)*DIR(2)*DIR(3)*SVOL
      DETAV=ECART(1)/DIR(1)+ECART(2)/DIR(2)+ECART(3)/DIR(3)
      DETAV1=PIRAD*(ABS(SIA*COA-SIA*COB*COG)*ECART(4)+ABS(SIB*COB-SIB*
     +COA*COG)*ECART(5)+ABS(SIG*COG-SIG*COA*COB)*ECART(6))/SVOL/SVOL
      DELTAV=VOL*(DETAV+DETAV1)
C     IF(KTESTWOLFF.EQ.0)WRITE(IW,1011)(DIR(I),I=1,6),VOL
C     IF(KTESTWOLFF.EQ.0.AND.KCLEF.EQ.1)WRITE(IW,10142)(ECART(I),I=1,6),
C    SDELTAV
      IF(KTESTWOLFF.EQ.0)WRITE(IW,1011)(DIR(I),I=1,6),VOL
      IF(KTESTWOLFF.EQ.0.AND.KCLEF.EQ.1)WRITE(IW,10142)(ECART(I),I=1,6)
  576 IF(KTESTWOLFF.EQ.1)GO TO 522
  579 IF(KDENS.EQ.1) GOTO 622
      WRITE(IW,7)KZ
  622 WRITE(IW,1004)
  522 CONTINUE
      DTHETA=0.
      DELTAQ=0.
      DO 525 I1=1,NINI
      I=I1+N-NINI
      IF(I1.GT.1.AND.I1.LT.(NINI-N+2))I=NINI-I1+2
      IF(I1.EQ.1)I=1
      DIFQ=100.
      KEY=1
      DO 545 J=1,IRJ(I,8)
      PH=IH(I,J,8)
      PK=IK(I,J,8)
      PL=IL(I,J,8)
      PH2=PH*PH
      PK2=PK*PK
      PL2=PL*PL
      GO TO(528,529,531,532,533,534),IND
  528 CALQ=(PH2+PK2+PL2)*AE2
      GO TO 536
  529 CALQ=(PH2+PK2)*AE2+PL2*BE2
      GO TO 536
  531 CALQ=(PH2+PK2+PH*PK)*AE2+PL2*BE2
      GO TO 536
  532 CALQ=PH2*AE2+PK2*BE2+PL2*CE2
      GO TO 536
  533 CALQ=PH2*AE2+PK2*BE2+PL2*CE2+2.*PH*PL*AE*CE*COSB
      GO TO 536
  534 CALQ=PH2*AE2+PK2*BE2+PL2*CE2+2.*PH*PK*AE*BE*COSG+2.*PK*PL*BE*CE*
     +COSA+2.*PH*PL*AE*CE*COSB
  536 DIFQ1=ABS(Q(I)-CALQ)
      DCAL=SQRT(CALQ)
      TCAL=2.*PIDEG*ASIN(WAVE2*DCAL)
      DCAL=1./DCAL
      DIFD=D(I)-DCAL
      DIFT=TH(I)-TCAL
      IF(KTESTWOLFF.EQ.1)GO TO 540
      GOTO (530,535),KEY
  535 WRITE(IW,1005)IH(I,J,8),IK(I,J,8),IL(I,J,8),DCAL,DIFD,CALQ,
     STCAL,DIFT
      GOTO 540
  530 WRITE(IW,1006)IH(I,J,8),IK(I,J,8),IL(I,J,8),D(I),DCAL,
     SDIFD,Q(I),CALQ,TH(I),TCAL,DIFT
  540 IF(DIFQ1.GE.DIFQ)GO TO 545
      DIFQ=DIFQ1
      DTET=ABS(DIFT)
      KEY=2
  545 CONTINUE
  550 DELTAQ=DIFQ+DELTAQ
      DTHETA=DTET+DTHETA
  525 CONTINUE
C     F I G U R E S   D E   M E R I T E
      DELTAQ=DELTAQ/NINI
      DTHETA=DTHETA/NINI
      FWOLFF=Q(N)/(2.*NPOSS*DELTAQ)
      IF(KTESTWOLFF.EQ.0)GO TO 542
      IF(FWOLFF.GT.FOM)THEN
      KTESTWOLFF=0
      GO TO 523
      ELSE
      FWOLFF=-1000.
      JCOUNT=JCOUNT-1
      GO TO 581
      ENDIF
  542 FINDEX=NINI/(NPOSS*DTHETA)
      FVAR=100.*(1.-(NPOSS*DTHETA*DTHETA/(2.*NINI*DTH*DTH)))
      IF(FVAR.LE.0.1)FVAR=0.0
      WRITE(IW,1007)NINI,NPOSS
      WRITE(IW,1008)DELTAQ,DTHETA,EPST
      WRITE(IW,1009)NINI,FWOLFF,NINI,FINDEX,DTHETA,NPOSS
      GOTO (560,580),LLAVE
CTEST N. 2 : FIGURES DE MERITE (A ENLEVER ?)
  560 R=2.*DTH*NPOSS
C                  DTH= ERREUR SUR 2THETA
      FWTEST=TAN(TH(N)/2.)/R
      FITEST=2.*NINI/R
      IF(FWTEST.LE.FWOLFF.AND.FITEST.LE.FINDEX)GO TO 580
      WRITE(IW,1043)
      GO TO 56542
  565 WRITE(IW,1010)
56542 LLAVE=2
      KCLEF=2
      DIR(1)=AA
      DIR(2)=BB
      DIR(3)=CC
      DIR(4)=BETA
      IF(NRIND.EQ.2)DIR(2)=CC
      IF(IND.EQ.6)THEN
      DIR(1)=AR
      DIR(2)=BR
      DIR(3)=CR
      DIR(4)=DR
      DIR(5)=ER
      DIR(6)=FR
      ENDIF
      VOL=VAP
      DO 2200 I=1,NRIND
 2200 PAR(I)=DIR(I)
      CALL PASAJE(IND)
      DO 2201 I=1,NRIND
 2201 PAR(I)=REC(I)
      KTESTWOLFF=0
      GO TO 570
C
    1 FORMAT(//20X,' C U B I C    S Y S T E M '
     S     //6X,'DIRECT PARAMETERS :',15X,'A=',F9.5,9X,'VOLUME=',F9.3)
  142 FORMAT(6X,'STANDARD DEVIATIONS :',15X,F9.5,16X,F9.3)
    2 FORMAT(//20X,'T E T R A G O N A L   S Y S T E M  '
     S       //6X,'DIRECT PARAMETERS :',15X,'A=',F9.5,5X,'C=',F9.5,9X,
     S       'VOLUME=',F9.3)
  249 FORMAT(6X,'STANDARD DEVIATIONS :',15X,F9.5,7X,F9.5,16X,F9.3)
    3 FORMAT(//20X,'H E X A G O N A L    S Y S T E M '
     S       //6X,'DIRECT PARAMETERS :',15X,'A=',F9.5,5X,'C=',F9.5,9X,
     S       'VOLUME=',F9.3)
  342 FORMAT(6X,'STANDARD DEVIATIONS :',15X,F9.5,7X,F9.5,16X,F9.3)
    4 FORMAT(//20X,'O R T H O R H O M B I C    S Y S T E M '
     S       //6X,'DIRECT PARAMETERS :',15X,'A=',F9.5,5X,'B=',F9.5,5X,
     S       'C=',F9.5,9X,'VOLUME=',F9.3)
  442 FORMAT(6X,'STANDARD DEVIATIONS :',15X,3(F9.5,7X),9X,F9.3)
    5 FORMAT(//20X,'M O N O C L I N I C    S Y S T E M '
     S       //6X,'DIRECT PARAMETERS :',15X,'A=',F9.5,5X,'B=',F9.5,5X,
     S           'C=',F9.5,5X,'BETA= ',F7.3,7X,'VOLUME=',F9.3)
  547 FORMAT(6X,'STANDARD DEVIATIONS :',15X,3(F9.5,7X),4X,F7.3,14X,
     SF9.3/)
 1011 FORMAT(//20X,'T R I C L I N I C    S Y S T E M '//6X,'DIRECT PARAM
     SETERS :',5X,'A=',F9.5,4X,'B=',F9.5,4X,'C=',F9.5,'   ALP=',
     SF7.3,4X,'BET=',F7.3,4X,'GAM=',F7.3,4X,'VOL=',F9.3)
10142 FORMAT(6X,'STANDARD DEVIATIONS :',5X,F9.5,6X,F9.5,6X,F9.5,7X,F9.5
     S,6X,F9.5,6X,F9.5,9X,F6.3/)
C   6 FORMAT(6X,'STANDARD DEVIATIONS :',15X,3(F9.5,7X),4X,F7.3,17X,
C    SF6.3/)
C 606 FORMAT(6X,'STANDARD DEVIATIONS :',3X,F9.5,6X,F9.5,6X,F9.5,7X,F9.5
C    +,6X,F9.5,6X,F9.5,9X,F6.3/)
    7 FORMAT(/6X,'NUMBER OF MOLECULES IN THE UNIT CELL :  ',7X,'Z=',I3/)
 1001 FORMAT(35X,3(F9.5,5X),F7.3,10X,F9.3,5X,'Z=',I2)
 1002 FORMAT(35X,3(F9.5,5X),F7.3,10X,F9.3)
 1003 FORMAT(10X,'STANDARD DEVIATIONS :',4X,3(F9.5,5X),F7.3)
 1004 FORMAT(/6X,'H',3X,'K',3X,'L',10X,'DOBS',6X,'DCAL',4X,'DOBS-DCAL'
     S       ,6X,'QOBS',6X,'QCAL',6X,'2TH.OBS',3X,
     S       '2TH.CAL',3X,'DIF.2TH.'/)
 1005 FORMAT(3X,3I4,15X,2(F10.5),13X,F10.5,12X,2(F9.3))
 1006 FORMAT(3X,3I4,5X,3(F10.5),3X,2(F10.5),3X,3(F9.3))
 1007 FORMAT(//10X,'* NUMBER OF LINES'/15X,'.- INPUT DATA =',I4/
     S       15X,'.- CALCULATED =',I4)
 1008 FORMAT(//10X,'* MEAN ABSOLUTE DISCREPANCIES'/
     S       46X,'<Q> =',E10.4/33X,'<DELTA(2-THETA)> =',E10.4/
     S       15X,'MAX. ERROR ACCEPTED (DEG. 2-THETA) =',E10.4)
 1009 FORMAT(//10X,'* FIGURES OF MERIT'/17X,'1.- M(',I3,') =',F7.1,
     S       25X,'(REF. 4)'/17X,'2.- F(',I3,') =',F7.1,'(',F6.4,',',I4,
     S       ')',12X,'(REF. 5)'/)
 1010 FORMAT(/10X, 'WARNING ! THE FOLLOWING SOLUTION IS NOT REFINED !'
     S///)
 1043 FORMAT(/10X,'WARNING ! THE NEXT SOLUTION IS SAME THAN THE LATER SO
     SLUTION BUT IT IS NOT REFINED.'///)
C
  580 V=VOL
  581 CONTINUE
      RETURN
      END
      SUBROUTINE PASAJE(IND)
      COMMON /REC/REC(6),PAR(6)
      PI=4.*ATAN(1.)
      PIRAD=PI/180.
      PIDEG=180./PI
      REC(1)=1./PAR(1)
      GO TO(70,20,30,40,50,60),IND
  20  REC(2)=1./PAR(2)
      GO TO 70
  30  REC(1)=2./PAR(1)/SQRT(3.)
      REC(2)=1./PAR(2)
      GO TO 70
  40  REC(2)=1./PAR(2)
      REC(3)=1./PAR(3)
      GO TO 70
  50  REC(1)=REC(1)/SIN(PAR(4)*PIRAD)
      REC(2)=1./PAR(2)
      REC(3)=1./PAR(3)/SIN(PAR(4)*PIRAD)
      REC(4)=180.-PAR(4)
      GO TO 70
  60  COSA=COS(PAR(4)*PIRAD)
      COSB=COS(PAR(5)*PIRAD)
      COSG=COS(PAR(6)*PIRAD)
      SINA=SIN(PAR(4)*PIRAD)
      SINB=SIN(PAR(5)*PIRAD)
      SING=SIN(PAR(6)*PIRAD)
      SVOL=SQRT(1.-COSA*COSA-COSB*COSB-COSG*COSG+2.*COSA*COSB*COSG)
      VOL=1./SVOL/PAR(1)/PAR(2)/PAR(3)
      REC(1)=PAR(2)*PAR(3)*SINA*VOL
      REC(2)=PAR(1)*PAR(3)*SINB*VOL
      REC(3)=PAR(1)*PAR(2)*SING*VOL
      REC(4)=PIDEG*ACOS((COSB*COSG-COSA)/SINB/SING)
      REC(5)=PIDEG*ACOS((COSA*COSG-COSB)/SINA/SING)
      REC(6)=PIDEG*ACOS((COSA*COSB-COSG)/SINA/SINB)
  70  RETURN
      END
      subroutine open_read1(unit,file)
      integer unit
      character*(*) file
      open (unit,file=file,status='old')
      return
      end
      subroutine open_write1(unit,file)
      integer unit
      character*(*) file
      open (unit,file=file,status='new')
      return
      end
      subroutine filedel(unit,file)
      integer unit
      character*(*) file
      open (unit,file=file,status='old')
      close (unit,status='delete')
      return
      end
